"""
This does the full thing:  it views from the retina, through V1,
to pyramid detection, then sets a fact on.  It also has the planning in.
It parses, and sets goals.
It does cognitive mapping.
So, it does the commands that are activated. 
These things are not all hooked in appropriately.
I haven't done a good range of error checking for planning or pictures.

Set the simulator in nealParams.py.
It only works with the environment running.
"""

import numpy
from PIL import Image

import nealParams as nealParameters

#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
    import spynnaker_external_devices_plugin.pyNN as externaldevices
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle
    import os.path


from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from timerClass import TimerClass

from parseClass import ParseClass
from planClass import PlanClass
from visionClass import visionArea
from cogmapClass import cogmapArea

numTimerSteps = 8

#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createBumpCells():
    fsa = FSAHelperFunctions(nealParameters.simulator)
    neal = NealCoverFunctions(nealParameters.simulator)

    if (nealParameters.simulator == 'spinnaker'):
        inputCells = sim.Population(1,
                    externaldevices.SpikeInjector,
                               {'port': 12349}, label="bumpSpikeToBoard")
        externaldevices.activate_live_output_for(inputCells,
                    database_notify_port_num=12349)


        bumpCells = sim.Population(5,sim.IF_cond_exp, fsa.CELL_PARAMS)
        #connect input to bump
        synWeight = 0.1
        connector = []

        for toNeuron in range (0,5):
            connector=connector+[(0,toNeuron,synWeight,nealParameters.DELAY)]
        neal.nealProjection(inputCells, bumpCells, connector,'excitatory')


    elif (nealParameters.simulator == 'nest'):
        bumpCells = sim.Population(5,sim.IF_cond_exp, fsa.CELL_PARAMS)
        bumpCells.record({'spikes'})
    else:
        print 'bad simulator in cabot3 createBumpCells'
        return 0

    return bumpCells

#If you are in nest read the bump file and if needed turn the bump fact on
def setNestBumpInput(currentTime,bumpCells):
    if (not os.path.isfile("world/bumps.txt")):
        return
    fileHandle = open("world/bumps.txt",'r')
    for inpLine in fileHandle:
        #print 'bump', inpLine
        splitInp = inpLine.split()
        inpTime = int(splitInp[0])
        inpTime+=10
        inpNeuronNumber = int(splitInp[1])
        if ((inpTime <= currentTime) and ((inpTime+30) > currentTime)):
        #if (inpTime == currentTime):
            print "pynn nest bump input ",inpTime
            #voltageList = [-50.0,-50.0,-50.0,-50.0,-50.0]
            voltageList = [-30.0,-50.0,-50.0,-50.0,-50.0]
            bumpCells.initialize(v=voltageList)

def setNestInput(currentSimulationTime,bumpCells):
    parse.setNestInputs(currentSimulationTime)
    setNestBumpInput(currentSimulationTime,bumpCells)

def newActionOutput(currentSimulationTime,plan):
    if (nealParameters.simulator=="nest"):
        output = []
        planActionCells = plan.getActionCells()
        actionData = planActionCells.get_data()
        actionSegments = actionData.segments[0]
        actionSpikeTrains = actionSegments.spiketrains
        for action in range (0,len(actionSpikeTrains)):
            if (len(actionSpikeTrains[action]) > 0):
                anActionsSpikes = actionSpikeTrains[action]
                for spike in range (0,len(anActionsSpikes)):
                    aSpike = anActionsSpikes[spike]
                    if (aSpike <= currentSimulationTime) and (aSpike > currentSimulationTime-30):
                        output = output + [action]
                        #print 'ho', currentSimulationTime,action

        return output
    return []

#open a file for communication 
def setNestOutput(currentSimulationTime,plan):
    if (currentSimulationTime == 0):
        nestOutputFileHandle = open("world/actions.txt",'a')
        nestOutputFileHandle.write(str(currentSimulationTime)+"\n")
        nestOutputFileHandle.close()
        return
    actions = newActionOutput(currentSimulationTime,plan) 
    if (len(actions) > 0):
        nestOutputFileHandle = open("world/actions.txt",'a')
        for actionNum in range (0,len(actions)):
            nestOutputFileHandle.write(str(actions[actionNum])+" "+
                                       str(currentSimulationTime)+"\n")
        nestOutputFileHandle.close()

#-----connect parse to plan functions
def connectParseFinalStateToGoal(parseFinalState,planGoalState, parser,planner):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    parseFinalStateStartNeuron = parseFinalState*fsa.CA_SIZE

    stateNeurons = parse.getStateCells()
    goalNeurons = plan.getGoalCells()

    fsa.stateTurnsOnState(stateNeurons,parseFinalState,goalNeurons,
                          planGoalState)
    fsa.stateTurnsOffState(goalNeurons,planGoalState,stateNeurons,
                           parseFinalState)

def connectParseFinalStatesToPlan(parser,planner): 
    connectParseFinalStateToGoal(parser.turnRightFinalState,
        planner.goalTurnRight,parser,planner)#turn right
    connectParseFinalStateToGoal(parser.turnLeftFinalState,
        planner.goalTurnLeft,parser,planner)#turn left
    connectParseFinalStateToGoal(parser.moveForwardFinalState,
        planner.goalStepForward,parser,planner)#move forward
    connectParseFinalStateToGoal(parser.moveBackFinalState,
        planner.goalStepBackward,parser,planner)#move back

    connectParseFinalStateToGoal(parser.moveRightFinalState,
        planner.goalMoveRight,parser,planner)#move right
    connectParseFinalStateToGoal(parser.moveLeftFinalState,
        planner.goalMoveLeft,parser,planner)#move left

    connectParseFinalStateToGoal(parser.turnTowardPyramidFinalState,
        planner.goalTurnTowardObject,parser,planner)#turnObj in parser to pyr
    
    connectParseFinalStateToGoal(parser.goToDoorFinalState,
        planner.goalGoToDoor,parser,planner)#go to door


    connectParseFinalStateToGoal(parser.exploreFinalState,planner.goalExplore,
                                 parser, planner)#explore

    #move before red pyramid, red stalactite, blue pyramid and blue stalactite
    connectParseFinalStateToGoal(parser.moveBeforeRedPyrFinalState,
                            plan.goalMoveBeforeRedPyramid,parser,planner)
    connectParseFinalStateToGoal(parser.moveBeforeBluePyrFinalState,
                            plan.goalMoveBeforeBluePyramid,parser,planner)
    #consider cover for idObj (e turns on idObj)
    #connectParseFinalStateToGoal(parser.moveBeforeRedStalFinalState,
    #                             plan.goalIDObj,parser,planner)#mbrs
    connectParseFinalStateToGoal(parser.moveBeforeRedStalFinalState,
                                 plan.goalMoveBeforeRedStal,parser,planner)

    #consider cover for goThroughDoor (c turns on goThroughDoor)
    #connectParseFinalStateToGoal(parser.moveBeforeBlueStalFinalState,
    #                             plan.goalGoThroughDoor,parser,planner)
    connectParseFinalStateToGoal(parser.moveBeforeBlueStalFinalState,
                            plan.goalMoveBeforeBlueStal,parser,planner)

#The parser also turns on the cogmap before
def connectParseToPlan(parser,planner): 
    connectParseFinalStatesToPlan(parser,planner)

import time as Time
def runCABot3(plan,vision,bumpCells):
    if (nealParameters.simulator=="spinnaker"):
        sim.run(nealParameters.SIM_LENGTH)
    elif (nealParameters.simulator=="nest"):
        callbackTimeStepLength = 30
        steps = int(nealParameters.SIM_LENGTH/callbackTimeStepLength)
        for step in range (0, steps):
            currentTime = step*callbackTimeStepLength
            print "nest simulation timestep ", currentTime
            setNestInput(currentTime,bumpCells)
            setNestOutput(currentTime,plan)
            vision.setNestVisionInput(currentTime)
            sim.run(callbackTimeStepLength)

def test():
    input = vision.getBWVisionInputCells()
    weight = 20.0
    connectors = []
    for i  in range(0,neuronsPerSubnet):
        #connectors = connectors + [(i,0,weight,DELAY)]
        connectors = connectors + [(i,i,weight,DELAY)]
    connectorList = FromListConnector(connectors) 
    nealProjection(input,testCells, connectorList,'excitatory')

def testColour():
    input = vision.getColourVisionInputCells()
    weight = 20.0
    connectors = []
    for i  in range(0,1200):
        #connectors = connectors + [(i,0,weight,DELAY)]
        connectors = connectors + [(i,i,weight,DELAY)]
    connectorList = FromListConnector(connectors) 
    nealProjection(input,testCells, connectorList,'excitatory')

def connectPlanToCogmap(plan,cogmap):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    planGoalCells = plan.getGoalCells()
    planFactCells = plan.getFactCells()
    cogmapSignalCells = cogmap.getSignalCells()
    cogmapBeforeShapeCells = cogmap.getBeforeShapeCells()

    #explore goal turns on the first signal
    fsa.stateTurnsOnState(planGoalCells,plan.goalExplore,cogmapSignalCells,0)

    #BluePyr,redStal,BlueStal,redPyr
    fsa.stateTurnsOnState(planFactCells,plan.bluePyrFact,cogmapSignalCells,2)
    fsa.stateTurnsOnState(planFactCells,plan.redStalFact,cogmapSignalCells,4)
    fsa.stateTurnsOnState(planFactCells,plan.blueStalFact,cogmapSignalCells,6)
    fsa.stateTurnsOnState(planFactCells,plan.redPyrFact,cogmapSignalCells,8)

#During an explore, a shape is IDed when it turns on something in the cogmap.
#So, the signal, turns the moduleObjID-ed on.
def cogmapExploreShapeIDsObj(cogmap,plan):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    cogmapSignalCells = cogmap.getSignalCells()

    planModuleCells = plan.getModuleCells()

    #all four shape signals
    fsa.stateTurnsOnState(cogmapSignalCells,2,planModuleCells,
                          plan.moduleObjIdentified)
    fsa.stateTurnsOnState(cogmapSignalCells,4,planModuleCells,
                               plan.moduleObjIdentified)
    fsa.stateTurnsOnState(cogmapSignalCells,6,planModuleCells,
                               plan.moduleObjIdentified)
    fsa.stateTurnsOnState(cogmapSignalCells,8,planModuleCells,
                               plan.moduleObjIdentified)


def cogmapOutStartsMoveToFacts(cogmap,plan):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    planFactCells = plan.getFactCells()
    cogmapOutShapeCells = cogmap.getOutShapeCells()

    fsa.stateTurnsOnStateSlow(cogmapOutShapeCells,0,planFactCells,
              plan.moveToBluePyramidFact) 
    fsa.stateTurnsOnStateSlow(cogmapOutShapeCells,1,planFactCells,
              plan.moveToRedStalFact) 
    fsa.stateTurnsOnStateSlow(cogmapOutShapeCells,2,planFactCells,
              plan.moveToBlueStalFact) 
    fsa.stateTurnsOnStateSlow(cogmapOutShapeCells,3,planFactCells,
              plan.moveToRedPyramidFact) 

#When the cogmap has read all four shapes, it goes to the postExploreState
#This should set the fact exploreDone
def cogmapPostExploreStateStartsExploreDoneFact(cogmap,plan):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    cogmapStateCells = cogmap.getStateCells()
    planFactCells = plan.getFactCells()

    fsa.stateTurnsOnStateSlow(cogmapStateCells,cogmap.postExploreState,
                             planFactCells,plan.exploreDoneFact) 

#When the plan figures out that it has gotten to the state, which you can
#tell via the action, it needs to move the state of the cm back to postexplore
def  planMovedToShapeRestartsPostExplore(cogmap,plan):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    planActionCells = plan.getActionCells()
    cogmapStateCells = cogmap.getStateCells()

    fsa.turnOnStateFromOneNeuron(planActionCells,plan.actionMovedToShape,
                                      cogmapStateCells,cogmap.postExploreState)


def connectPlanAndCogmap(plan,cogmap):
    connectPlanToCogmap(plan,cogmap)
    cogmapExploreShapeIDsObj(cogmap,plan)
    cogmapOutStartsMoveToFacts(cogmap,plan) 
    cogmapPostExploreStateStartsExploreDoneFact(cogmap,plan)
    planMovedToShapeRestartsPostExplore(cogmap,plan)


def connectVisionToPlan(vision,plan):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    visionObjRecCells = vision.getObjRecCells()
    planFactCells = plan.getFactCells()
    objToFactSynWeight = 0.004 
    objToDoorFactWeight = 0.004 

    if (nealParameters.simulator=="spinnaker"):
        objToInDoorFactSynWeight = 0.008 #.002 too little
        objToShapeFactSynWeight = 0.0018
    elif (nealParameters.simulator=="nest"):
        objToInDoorFactSynWeight = 0.01
        objToShapeFactSynWeight = 0.0025
    
    #objects
    fsa.stateStimulatesState(visionObjRecCells,vision.objLeft,
        planFactCells,plan.objLeftFact,objToFactSynWeight) #leftObj->ltobj
    fsa.stateStimulatesState(visionObjRecCells,vision.objCtr,
        planFactCells,plan.objCtrFact,objToFactSynWeight) #ctrObj->ctrobj
    fsa.stateStimulatesState(visionObjRecCells,vision.objRight,
        planFactCells,plan.objRightFact,objToFactSynWeight) #rtObj->rtobj

    #doors
    fsa.stateStimulatesState(visionObjRecCells,vision.doorLeft,
        planFactCells,plan.doorLeftFact,objToDoorFactWeight) #ltDoor->ltDoor
    fsa.stateStimulatesState(visionObjRecCells,vision.doorCtr,
        planFactCells,plan.doorCtrFact,objToDoorFactWeight) #ctrDoor->ctrDoor
    fsa.stateStimulatesState(visionObjRecCells,vision.doorRight,
        planFactCells,plan.doorRightFact,objToDoorFactWeight) #rtDoor->rtDoor

    #inFrontDoor 
    fsa.stateStimulatesState(visionObjRecCells,vision.inFrontDoorObj,
        planFactCells,plan.inFrontDoorFact,objToInDoorFactSynWeight)

    #inDoors
    fsa.stateStimulatesState(visionObjRecCells,vision.inDoorLeftObj,
        planFactCells,plan.inDoorLeftFact,objToInDoorFactSynWeight)#ltDr->ltDoor
    fsa.stateStimulatesState(visionObjRecCells,vision.inDoorCtrObj,
        planFactCells,plan.inDoorCtrFact,objToInDoorFactSynWeight) 
    fsa.stateStimulatesState(visionObjRecCells,vision.inDoorRightObj,
        planFactCells,plan.inDoorRightFact,objToInDoorFactSynWeight) 

    #bluepyrs
    fsa.stateStimulatesState(visionObjRecCells,vision.bluePyrLeftObj,
        planFactCells,plan.bluePyrLeftFact,objToShapeFactSynWeight) 
    fsa.stateStimulatesState(visionObjRecCells,vision.bluePyrCtrObj,
        planFactCells,plan.bluePyrCtrFact,objToShapeFactSynWeight) 
    fsa.stateStimulatesState(visionObjRecCells,vision.bluePyrRightObj,
        planFactCells,plan.bluePyrRightFact,objToShapeFactSynWeight) 

    #redpyrs
    fsa.stateStimulatesState(visionObjRecCells,vision.redPyrLeftObj,
        planFactCells,plan.redPyrLeftFact,objToShapeFactSynWeight) 
    fsa.stateStimulatesState(visionObjRecCells,vision.redPyrCtrObj,
        planFactCells,plan.redPyrCtrFact,objToShapeFactSynWeight) 
    fsa.stateStimulatesState(visionObjRecCells,vision.redPyrRightObj,
        planFactCells,plan.redPyrRightFact,objToShapeFactSynWeight) 

    #bluestals
    fsa.stateStimulatesState(visionObjRecCells,vision.blueStalLeftObj,
        planFactCells,plan.blueStalLeftFact,objToShapeFactSynWeight) 
    fsa.stateStimulatesState(visionObjRecCells,vision.blueStalCtrObj,
        planFactCells,plan.blueStalCtrFact,objToShapeFactSynWeight) 
    fsa.stateStimulatesState(visionObjRecCells,vision.blueStalRightObj,
        planFactCells,plan.blueStalRightFact,objToShapeFactSynWeight) 

    #redstals
    fsa.stateStimulatesState(visionObjRecCells,vision.redStalLeftObj,
        planFactCells,plan.redStalLeftFact,objToShapeFactSynWeight) 
    fsa.stateStimulatesState(visionObjRecCells,vision.redStalCtrObj,
        planFactCells,plan.redStalCtrFact,objToShapeFactSynWeight) 
    fsa.stateStimulatesState(visionObjRecCells,vision.redStalRightObj,
        planFactCells,plan.redStalRightFact,objToShapeFactSynWeight) 

def connectVisionAndPlan(vision,plan):
    connectVisionToPlan(vision,plan)

#the move before parses directly turn on the cogmap beforeRooms
def connectParseAndCogmap(parser,cogmap):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    parseStateCells = parser.getStateCells()
    cogmapBeforeShapeCells = cogmap.getBeforeShapeCells()

    #turn on cogmap before shape bp,rs,bs,rp
    fsa.stateStimulatesState(parseStateCells,parser.moveBeforeBluePyrFinalState,
                             cogmapBeforeShapeCells,0,0.03)
    #consider idobj and goThroughDoor
    fsa.stateStimulatesState(parseStateCells,parser.moveBeforeRedStalFinalState,
                             cogmapBeforeShapeCells,1,0.03)
    fsa.stateStimulatesState(parseStateCells,parser.moveBeforeBlueStalFinalState,
                             cogmapBeforeShapeCells,2,0.03)
    fsa.stateStimulatesState(parseStateCells,parser.moveBeforeRedPyrFinalState,
                             cogmapBeforeShapeCells,3,0.03)


def connectSpinnakerBumpToPlan(bumpCells,bumpTimer,plan):
    planFactCells = plan.getFactCells()

    bumpTimer.oneNeuronStartsTimer(bumpCells,0)
    bumpTimer.timerStartsState(planFactCells,plan.bumpFact)

def connectNestBumpToPlan(bumpCells,plan):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    planFactCells = plan.getFactCells()

    fsa.turnOnStateFromOneNeuron(bumpCells,0,planFactCells,plan.bumpFact)

def cmTestMakeSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        inputSpikeTimes3 = inputTimes[2]
        inputSpikeTimes4 = inputTimes[3]
        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')
        spikeArray3 = {'spike_times': [inputSpikeTimes3]}
        spikeGen3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='inputSpikes_3')
        spikeArray4 = {'spike_times': [inputSpikeTimes4]}
        spikeGen4=sim.Population(1,sim.SpikeSourceArray,spikeArray4,
                                   label='inputSpikes_4')
        spikeArray5 = {'spike_times': [inputTimes[4]]}
        spikeGen5=sim.Population(1,sim.SpikeSourceArray,spikeArray5,
                                   label='inputSpikes_5')
        spikeArray6 = {'spike_times': [inputTimes[5]]}

        return [spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,]
        #return [spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,spikeGen6]
        #return [spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,spikeGen6,spikeGen7]

#Learning the cogmap can be dodgy.  Put a test in that puts
#the relevant features.  
def cogmapTestCellsOn(cm):
        spikeSources = cmTestMakeSpikeSources([10.0,110.0,250.0,500.0,750.0,
                                               1000.0, 1500.0]) 
        #turn on state0 that turns off all shapes.
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[0],cm.stateCells,0)#PreExplore
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[1],cm.stateCells,12)#testExplore
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[2],cm.inShapeCells,0)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[3],cm.inShapeCells,1)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[4],cm.inShapeCells,2)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[5],cm.inShapeCells,3)
        #test the binding
        #cm.fsa.turnOnStateFromSpikeSource(spikeSources[6],cm.stateCells,13)
        #test the room before
        #cm.fsa.turnOnStateFromSpikeSource(spikeSources[6],cm.stateCells,14)
        #cm.fsa.turnOnStateFromSpikeSource(spikeSources[6],cm.outShapeCells,1)

#If you want to test that go before x works, call this.
#It sets the rooms in order bp,rs,bs,rp.  Change this order by changing
#the signal order below.  You then execute the appropriate command in
#the environment e.g e move before the red stalactite.
def cogmapTestCMSignalCells(cm):
        signalCells = cm.getSignalCells()
        #spikeSources = cmTestMakeSpikeSources([10.0,110.0,250.0,500.0,750.0,
        #                                       1000.0, 1500.0]) 
        spikeSources = cmTestMakeSpikeSources([10.0,500.0,1000.0,1500.0,2000.0,
                                               2500.0, 3000.0]) 
        #turn on state0 that turns off all shapes.
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[0],signalCells,0)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[1],signalCells,2)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[2],signalCells,4)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[3],signalCells,6)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[4],signalCells,8)
        

#------------Main Body---------------
#import the simulator via nealParams.py
from time import *
startTime = localtime()

neal = NealCoverFunctions(nealParameters.simulator)
init()

#for planAction running reset reset
bumpCells = createBumpCells()
if (nealParameters.simulator == 'spinnaker'):
    bumpCells.record()
elif (nealParameters.simulator == 'nest'):
    bumpCells.record({'spikes'})

#initialize objects
parse = ParseClass(nealParameters.simulator)
parse.createParseInputs()
parse.connectInputsToWordsAndFirstState()
#parse.recordSpinnakerInputs()

plan = PlanClass(nealParameters.simulator)

vision = visionArea(nealParameters.simulator)
vision.allocateVisionNeurons()
vision.makeVisionSynapses()
if (not nealParameters.printSmall):
    vision.setVisionRecording()

cogmap = cogmapArea()
initSpikeArray = {'spike_times': [10.0]}
initSpikeGen=sim.Population(1,sim.SpikeSourceArray,initSpikeArray,
                            label='InitInputSpikes')
cogmap.startFSA(initSpikeGen)
#cogmap.setUpFullAndBindTest()
#cogmapTestCellsOn(cogmap)
#cogmapTestCMSignalCells(cogmap)

#connect subsystems
connectParseToPlan(parse,plan)
connectPlanAndCogmap(plan,cogmap)
connectVisionAndPlan(vision,plan)
connectParseAndCogmap(parse,cogmap)
if (nealParameters.simulator == 'spinnaker'):
    #make a timer for bump
    bumpTimer = TimerClass(nealParameters.simulator,10)
    bumpTimer.makeStopNoRestartTimerSynapses()
    connectSpinnakerBumpToPlan(bumpCells,bumpTimer,plan)

elif (nealParameters.simulator == 'nest'):
    connectNestBumpToPlan(bumpCells,plan)

runCABot3(plan,vision,bumpCells)

#print out debugging trace
if (nealParameters.simulator == 'spinnaker'):
    bumpCells.printSpikes('results/cabotBump.sp')
    bumpTimer.printResults("results/cabotBumpTimer")
elif (nealParameters.simulator == 'nest'):
    bumpCells.printSpikes('results/cabotBump.pkl')


parse.printResults()
#parse.printInputs()
plan.printResults()
if (not nealParameters.printSmall):
    vision.printVisionNets()
cogmap.printCogMapNets()

print 'start',startTime
print 'end',localtime()

