#The heart of this class is two sets of CAs one for the rooms and
#one for the shapes.  These are bound together in response to the
#environment.
#They can also be activated to retrieve the binding.
#When you're filling up the map, you first say you're exploring, by turning
#on the signalState 0.  You then turn on the shapes as you see them by
#turning on the signalState 2,4,6,8.

#Next room works with roomBindCells.  The first roomBind comes on when
#the explore state comes on.  Each roomBind turns on its nextRoom, but
#the nextRooms are prevented by the waitForShape state.  So, the next room 
#comes on when bind
#comes on.  The nextRoom state in collaboration with the current nextRoom,
#turns on the next roomBindCells.

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
    import spynnaker_external_devices_plugin.pyNN as externaldevices
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from timerClass import TimerClass

class cogmapArea:
    def __init__(self):
        self.neal = NealCoverFunctions(nealParameters.simulator)
        self.fsa = FSAHelperFunctions(nealParameters.simulator)
 
        self.numberRooms = 4
        self.numberTestStates = 4
        self.numberStates = 12+self.numberTestStates #12 real 4 test
        self.numberSignalCAs = 10
        #crh
        #preExplore, explore,waitForShape,bind,stopBind, #0-4
        #nextRoom,postExplore,getPriorBefore, getPrior, getPriorRoom #5-9
        #setPriorRoomBind,priorShapeFound #10-11
        #testExplore,testBindState,testBeforeState
        self.allocateCogMapNeurons()
        self.makeCogMapSynapses()
        self.setupCogMapRecording()

    #When the system starts running, it should turn on the cognitive maps
    #preExplore state.
    def startFSA(self,spikeSource):
        self.fsa.turnOnStateFromSpikeSource(spikeSource,self.stateCells,0)

    def createTimers(self):
        exploreToWait = TimerClass(nealParameters.simulator,10)
        if (nealParameters.simulator=="spinnaker"):
            bindToStopBind = TimerClass(nealParameters.simulator,12)
        elif (nealParameters.simulator=="nest"):
            bindToStopBind = TimerClass(nealParameters.simulator,10)
        stopToNextRoom = TimerClass(nealParameters.simulator,10)
        nextRoomToWait = TimerClass(nealParameters.simulator,10)
        priorToPriorRoom = TimerClass(nealParameters.simulator,10)
        #the timerClass sets recording
        exploreToWait.makeStopNoRestartTimerSynapses()
        bindToStopBind.makeStopNoRestartTimerSynapses()
        stopToNextRoom.makeStopNoRestartTimerSynapses()
        nextRoomToWait.makeStopNoRestartTimerSynapses()
        priorToPriorRoom.makeStopNoRestartTimerSynapses()
        return [exploreToWait,bindToStopBind,stopToNextRoom,nextRoomToWait,
                priorToPriorRoom]

    def allocateCogMapNeurons(self):
        numberBaseCells = self.numberRooms*self.fsa.CA_SIZE
        numberStateCells = self.numberStates*self.fsa.CA_SIZE
        numberSignalCells = self.numberSignalCAs*self.fsa.CA_SIZE
        self.roomBindCells = sim.Population(numberBaseCells,
                sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        self.shapeBindCells = sim.Population(numberBaseCells,
                sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        self.stateCells = sim.Population(numberStateCells,
                sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        timers = self.createTimers()
        self.exploreToWaitTimer = timers[0]
        self.bindToStopBindTimer = timers[1]
        self.stopToNextRoomTimer = timers[2]
        self.nextRoomToWaitTimer = timers[3]
        self.priorToPriorRoomTimer = timers[4]
        self.inShapeCells = sim.Population(numberBaseCells,
                sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        self.nextRoomCells = sim.Population(numberBaseCells,
                sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        self.beforeShapeCells = sim.Population(numberBaseCells,
                sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        self.priorRoomCells = sim.Population(numberBaseCells,
                sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        self.outShapeCells = sim.Population(numberBaseCells,
                sim.IF_cond_exp,self.fsa.CELL_PARAMS)
        self.signalStateCells = sim.Population(numberSignalCells,
                sim.IF_cond_exp,self.fsa.CELL_PARAMS)

    def stateTurnsOffAllRooms(self,stateNumber):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOffState(self.stateCells,stateNumber,
                                        self.roomBindCells,roomCANum)

    def stateTurnsOffAllShapes(self,stateNumber):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOffState(self.stateCells,stateNumber,
                                        self.shapeBindCells,roomCANum)


    #Make synapses for a set of neurons to turn on the next as the
    #basis of a timer.  Could move to FSAClass.
    def connectTimerElements(self,element):
        synWeight = 0.013 #every 5ms in spinn, every 6ms in nest
        connector = []
        fromStart = element*self.fsa.CA_SIZE

        #forward to turn on next
        for fromNeuron in range (fromStart,fromStart+self.fsa.CA_SIZE):
            for toNeuron in range (fromStart+self.fsa.CA_SIZE,
                                   fromStart+(2*self.fsa.CA_SIZE)):
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                      nealParameters.DELAY)]
        self.neal.nealProjection(self.timerCells, self.timerCells, 
                                 connector,'excitatory')

        #back to inhibit all earlier
        if (fromStart == 0):
            return
        connector = []
        inhSynWeight = -0.3
        for fromNeuron in range (fromStart,fromStart+self.fsa.CA_SIZE):
            for toNeuron in range (0, fromStart):
                connector=connector+[(fromNeuron,toNeuron,inhSynWeight,
                                      nealParameters.DELAY)]
        self.neal.nealProjection(self.timerCells, self.timerCells, 
                                 connector,'inhibitory')

    def makeLearningSynapses(self):
        numberBaseCells = self.numberRooms*self.fsa.CA_SIZE
        if (nealParameters.simulator=="spinnaker"):
            synWeight = 0.0
        elif (nealParameters.simulator=="nest"):
            synWeight = 0.001

        connector = []
        for fromNeuron in range (0,numberBaseCells):
            for toNeuron in range (0,numberBaseCells):
                connector=connector+[(fromNeuron,toNeuron,synWeight,
                                      nealParameters.DELAY)]

        connList = sim.FromListConnector(connector)
        if (nealParameters.simulator=="spinnaker"):
            stdp_model = sim.STDPMechanism(
                    timing_dependence = sim.SpikePairRule(tau_plus=16.7, 
                                        tau_minus=3.7, nearest=True),
                    weight_dependence = sim.AdditiveWeightDependence(w_min=0.0, 
                         w_max=0.015, A_plus=0.01, A_minus=0.002) #a_plus=0.005 
                )
            sim.Projection(self.shapeBindCells,self.roomBindCells,connList,
                synapse_dynamics = sim.SynapseDynamics(slow = stdp_model),
                  target='excitatory'   )
            sim.Projection(self.roomBindCells,self.shapeBindCells,connList,
                synapse_dynamics = sim.SynapseDynamics(slow = stdp_model),
                  target='excitatory'   )


        elif (nealParameters.simulator=="nest"):
                stdp_model = sim.STDPMechanism(
                    timing_dependence=sim.SpikePairRule(tau_plus=16.7, 
                                #tau_minus=3.7,A_plus=0.01, A_minus=0.0001),
                            tau_minus=3.7,A_plus=0.005, A_minus=0.002),
                    weight_dependence=sim.AdditiveWeightDependence(w_min=0, 
                                                                   w_max=0.02),
                    weight=0.0,
                    delay=nealParameters.DELAY,
                    dendritic_delay_fraction=float(1))

                sim.Projection(self.shapeBindCells,self.roomBindCells,
                               sim.AllToAllConnector(),
                               synapse_type = stdp_model)

                sim.Projection(self.roomBindCells,self.shapeBindCells,
                               sim.AllToAllConnector(),
                               synapse_type = stdp_model)

        else:
                print 'bad simulator in cogmap.makeLearningSynapses', nealParameters.simulator

    def connectInShapeToShapeBind(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOnState(self.inShapeCells,roomCANum,
                                      self.shapeBindCells,roomCANum)
            self.fsa.stateTurnsOffState(self.shapeBindCells,roomCANum,
                                    self.inShapeCells,roomCANum)

            
    def connectRoomBindAndNextRoom(self):
        synWeight = 0.001 
        connector = []
        for roomCANum in range (0,self.numberRooms): 
            #too hot
            #self.fsa.stateHalfTurnsOnState(self.roomBindCells,roomCANum,self.nextRoomCells,roomCANum)
            fromStart = roomCANum*self.fsa.CA_SIZE
            for fromNeuron in range (fromStart,fromStart+self.fsa.CA_SIZE):
                for toNeuron in range (fromStart,fromStart+self.fsa.CA_SIZE):
                    connector=connector+[(fromNeuron,toNeuron,synWeight,
                                      nealParameters.DELAY)]
        self.neal.nealProjection(self.roomBindCells, self.nextRoomCells, 
                                 connector,'excitatory')


        #nextRooms primes the next roomBind
        for roomCANum in range (0,self.numberRooms-1): 
            self.fsa.stateHalfTurnsOnState(self.nextRoomCells,roomCANum,
                                           self.roomBindCells,roomCANum+1)

    #state names
    preExploreState = 0
    exploreState = 1
    waitForShapeState = 2
    bindState = 3
    stopBindState = 4
    nextRoomState = 5
    postExploreState = 6
    getPriorBeforeState = 7
    getPriorState = 8
    getPriorRoomState = 9
    setPriorRoomState = 10
    shapeFoundState = 11
    testExpState = 12
    testBindState=13
    testStopBindState=14
    testBoundState=15

    def preExplorePreventsInput(self):
        for room in range (0,self.numberRooms):
            self.fsa.stateTurnsOffState(self.stateCells,self.preExploreState,
                                    self.inShapeCells,room)

    def waitForShapeStateStopsNextRooms(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOffState(self.stateCells,self.waitForShapeState,
                                    self.nextRoomCells,roomCANum)

    def shapeBindPrimesBindState(self):
        for roomCANum in range (0,self.numberRooms): 
            #self.fsa.stateHalfTurnsOnState(self.shapeBindCells,roomCANum,
            self.fsa.stateStimulatesState(self.shapeBindCells,roomCANum,
                    self.stateCells,self.bindState,0.0004)

    #to get the next room on, use the current bindRoom and bindState
    def bindStatePrimesNextRooms(self):
        for roomCANum in range (0,self.numberRooms):
            self.fsa.stateHalfTurnsOnState(self.stateCells,self.bindState,
                                           self.nextRoomCells,roomCANum)

    def nextRoomStatePrimesRoomBind(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateHalfTurnsOnState(self.stateCells,self.nextRoomState,
                                           self.roomBindCells,roomCANum)

    def lastNextRoomPrimesPostExploreState(self):
        self.fsa.stateHalfTurnsOnState(self.nextRoomCells,self.numberRooms-1,
                self.stateCells,self.postExploreState)

    def postExploreStatesPreventNextRoom(self):
        for state in range (self.postExploreState,7):  #undone add extra states
            for room in range (0,self.numberRooms):
                self.fsa.stateTurnsOffState(self.stateCells,state,
                                    self.nextRoomCells,room)

    def beforeShapesPrimeGetPriorBeforeState(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateHalfTurnsOnState(self.beforeShapeCells,roomCANum,
                                self.stateCells,self.getPriorBeforeState)

    #The FSA is usually preventing prior rooms, except during
    #getting the prior room.
    def mostStatesPreventPriorRoom(self):
        for state in range (0,self.numberStates-3):  #ignore the tests
            if (state != self.getPriorBeforeState) and (state != self.getPriorState) and (state != self.getPriorRoomState) and (state != self.setPriorRoomState):
                for room in range (0,self.numberRooms):
                    self.fsa.stateTurnsOffState(self.stateCells,state,
                                    self.priorRoomCells,room)

    def priorRoomsPrimeGetPriorState(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateHalfTurnsOnState(self.priorRoomCells,roomCANum,
                                       self.stateCells,self.getPriorState)

    def getPriorStateStopsBindsAndBefore(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOffState(self.stateCells,self.getPriorState,
                                    self.shapeBindCells,roomCANum)
            self.fsa.stateTurnsOffState(self.stateCells,self.getPriorState,
                                    self.roomBindCells,roomCANum)
            self.fsa.stateTurnsOffState(self.stateCells,self.getPriorState,
                                    self.beforeShapeCells,roomCANum)


    #From getPriorRoomState turn on roomBind (the priorRoom primes
    #the one before.  When a roomBind comes on, we move to setPriorRoomState.
    #When the output comes on we move to foundState
    def getPriorRoomPrimesRoomBind(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateHalfTurnsOnState(self.stateCells,
                    self.getPriorRoomState,self.roomBindCells,roomCANum)

    def roomBindPrimesSetPrior(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateHalfTurnsOnState(self.roomBindCells,roomCANum,
                        self.stateCells, self.setPriorRoomState)

    def setPriorRoomStopsPriorRooms(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOffState(self.stateCells,self.setPriorRoomState,
                                    self.priorRoomCells,roomCANum)

    def getPriorRoomStatePreventsRoomBind(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOffState(self.stateCells,self.getPriorRoomState,
                                    self.roomBindCells,roomCANum)

    #This input plus the prime from the priorRoom already on, should
    #turn on the rooom before.
    def getPriorRoomStatePrimesPriorRooms(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateHalfTurnsOnState(self.stateCells,
                    self.getPriorRoomState, self.priorRoomCells,roomCANum)

    def mostStatesPreventOutShape(self):
        #ignore the tests setPriorRoom and foundShape
        for state in range (0,self.numberStates-(2+self.numberTestStates)):  
                for room in range (0,self.numberRooms):
                    self.fsa.stateTurnsOffState(self.stateCells,state,
                                    self.outShapeCells,room)

    def outShapesPrimeShapeFoundState(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateHalfTurnsOnState(self.outShapeCells,roomCANum,
                                self.stateCells,self.shapeFoundState)

    def shapeFoundStateStopsBindsAndPrior(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOffState(self.stateCells,self.shapeFoundState,
                                    self.shapeBindCells,roomCANum)
            self.fsa.stateTurnsOffState(self.stateCells,self.shapeFoundState,
                                    self.roomBindCells,roomCANum)
            #mostStatesPreventPriorRoom
            #self.fsa.stateTurnsOffState(self.stateCells,self.shapeFoundState,
            #                        self.priorRoomCells,roomCANum)



    def makeFSA(self):
        self.preExplorePreventsInput()

        #PreExplore primes Explore 
        self.fsa.stateHalfTurnsOnState(self.stateCells,0,self.stateCells,
                                       self.exploreState)
        self.fsa.stateTurnsOffState(self.stateCells,self.exploreState,
                                    self.stateCells,0)

        #Explore + Timer -> waitForShape (explore turns on timer)
        #turn on the first roomBind room
        self.exploreToWaitTimer.timerStartsState(self.roomBindCells,0) 
        self.exploreToWaitTimer.stateStartsTimer(self.stateCells,
                                                 self.exploreState)
        self.exploreToWaitTimer.timerStartsState(self.stateCells,
                                                 self.waitForShapeState)
        self.fsa.stateTurnsOffState(self.stateCells,self.waitForShapeState,
                                    self.stateCells,self.exploreState)

        #waitForShape + shape -> Bind
        self.waitForShapeStateStopsNextRooms()
        self.shapeBindPrimesBindState()
        self.fsa.stateHalfTurnsOnState(self.stateCells,self.waitForShapeState,
                self.stateCells, self.bindState)
        self.fsa.stateTurnsOffState(self.stateCells,self.bindState,
                                    self.stateCells,self.waitForShapeState)

        #bind + timer -> stopBind
        self.bindStatePrimesNextRooms()
        self.bindToStopBindTimer.stateStartsTimer(self.stateCells,
                                                 self.bindState)
        self.bindToStopBindTimer.timerStartsState(self.stateCells,
                                                 self.stopBindState)


        self.stateTurnsOffAllRooms(self.stopBindState)
        self.stateTurnsOffAllShapes(self.stopBindState)
        self.fsa.stateTurnsOffState(self.stateCells,self.stopBindState,
                                    self.stateCells,self.bindState)

        #stopBind + timer -> nextRoom
        self.stopToNextRoomTimer.stateStartsTimer(self.stateCells,
                                                 self.stopBindState)
        self.stopToNextRoomTimer.timerStartsState(self.stateCells,
                                                 self.nextRoomState)
        self.fsa.stateTurnsOffState(self.stateCells,self.nextRoomState,
                self.stateCells,self.stopBindState)

        #nextRoom + timer -> waitForShape
        self.nextRoomStatePrimesRoomBind()
        self.nextRoomToWaitTimer.stateStartsTimer(self.stateCells,
                                                 self.nextRoomState)
        self.nextRoomToWaitTimer.timerStartsState(self.stateCells,
                                                 self.waitForShapeState)
        self.fsa.stateTurnsOffState(self.stateCells,self.waitForShapeState,
                self.stateCells, self.nextRoomState)

        #nextRoom + lastRm -> postExplore
        #shut off timer from other transition from nextRoom
        self.lastNextRoomPrimesPostExploreState()
        self.fsa.stateHalfTurnsOnState(self.stateCells,self.nextRoomState,
                                       self.stateCells,self.postExploreState)
        self.fsa.stateTurnsOffState(self.stateCells,self.postExploreState,
                                    self.stateCells,self.nextRoomState)
        self.nextRoomToWaitTimer.stateStopsTimer(self.stateCells,
                                                 self.postExploreState)
        self.postExploreStatesPreventNextRoom()

        #postExplore + beforeShape -> getPriorBefore
        self.fsa.stateHalfTurnsOnState(self.stateCells,self.postExploreState,
                                       self.stateCells,self.getPriorBeforeState)
        self.fsa.stateTurnsOffState(self.stateCells,self.getPriorBeforeState,
                                    self.stateCells,self.postExploreState)
        self.beforeShapesPrimeGetPriorBeforeState()

        self.mostStatesPreventPriorRoom()

        #getPriorBefore + priorRoom -> getPrior
        self.fsa.stateHalfTurnsOnState(self.stateCells,self.getPriorBeforeState,
                                       self.stateCells,self.getPriorState)
        self.fsa.stateTurnsOffState(self.stateCells,self.getPriorState,
                                    self.stateCells,self.getPriorBeforeState)
        self.priorRoomsPrimeGetPriorState()
        self.getPriorStateStopsBindsAndBefore()

        #getPrior + slide -> getPriorRoom
        self.priorToPriorRoomTimer.stateStartsTimer(self.stateCells,
                                                 self.getPriorState)
        self.priorToPriorRoomTimer.timerStartsState(self.stateCells,
                                                 self.getPriorRoomState)
        self.fsa.stateTurnsOffState(self.stateCells,self.getPriorRoomState,
                                    self.stateCells,self.getPriorState)

        self.getPriorRoomPrimesRoomBind() 

        #getPriorRoom roomBind -> setPriorRoom
        self.fsa.stateHalfTurnsOnState(self.stateCells,self.getPriorRoomState,
                                      self.stateCells,self.setPriorRoomState)
        self.fsa.stateTurnsOffState(self.stateCells,self.setPriorRoomState,
                                    self.stateCells,self.getPriorRoomState)
        self.roomBindPrimesSetPrior()
        self.mostStatesPreventOutShape()
        self.setPriorRoomStopsPriorRooms()

        #setPriorRoom + outShape -> shapefound
        self.fsa.stateHalfTurnsOnState(self.stateCells,self.setPriorRoomState,
                                      self.stateCells,self.shapeFoundState)
        self.fsa.stateTurnsOffState(self.stateCells,self.shapeFoundState,
                                    self.stateCells,self.setPriorRoomState)

        #shapeFound -> postExplore with external help.
        self.fsa.stateHalfTurnsOnState(self.stateCells,self.shapeFoundState,
                                      self.stateCells,self.postExploreState)
        self.fsa.stateTurnsOffState(self.stateCells,self.postExploreState,
                                    self.stateCells,self.shapeFoundState)


        self.outShapesPrimeShapeFoundState()
        self.shapeFoundStateStopsBindsAndPrior()

    def statesPreventBeforeShapes(self):
        for state in range(self.preExploreState,self.nextRoomState):
            for room in range (0,self.numberRooms):
                self.fsa.stateTurnsOffState(self.stateCells,state,
                                    self.beforeShapeCells,room)


    #the before shape cells are turned on when starting to 
    #retrieve the room before.  Connect CAs one to one.
    def connectBeforeShapeToShapeBind(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOnState(self.beforeShapeCells,roomCANum,
                                          self.shapeBindCells,roomCANum)

    def connectRoomBindAndPriorRoom(self):
        #RoomBind turns on Prior
        self.fsa.stateTurnsOnState(self.roomBindCells,0,
                                          self.priorRoomCells,0)
        for roomCANum in range (1,self.numberRooms): 
            #RoomBind turns on Prior
            self.fsa.stateTurnsOnState(self.roomBindCells,roomCANum,
                                          self.priorRoomCells,roomCANum)
            #Prior Primes RoomBind
            self.fsa.stateHalfTurnsOnState(self.priorRoomCells,roomCANum,
                                           self.roomBindCells,roomCANum-1)
        #Prior Primes RoomBind 0 primes 3
        self.fsa.stateHalfTurnsOnState(self.priorRoomCells,0,
                                           self.roomBindCells,3)


    def connectShapeBindToOutShape(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOnState(self.shapeBindCells,roomCANum,
                                          self.outShapeCells,roomCANum)

    exploreOnSignal = 0
    exploreSignalDone = 1
    #turn the signal on.  It switches from preExplore to Explore.  Then
    #the signalDone comes on.
    def connectSignalInternally(self):
        #pairwise
        for statePair in range (0,self.numberSignalCAs/2):
            onSignal = statePair*2
            signalDone = (statePair*2)+1
            self.fsa.stateHalfTurnsOnState(self.signalStateCells,onSignal,
                                           self.signalStateCells,signalDone)
            self.fsa.stateTurnsOffState(self.signalStateCells,signalDone,
                                    self.signalStateCells,onSignal)

    def connectSignalToFSA(self):
        self.fsa.stateHalfTurnsOnState(self.signalStateCells,
                                       self.exploreOnSignal,
                                       self.stateCells,self.exploreState)
        self.fsa.stateHalfTurnsOnState(self.stateCells,self.exploreState,
                                self.signalStateCells,self.exploreSignalDone)
        #preExplore prevents the shape signals
        for roomCANum in range (0,self.numberRooms): 
            signalOnNum = roomCANum*2 + 2
            self.fsa.stateTurnsOffState(self.stateCells,self.preExploreState,
                                    self.signalStateCells,signalOnNum)

    def connectSignalToInputShape(self):
        for roomCANum in range (0,self.numberRooms): 
            signalOnNum = roomCANum*2 + 2
            signalDoneNum = roomCANum*2 + 3
            #the signal turns on the inputShape
            self.fsa.stateTurnsOnState(self.signalStateCells,signalOnNum,
                                          self.inShapeCells,roomCANum)
            #the inputShape sets the signalDoneOn
            self.fsa.stateTurnsOnState(self.inShapeCells,roomCANum,
                                          self.signalStateCells,signalDoneNum)
                                          
    def makeCogMapSynapses(self):
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.makeCA(self.roomBindCells,roomCANum)
            self.fsa.makeCA(self.shapeBindCells,roomCANum)
            self.fsa.makeCA(self.inShapeCells,roomCANum)
            self.fsa.makeCA(self.nextRoomCells,roomCANum)
            self.fsa.makeCA(self.beforeShapeCells,roomCANum)
            self.fsa.makeCA(self.priorRoomCells,roomCANum)
            self.fsa.makeCA(self.outShapeCells,roomCANum)

        for stateCANum in range (0,self.numberStates): 
            self.fsa.makeCA(self.stateCells,stateCANum)

        for signalStateCANum in range (0,self.numberSignalCAs): 
            self.fsa.makeCA(self.signalStateCells,signalStateCANum)

        self.makeLearningSynapses()
        self.connectInShapeToShapeBind()
        self.connectRoomBindAndNextRoom()

        #FSA preventing other cogmap subnets is in FSA
        self.makeFSA()
        
        self.statesPreventBeforeShapes()

        self.connectBeforeShapeToShapeBind()
        self.connectRoomBindAndPriorRoom()
        self.connectShapeBindToOutShape()

        self.connectSignalInternally()
        self.connectSignalToFSA()

        self.connectSignalToInputShape()
        #self.testBindingSynapses()
        
    def setupCogMapRecording(self):
        if (nealParameters.simulator == 'spinnaker'):
                self.roomBindCells.record()
                self.shapeBindCells.record()
                self.stateCells.record()
                self.inShapeCells.record()
                self.nextRoomCells.record()
                self.beforeShapeCells.record()
                self.priorRoomCells.record()
                self.outShapeCells.record()
                self.signalStateCells.record()
        elif (nealParameters.simulator == 'nest'):
            if (not nealParameters.printSmall):
                self.roomBindCells.record({'spikes'})
                self.shapeBindCells.record({'spikes'})
                self.stateCells.record({'spikes'})
                self.nextRoomCells.record({'spikes'})
                self.beforeShapeCells.record({'spikes'})
                self.priorRoomCells.record({'spikes'})
                self.outShapeCells.record({'spikes'})
                self.signalStateCells.record({'spikes'})
            self.inShapeCells.record({'spikes'})

    def printTimers(self):
        if (not nealParameters.printSmall):
            self.exploreToWaitTimer.printResults("results/cmExploreWaitTimer")
            self.bindToStopBindTimer.printResults("results/cmBindStopBindTimer")
            self.stopToNextRoomTimer.printResults("results/cmStopNextTimer")
            self.nextRoomToWaitTimer.printResults("results/cmNextWaitTimer")
            self.priorToPriorRoomTimer.printResults("results/cmPriorPriorRoomTimer")

    def printCogMapNets(self):
        self.printTimers()
        if (nealParameters.simulator == 'spinnaker'):
                suffix = '.sp'
        elif (nealParameters.simulator == 'nest'):
                suffix = '.pkl'
        if (not nealParameters.printSmall):
            self.roomBindCells.printSpikes('results/cmRoomBind'+suffix)
            self.shapeBindCells.printSpikes('results/cmShapeBind'+suffix)
            self.stateCells.printSpikes('results/cmState'+suffix)
            self.inShapeCells.printSpikes('results/cmInShape'+suffix)
            self.nextRoomCells.printSpikes('results/cmNextRoom'+suffix)
            self.beforeShapeCells.printSpikes('results/cmBeforeShape'+suffix)
            self.priorRoomCells.printSpikes('results/cmPriorRoom'+suffix)
            self.outShapeCells.printSpikes('results/cmOutShape'+suffix)
            self.signalStateCells.printSpikes('results/cmSignalState'+suffix)
        self.inShapeCells.printSpikes('results/cmInShape'+suffix)

    def testMakeSpikeSource(self):
        inputSpikeTimes = [10.0]
        spikeArray = {'spike_times': [inputSpikeTimes]}
        spikeGen=sim.Population(1,sim.SpikeSourceArray,spikeArray,
                                   label='inputSpikes_0')
        return spikeGen
    
    #Turn on room and shapeBind.  Then turn off the binding.  Then
    #turn on the shape to see if the room comes on.
    def testBind(self,spikeSourceState,spikeSourceBind,spikeSourceStopBind,
                 spikeSourceTestBound,spikeSourceTestBound2):
        self.fsa.turnOnStateFromSpikeSource(spikeSourceState,
                self.stateCells,self.testBindState)
        self.fsa.turnOnStateFromSpikeSource(spikeSourceBind,self.roomBindCells,
                                            0)
        self.fsa.turnOnStateFromSpikeSource(spikeSourceBind,self.shapeBindCells,
                                            0)
        #roomBind turns on nextRoom, so prevent this for the test.
        for roomCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOffState(self.stateCells,self.testBindState,
                                        self.priorRoomCells,roomCANum)
        #shapeBind turns on outShape, so prevent this for the test.
        for shapeCANum in range (0,self.numberRooms): 
            self.fsa.stateTurnsOffState(self.stateCells,self.testBindState,
                                        self.outShapeCells,shapeCANum)

        #Once bound, turn off the state and the binds
        self.fsa.turnOnStateFromSpikeSource(spikeSourceStopBind,
                self.stateCells,self.testStopBindState)
        self.fsa.stateTurnsOffState(self.stateCells,self.testStopBindState,
                                        self.stateCells,self.testBindState)
        self.fsa.stateTurnsOffState(self.stateCells,self.testStopBindState,
                                    self.roomBindCells, 0)
        self.fsa.stateTurnsOffState(self.stateCells,self.testStopBindState,
                                    self.shapeBindCells, 0)

        #Then test the binding, by turning on the shape.
        self.fsa.turnOnStateFromSpikeSource(spikeSourceTestBound,
                self.stateCells,self.testBoundState)
        self.fsa.stateTurnsOffState(self.stateCells,self.testBoundState,
                                        self.stateCells,self.testStopBindState)
        
        self.fsa.turnOnStateFromSpikeSource(spikeSourceTestBound2,
                            self.shapeBindCells,0)


    def testCogMap(self):
        spikeGen = self.testMakeSpikeSource()
        self.testBind(spikeGen)

    #Use bogus states to test a single binding
    def testBindingSynapses(self):
        testTurnOff = 12
        testRest = 13
        self.stateTurnsOffAllRooms(testTurnOff)
        self.stateTurnsOffAllShapes(testTurnOff)

        self.fsa.stateTurnsOffState(self.stateCells,testRest,self.stateCells,
                                    testTurnOff)


    def setUpFullAndBindTest(self):
        self.fsa.stateHalfTurnsOnState(self.stateCells,self.testExpState,
                                       self.stateCells,self.exploreState)
        self.fsa.stateTurnsOffState(self.stateCells,self.exploreState,
                                    self.stateCells,self.testExpState)

#-- Access Functions
    def getSignalCells(self):
        return self.signalStateCells

    def getBeforeShapeCells(self):
        return self.beforeShapeCells

    def getStateCells(self):
        return self.stateCells

    def getOutShapeCells(self):
        return self.outShapeCells
