"""
Setup the cogmap by binding rooms to shapes.  Test these bindings by
presenting a room or a shape (change the room or shape in 
comapClass.testBind).  Change the timing in the call to makeSpikeSources.
You should see one cmRoomBind and one cmShapeBind turned on after the second
source time, then cmShapeBind is turned on after the fifth source and the
bound room should come on.  Aside from State, nothing else should come
on before the fifth source.
"""

#imports for PyNN
import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

from nealCoverClass import NealCoverFunctions
from cogmapClass import cogmapArea

def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def makeSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        inputSpikeTimes3 = inputTimes[2]
        inputSpikeTimes4 = inputTimes[3]
        inputSpikeTimes5 = inputTimes[4]
        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')
        spikeArray3 = {'spike_times': [inputSpikeTimes3]}
        spikeGen3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='inputSpikes_3')
        spikeArray4 = {'spike_times': [inputSpikeTimes4]}
        spikeGen4=sim.Population(1,sim.SpikeSourceArray,spikeArray4,
                                   label='inputSpikes_4')
        spikeArray5 = {'spike_times': [inputSpikeTimes5]}
        spikeGen5=sim.Population(1,sim.SpikeSourceArray,spikeArray5,
                                   label='inputSpikes_5')
        return [spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5]


#------------Main Body---------------
neal = NealCoverFunctions(nealParameters.simulator)
init()

spikeSources = makeSpikeSources([10.0,50.0,150.0,200,300.0])

cogmap = cogmapArea()
cogmap.testBind(spikeSources[0],spikeSources[1],spikeSources[2],spikeSources[3],
                spikeSources[4])

sim.run(400)

cogmap.printCogMapNets()
