"""
This turns the preExplore state on then switches to explore, then
waits for a shape that is never presented.
"""

#imports for PyNN
import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

from nealCoverClass import NealCoverFunctions
from cogmapClass import cogmapArea

def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
              min_delay=nealParameters.DELAY,
              max_delay=nealParameters.DELAY, debug=0)


def runTest(time):
    sim.run(time)

def makeSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')

        return [spikeGen1,spikeGen2]

def testCellsOn(cm):
        spikeSources = makeSpikeSources([10.0,110.0])

        #turn on state0 that turns off all shapes.
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[0],cm.stateCells,
                                          cm.preExploreState)#PreExplore
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[1],cm.stateCells,
                                          cm.testExpState)#testExplore



#------------Main Body---------------
neal = NealCoverFunctions(nealParameters.simulator)
init()

cogmap = cogmapArea()
cogmap.setUpFullAndBindTest()
testCellsOn(cogmap)

runTest(nealParameters.SIM_LENGTH)

cogmap.printCogMapNets()
