"""
This turns the preExplore state on then switches to explore, then
waits for a shape.  This is presented, and the system should slide
through the bind, stopBind and nextRoom states back to waitForShape.
It should also bind one shape to one room.  It should do this for 
all four room shape pairs.
"""

#imports for PyNN
import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

from nealCoverClass import NealCoverFunctions
from cogmapClass import cogmapArea

def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
              min_delay=nealParameters.DELAY,
              max_delay=nealParameters.DELAY, debug=0)

#set up the spike sources for external activation.
def makeSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        inputSpikeTimes3 = inputTimes[2] #first shape
        inputSpikeTimes4 = inputTimes[3]
        inputSpikeTimes5 = inputTimes[4]
        inputSpikeTimes6 = inputTimes[5] #fourth shape
        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')
        spikeArray3 = {'spike_times': [inputSpikeTimes3]}
        spikeGen3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='inputSpikes_3')
        spikeArray4 = {'spike_times': [inputSpikeTimes4]}
        spikeGen4=sim.Population(1,sim.SpikeSourceArray,spikeArray4,
                                   label='inputSpikes_4')
        spikeArray5 = {'spike_times': [inputSpikeTimes5]}
        spikeGen5=sim.Population(1,sim.SpikeSourceArray,spikeArray5,
                                   label='inputSpikes_5')
        spikeArray6 = {'spike_times': [inputSpikeTimes6]}
        spikeGen6=sim.Population(1,sim.SpikeSourceArray,spikeArray6,
                                   label='inputSpikes_6')


        return [spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,spikeGen6]

def testCellsOn(cm):
        spikeSources = makeSpikeSources([10.0,110.0, 400.0, 900.0, 1400.0, 
                                         1900.00])

        #turn on state0 that turns off all shapes.
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[0],cm.stateCells,0)#PreExplore
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[1],cm.stateCells,12)#testExplore
        #shapes
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[2],cm.inShapeCells,0)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[3],cm.inShapeCells,1)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[4],cm.inShapeCells,2)
        cm.fsa.turnOnStateFromSpikeSource(spikeSources[5],cm.inShapeCells,3)


#------------Main Body---------------
neal = NealCoverFunctions(nealParameters.simulator)
init()

cogmap = cogmapArea()
cogmap.setUpFullAndBindTest()
testCellsOn(cogmap)

sim.run(2500.0)

cogmap.printCogMapNets()
