"""
This plan class includes the primitives, context sensitive turn to object,
and compounds with a timer.  It's got goToDoor, IDObj, and GoThrough Door.
I'm putting in explore.
"""

import nealParams as nealParameters
#get the simulator
if  (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
    import spynnaker_external_devices_plugin.pyNN as ExternalDevices
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle
import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from timerClass import TimerClass


class PlanClass:
    NUMBER_GOALS = 15
    goalTurnRight = 0
    goalTurnLeft = 1
    goalStepForward = 2
    goalStepBackward = 3 
    goalMoveRight = 4 
    goalMoveLeft = 5
    goalTurnTowardObject = 6
    goalGoToDoor = 7
    goalExplore = 8
    goalIDObj = 9
    goalGoThroughDoor = 10
    goalMoveBeforeBluePyramid = 11
    goalMoveBeforeRedStal = 12
    goalMoveBeforeBlueStal = 13
    goalMoveBeforeRedPyramid = 14

    NUMBER_MODULES = 50
    moduleTurnRight = 0
    moduleTurnLeft = 1
    moduleStepForward = 2
    moduleStepBackward = 3
    moduleMoveRightRight = 4
    moduleMoveRightForward = 5
    moduleMoveLeftLeft = 6
    moduleMoveLeftForward = 7
    moduleGoToDoor = 8
    moduleTurnRightDoor = 9
    moduleStepForwardDoor = 10
    moduleTurnLeftDoor = 11
    moduleBackDoor = 12
    moduleInDoor = 13
    moduleIDObj  = 14
    moduleTurnLeftObj = 15
    moduleForwardObj = 16
    moduleTurnRightObj = 17
    moduleBackObj = 18
    moduleCantIDObj = 19
    moduleObjIdentified = 20
    moduleGoThroughDoor = 21
    moduleTurnLeftInDoor = 22
    moduleForwardInDoor = 23
    moduleTurnRightInDoor = 24
    moduleBackInDoor = 25
    moduleNoInDoorFound = 26
    moduleThroughDoor = 27
    moduleExplore = 28
    moduleExploreStart = 29
    moduleExploreIDObj = 30
    moduleExploreGoToDoor = 31
    moduleExploreGoThroughDoor = 32
    moduleExploreDone = 33
    moduleMoveBeforeBluePyramid = 34
    moduleMoveBeforeRedStal = 35
    moduleMoveBeforeBlueStal = 36
    moduleMoveBeforeRedPyramid = 37
    moduleMoveToBluePyramid = 38
    moduleMoveToRedStal = 39
    moduleMoveToBlueStal = 40
    moduleMoveToRedPyramid = 41
    moduleMovedToBluePyramid = 42
    moduleMovedToRedStal = 43
    moduleMovedToBlueStal = 44
    moduleMovedToRedPyramid = 45
    moduleMoveStart = 46
    moduleMoveIDObj = 47
    moduleMoveGoToDoor = 48
    moduleMoveGoThroughDoor = 49

    NUMBER_FACTS = 36
    bumpFact = 0
    objLeftFact = 1
    objCtrFact = 2
    objRightFact = 3
    doorLeftFact = 4
    doorCtrFact = 5
    doorRightFact = 6
    redPyrLeftFact = 7
    redPyrCtrFact = 8
    redPyrRightFact = 9
    bluePyrLeftFact = 10
    bluePyrCtrFact = 11
    bluePyrRightFact = 12
    redStalLeftFact = 13
    redStalCtrFact = 14
    redStalRightFact = 15
    blueStalLeftFact = 16
    blueStalCtrFact = 17
    blueStalRightFact = 18
    inDoorLeftFact = 19
    inDoorCtrFact = 20
    inDoorRightFact = 21
    bluePyrFact = 22
    redStalFact = 23
    blueStalFact = 24
    redPyrFact = 25
    exploreDoneFact = 26
    moveBeforeBluePyramidFact = 27
    moveBeforeRedStalFact = 28
    moveBeforeBlueStalFact = 29
    moveBeforeRedPyramidFact = 30
    moveToBluePyramidFact = 31
    moveToRedStalFact = 32
    moveToBlueStalFact = 33
    moveToRedPyramidFact = 34
    inFrontDoorFact = 35

    NUMBER_ACTIONS = 9
    actionRight = 0
    actionLeft = 1
    actionForward = 2
    actionBackward = 3
    actionObjectIDed = 4
    actionInDoor = 5
    actionThroughDoor = 6
    actionExploreDone = 7
    actionMovedToShape = 8

    def __init__(self, simName):
        self.simulator = simName
        self.neal = NealCoverFunctions(self.simulator)
        self.fsa = FSAHelperFunctions(self.simulator)

        planCells = self.createNeurons()
        self.goalCells = planCells[0]
        self.moduleCells =planCells[1]
        self.factCells =planCells[2]
        self.actionCells =planCells[3]
        self.resetCells =planCells[4]
        timerCells = self.createTimers()
        self.moveLeftTimer = timerCells[0]
        self.moveRightTimer = timerCells[1]
        self.goTestInDoorTimer = timerCells[2]
        self.goToDoorTimer = timerCells[3]
        self.iDObjTimer = timerCells[4]
        self.cantIDObjTimer = timerCells[5]
        self.goThroughDoorTimer = timerCells[6]
        self.noInDoorFoundTimer = timerCells[7]
        self.setupRecording()
        self.connectPlanner()

    #create the neurons for the goals and actions.
    def createNeurons(self):
        numberGoalCells = self.NUMBER_GOALS * self.fsa.CA_SIZE
        numberModuleCells = self.NUMBER_MODULES * self.fsa.CA_SIZE
        numberFactCells = self.NUMBER_FACTS * self.fsa.CA_SIZE
        numberActionCells = self.NUMBER_ACTIONS
        localGoalCells=sim.Population(numberGoalCells,sim.IF_cond_exp,
                    self.fsa.CELL_PARAMS)
        localModuleCells=sim.Population(numberModuleCells,sim.IF_cond_exp,
                    self.fsa.CELL_PARAMS)
        localFactCells=sim.Population(numberFactCells,sim.IF_cond_exp,
                    self.fsa.CELL_PARAMS)
        resetCells=sim.Population(self.fsa.CA_SIZE,sim.IF_cond_exp,
                    self.fsa.CELL_PARAMS)
        if (nealParameters.simulator == 'spinnaker') and (nealParameters.useSpikeServer):
                localActionCells=sim.Population(numberActionCells,
                    sim.IF_cond_exp, self.fsa.CELL_PARAMS,
                    label='actionFromBoard')
                ExternalDevices.activate_live_output_for(localActionCells,
                    database_notify_port_num=12346)

            
        else:
            localActionCells=sim.Population(numberActionCells,
                    sim.IF_cond_exp, self.fsa.CELL_PARAMS)

        return [localGoalCells,localModuleCells,localFactCells,localActionCells,resetCells]

    def createTimers(self):
        moveLeft = TimerClass(nealParameters.simulator,10)
        moveRight = TimerClass(nealParameters.simulator,10)
        goTestInDoor = TimerClass(nealParameters.simulator,10)
        goToDoor = TimerClass(nealParameters.simulator,20)
        iDObj = TimerClass(nealParameters.simulator,18) #sep5 15
        cantIDObj = TimerClass(nealParameters.simulator,15)
        goThroughDoor = TimerClass(nealParameters.simulator,10)
        noInDoorFound = TimerClass(nealParameters.simulator,10)
        #the timerClass sets recording
        moveLeft.makeStopTimerSynapses()
        moveRight.makeStopTimerSynapses()
        goTestInDoor.makeOneStateStopTimerSynapses()
        goToDoor.makeOneStateStopTimerSynapses()
        iDObj.makeOneStateStopTimerSynapses()
        cantIDObj.makeOneStateStopTimerSynapses()
        goThroughDoor.makeOneStateStopTimerSynapses()
        noInDoorFound.makeOneStateStopTimerSynapses()
        return [moveLeft,moveRight,goTestInDoor,goToDoor,iDObj,cantIDObj,
                goThroughDoor,noInDoorFound]

    def setupRecording(self):
        if (nealParameters.simulator=="nest"):
            if (not nealParameters.printSmall):
                self.goalCells.record({'spikes','v'})
                self.moduleCells.record({'spikes','v'})
                self.resetCells.record({'spikes','v'})

            self.factCells.record({'spikes','v'})
            self.actionCells.record({'spikes','v'})

        elif (nealParameters.simulator == 'spinnaker'):
            self.goalCells.record()
            self.moduleCells.record()
            self.factCells.record()
            self.actionCells.record()
            self.resetCells.record()

    def makeGoalCAs(self):
        for goalNumber  in range (0,self.NUMBER_GOALS):
            self.fsa.makeCA(self.goalCells,goalNumber)

    def makeModuleCAs(self):
        for moduleNumber  in range (0,self.NUMBER_MODULES):
            self.fsa.makeCA(self.moduleCells,moduleNumber)

    def makeFactCAs(self):
        for factNumber  in range (0,self.NUMBER_FACTS):
            self.fsa.makeCA(self.factCells,factNumber)

    #the first state starts the timer, the timer starts the second state
    def makeCompound(self,firstState,secondState,actionNum,timer):
        timer.stateHalfStartsTimer(self.moduleCells,firstState)
        timer.oneNeuronHalfStartsTimer(self.actionCells,actionNum)
        timer.timerPreventsState(self.moduleCells,firstState)
        timer.timerStartsState(self.moduleCells,secondState)
        
    def connectModules(self):
        self.makeCompound(self.moduleMoveRightRight,self.moduleMoveRightForward,
                          self.actionRight,self.moveRightTimer)
        #rightForward turns off rightRight
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleMoveRightForward,
                                    self.moduleCells,self.moduleMoveRightRight)
        self.makeCompound(self.moduleMoveLeftLeft,self.moduleMoveLeftForward,
                          self.actionLeft,self.moveLeftTimer)
        #leftForward turns off leftRight
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleMoveLeftForward,
                                    self.moduleCells,self.moduleMoveLeftLeft)

    def connectGoalsAndModules(self):
        #connect the primitives
        self.fsa.stateTurnsOnState(self.goalCells,self.goalTurnRight,
                                   self.moduleCells,self.moduleTurnRight)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleTurnRight,
                                    self.goalCells,self.goalTurnRight)
        self.fsa.stateTurnsOnState(self.goalCells,self.goalTurnLeft,
                                   self.moduleCells,self.moduleTurnLeft)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleTurnLeft,
                                    self.goalCells,self.goalTurnLeft)
        self.fsa.stateTurnsOnState(self.goalCells,self.goalStepForward,
                                   self.moduleCells,self.moduleStepForward)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleStepForward,
                                    self.goalCells,self.goalStepForward)
        self.fsa.stateTurnsOnState(self.goalCells,self.goalStepBackward,
                                   self.moduleCells,self.moduleStepBackward)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleStepBackward,
                                    self.goalCells,self.goalStepBackward)

        #compounds
        self.fsa.stateTurnsOnState(self.goalCells,self.goalMoveRight,
                                   self.moduleCells,self.moduleMoveRightRight)
        self.fsa.stateTurnsOffState(self.moduleCells,
                self.moduleMoveRightForward, self.goalCells,self.goalMoveRight)
        self.fsa.stateTurnsOnState(self.goalCells,self.goalMoveLeft,
                                   self.moduleCells,self.moduleMoveLeftLeft)
        self.fsa.stateTurnsOffState(self.moduleCells,
                self.moduleMoveLeftForward, self.goalCells,self.goalMoveLeft)

        #turn toward object
        self.fsa.stateHalfTurnsOnState(self.goalCells,self.goalTurnTowardObject,
                                       self.moduleCells,self.moduleTurnRightObj)
        self.fsa.stateHalfTurnsOnState(self.goalCells,self.goalTurnTowardObject,
                                       self.moduleCells,self.moduleTurnLeftObj)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleTurnRightObj, 
                                    self.goalCells,self.goalTurnTowardObject)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleTurnLeftObj, 
                                    self.goalCells,self.goalTurnTowardObject)
                            
    def connectMoveDoneAction(self):
        #turn on the action
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
            self.moduleMovedToBluePyramid,self.actionCells,
                                       self.actionMovedToShape)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
            self.moduleMovedToRedStal,self.actionCells,
                                       self.actionMovedToShape)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
            self.moduleMovedToBlueStal,self.actionCells,
                                       self.actionMovedToShape)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
            self.moduleMovedToRedPyramid,self.actionCells,
                                       self.actionMovedToShape)

        #action turns off top level goals
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.goalCells,self.goalMoveBeforeBluePyramid)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.goalCells,self.goalMoveBeforeRedStal)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.goalCells,self.goalMoveBeforeBlueStal)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.goalCells,self.goalMoveBeforeRedPyramid)

        #action turns off the subgoal that should be running
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.goalCells,self.goalIDObj)
        #and the next
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.goalCells,self.goalGoToDoor) 
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.moduleCells,self.moduleMoveGoToDoor) 

        #action turns off modules
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.moduleCells,self.moduleMoveToBluePyramid)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.moduleCells,self.moduleMovedToBluePyramid)

        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                    self.moduleCells,self.moduleMoveToRedStal)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.moduleCells,self.moduleMovedToRedStal)

        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                    self.moduleCells,self.moduleMoveToBlueStal)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.moduleCells,self.moduleMovedToBlueStal)

        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.moduleCells,self.moduleMoveToRedPyramid)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.moduleCells,self.moduleMovedToRedPyramid)

        #turns off the submodule that should be running
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                    self.actionMovedToShape,
                        self.moduleCells,self.moduleMoveIDObj)

        #move submodules are turned off by their Actions
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionObjectIDed,
                        self.moduleCells,self.moduleMoveIDObj)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionInDoor,
                        self.moduleCells,self.moduleMoveGoToDoor)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionThroughDoor,
                        self.moduleCells,self.moduleMoveGoThroughDoor)


    #The actions ObjectIDed, InDoor, ThroughDoor, and ExploreDone turn off 
    #goals too. Consider, should move those to their own function.
    def connectModulesAndActions(self):
        #primitives
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,self.moduleTurnRight,
                                       self.actionCells,self.actionRight)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionRight, 
                                        self.moduleCells,self.moduleTurnRight)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,self.moduleTurnLeft,
                                       self.actionCells,self.actionLeft)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionLeft, 
                                        self.moduleCells,self.moduleTurnLeft)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,self.moduleStepForward,
                                       self.actionCells,self.actionForward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionForward, 
                                        self.moduleCells,self.moduleStepForward)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,self.moduleStepBackward,
                                       self.actionCells,self.actionBackward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionBackward, 
                                    self.moduleCells,self.moduleStepBackward)

        #compounds
        oneHalfWt = 0.05
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                    self.moduleMoveRightRight,self.actionCells,self.actionRight)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionRight,
                    self.moduleCells,self.moduleMoveRightRight)      
        self.fsa.oneNeuronStimulatesState(self.actionCells,self.actionRight,
                    self.moduleCells,self.moduleMoveRightForward,oneHalfWt)     

        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                self.moduleMoveRightForward,self.actionCells,self.actionForward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionForward,
            self.moduleCells,self.moduleMoveRightForward)      

        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                    self.moduleMoveLeftLeft,self.actionCells,self.actionLeft)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionLeft,
                    self.moduleCells,self.moduleMoveLeftLeft)      
        self.fsa.oneNeuronStimulatesState(self.actionCells,self.actionLeft,
                    self.moduleCells,self.moduleMoveLeftForward,oneHalfWt)      
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                self.moduleMoveLeftForward,self.actionCells,self.actionForward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionForward,
            self.moduleCells,self.moduleMoveLeftForward)      

        #goToDoor
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                    self.moduleTurnLeftDoor,self.actionCells,self.actionLeft)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionLeft, 
                self.moduleCells,self.moduleTurnLeftDoor)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
            self.moduleStepForwardDoor,self.actionCells,self.actionForward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionForward, 
                self.moduleCells,self.moduleStepForwardDoor)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                    self.moduleTurnRightDoor,self.actionCells,self.actionRight)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionRight, 
                self.moduleCells,self.moduleTurnRightDoor)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,self.moduleBackDoor,
                                       self.actionCells,self.actionBackward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionBackward, 
                self.moduleCells,self.moduleBackDoor)

        #IDObj
        #left
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                    self.moduleTurnLeftObj,self.actionCells,self.actionLeft)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionLeft, 
                self.moduleCells,self.moduleTurnLeftObj)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionLeft, 
                self.moduleCells,self.moduleCantIDObj)
        #ctr
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                    self.moduleForwardObj,self.actionCells,self.actionForward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionForward, 
                self.moduleCells,self.moduleForwardObj)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionForward, 
                self.moduleCells,self.moduleCantIDObj)
        #right
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                    self.moduleTurnRightObj,self.actionCells,self.actionRight)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionRight, 
                self.moduleCells,self.moduleTurnRightObj)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionRight, 
                self.moduleCells,self.moduleCantIDObj)
        #back
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                        self.moduleBackObj,self.actionCells,self.actionBackward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionBackward, 
                self.moduleCells,self.moduleBackObj)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionBackward, 
                self.moduleCells,self.moduleCantIDObj)

        #foundDoor
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                    self.moduleInDoor,self.actionCells,self.actionInDoor)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionInDoor, 
                            self.moduleCells,self.moduleGoToDoor)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionInDoor, 
                            self.moduleCells,self.moduleInDoor)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionInDoor, 
                self.goalCells,self.goalGoToDoor)
        #foundObject IdObj
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                self.moduleObjIdentified,self.actionCells,self.actionObjectIDed)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionObjectIDed, 
                self.moduleCells,self.moduleIDObj)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionObjectIDed, 
                self.moduleCells,self.moduleObjIdentified)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionObjectIDed,
                self.goalCells,self.goalIDObj)

        #goThroughDoor
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                    self.moduleTurnLeftInDoor,self.actionCells,self.actionLeft)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionLeft, 
                self.moduleCells,self.moduleTurnLeftInDoor)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                self.moduleForwardInDoor,self.actionCells,self.actionForward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionForward, 
                self.moduleCells,self.moduleForwardInDoor)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionForward, 
                self.moduleCells,self.moduleNoInDoorFound)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                self.moduleTurnRightInDoor,self.actionCells,self.actionRight)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionRight, 
                self.moduleCells,self.moduleTurnRightInDoor)
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                self.moduleBackInDoor,self.actionCells,self.actionBackward)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionBackward, 
                self.moduleCells,self.moduleBackInDoor)
        #goThroughDoor done
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,
                self.moduleThroughDoor,self.actionCells,self.actionThroughDoor)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
                self.actionThroughDoor,self.moduleCells,self.moduleThroughDoor)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
            self.actionThroughDoor,self.moduleCells,self.moduleNoInDoorFound)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
            self.actionThroughDoor,self.moduleCells,self.moduleGoThroughDoor)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,
            self.actionThroughDoor,self.goalCells,self.goalGoThroughDoor)

        #explore submodules are turned off by their Actions
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionObjectIDed,
                        self.moduleCells,self.moduleExploreIDObj)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionInDoor,
                    self.moduleCells,self.moduleExploreGoToDoor)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionThroughDoor,
                        self.moduleCells,self.moduleExploreGoThroughDoor)

        #moduleExploreDone turns on actionExploreDone
        self.fsa.stateTurnsOnOneNeuron(self.moduleCells,self.moduleExploreDone,
                                       self.actionCells,self.actionExploreDone)
        #exploreDone action turns off explore modules and Goals
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionExploreDone,
                        self.moduleCells,self.moduleExploreIDObj)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionExploreDone,
                        self.moduleCells,self.moduleExplore)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionExploreDone,
                        self.moduleCells,self.moduleExploreDone)
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionExploreDone,
                self.moduleCells,self.moduleExploreGoToDoor) #The next module
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionExploreDone,
                self.moduleCells,self.moduleGoToDoor) 
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionExploreDone,
                        self.goalCells,self.goalIDObj) 
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionExploreDone,
                        self.goalCells,self.goalGoToDoor) #The next one goal
        self.fsa.oneNeuronTurnsOffState(self.actionCells,self.actionExploreDone,
                        self.goalCells,self.goalExplore)

        self.connectMoveDoneAction()


    def factPreventsFact(self,fromCA,toCA):
        self.fsa.stateTurnsOffState(self.factCells,fromCA,self.factCells,toCA)

    def factsPreventFacts(self):
        #objects prevent Other Objects
        self.factPreventsFact(self.objLeftFact,self.objCtrFact)  
        self.factPreventsFact(self.objLeftFact,self.objRightFact) 
        self.factPreventsFact(self.objCtrFact,self.objLeftFact) 
        self.factPreventsFact(self.objCtrFact,self.objRightFact) 
        self.factPreventsFact(self.objRightFact,self.objLeftFact) 
        self.factPreventsFact(self.objRightFact,self.objCtrFact) 

        #indoors prevent indoors
        if (nealParameters.simulator=="spinnaker"):
            inhibWt = 0.05
        elif (nealParameters.simulator=="nest"):
            inhibWt = -0.05  
        self.fsa.stateInhibitsState(self.factCells,self.inDoorLeftFact,
                                    self.factCells,self.inDoorCtrFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.inDoorLeftFact,
                                    self.factCells,self.inDoorRightFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.inDoorCtrFact,
                                    self.factCells,self.inDoorLeftFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.inDoorCtrFact,
                                    self.factCells,self.inDoorRightFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.inDoorRightFact,
                                    self.factCells,self.inDoorLeftFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.inDoorRightFact,
                                    self.factCells,self.inDoorCtrFact,inhibWt)

        #doors prevent other doors
        #aug30
        self.fsa.stateInhibitsState(self.factCells,self.doorLeftFact,
                                    self.factCells,self.doorCtrFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.doorLeftFact,
                                    self.factCells,self.doorRightFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.doorCtrFact,
                                    self.factCells,self.doorLeftFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.doorCtrFact,
                                    self.factCells,self.doorRightFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.doorRightFact,
                                    self.factCells,self.doorLeftFact,inhibWt)
        self.fsa.stateInhibitsState(self.factCells,self.doorRightFact,
                                    self.factCells,self.doorCtrFact,inhibWt)

        #Make all the red;blue;pyramid,stalactite; left,ctr,right
        #mutually exclusive
        for fromObj in range (self.redPyrLeftFact,self.blueStalRightFact): 
            for toObj in range (self.redPyrLeftFact,self.blueStalRightFact): 
                if (fromObj != toObj):
                    self.factPreventsFact(fromObj,toObj)


    def factPrimesFact(self,fromCA,toCA,preventWeight):
        connector = []
        for fromOffset in range (0,self.fsa.CA_SIZE):
            fromNeuron = fromOffset + (fromCA*self.fsa.CA_SIZE)
            for toOffset in range (0,self.fsa.CA_SIZE):
                toNeuron = toOffset + (toCA*self.fsa.CA_SIZE)
                connector = connector + [(fromNeuron,toNeuron,preventWeight,
                                      nealParameters.DELAY)]
        self.neal.nealProjection(self.factCells,self.factCells,connector,
                                 'excitatory')


    def factsPrimeFacts(self):
        #undone these object primings are all wrong.  5 is not objleft nor
        #11 redPyrLeft.  Try without this call.
        #if (nealParameters.simulator=="spinnaker"):
        objToPyrWeight = 0.000001 #objectToPyramids (and Stalactites) jul5 .00004
        #objects prime location related objects
        #obj to redPyr
        self.factPrimesFact(5,11,objToPyrWeight) #objLeft primes redPyrLeft
        self.factPrimesFact(6,12,objToPyrWeight) #objCentre primes redPyrCentre
        self.factPrimesFact(7,13,objToPyrWeight) #objRight primes redPyrRight
        #obj to bluePyr 
        self.factPrimesFact(5,14,objToPyrWeight) #objLeft primes bluePyrLeft
        self.factPrimesFact(6,15,objToPyrWeight) #objCentre primes bluePyrCentre
        self.factPrimesFact(7,16,objToPyrWeight) #objRight primes bluePyrRight
        #obj to redStal
        self.factPrimesFact(5,17,objToPyrWeight) #objLeft primes redStalLeft
        self.factPrimesFact(6,18,objToPyrWeight) #objCentre primes redStalCentre
        self.factPrimesFact(7,19,objToPyrWeight) #objRight primes redStalRight
        #obj to blueStal
        self.factPrimesFact(5,20,objToPyrWeight) #objLeft primes blueStalLeft
        self.factPrimesFact(6,21,objToPyrWeight) #objCentre primes blueStalCntre
        self.factPrimesFact(7,22,objToPyrWeight) #objRight primes blueStalRight

    def posShapeFactsStartShapeFacts(self):
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.bluePyrLeftFact,
                                   self.factCells,self.bluePyrFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.bluePyrCtrFact,
                                   self.factCells,self.bluePyrFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.bluePyrRightFact,
                                   self.factCells,self.bluePyrFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.blueStalLeftFact,
                                   self.factCells,self.blueStalFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.blueStalCtrFact,
                                   self.factCells,self.blueStalFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.blueStalRightFact,
                                   self.factCells,self.blueStalFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.redPyrLeftFact,
                                   self.factCells,self.redPyrFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.redPyrCtrFact,
                                   self.factCells,self.redPyrFact) 
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.redPyrRightFact,
                                   self.factCells,self.redPyrFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.redStalLeftFact,
                                   self.factCells,self.redStalFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.redStalCtrFact,
                                   self.factCells,self.redStalFact)
        self.fsa.stateTurnsOnStateSlow(self.factCells,self.redStalRightFact,
                                   self.factCells,self.redStalFact)

    def connectFacts(self):
        self.posShapeFactsStartShapeFacts()
        self.factsPreventFacts()
        self.factsPrimeFacts()

    def actionCausesReset(self):
        #the reset isn't really a CA but it's that size.  Any action
        #should turn it on. 
        for actionNumber  in range (0,self.NUMBER_ACTIONS):
            self.fsa.turnOnStateFromOneNeuron(self.actionCells,actionNumber,
                                              self.resetCells,0)

    #A reset turns all the facts off (they can be turned on again)
    def resetStopsFacts(self):
        for factNumber  in range (0,self.NUMBER_FACTS):
            self.fsa.stateTurnsOffState(self.resetCells,0,
                                        self.factCells,factNumber)

    #goal keeps module on.
    #Use the goTestInDoorTimer first to test if we're in the door.  
    #Then start the goToDoorTimer.
    #if door right, turn right
    #if door left, turn left
    #if door ctr, move forward
    #if door bump, move backward
    #if no door turn right (via timer)
    #stop via moduleInDoor, when inFrontDoor
    def connectGoToDoor(self):
        self.fsa.stateTurnsOnStateSlow(self.goalCells,self.goalGoToDoor,
                                   self.moduleCells,self.moduleGoToDoor)

        #module and doorLt fact -> module left
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleGoToDoor,
            self.moduleCells,self.moduleTurnLeftDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.doorLeftFact,
            self.moduleCells,self.moduleTurnLeftDoor)

        #module and doorRt fact -> module right
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleGoToDoor,
            self.moduleCells,self.moduleTurnRightDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.doorRightFact,
            self.moduleCells,self.moduleTurnRightDoor)

        #module and doorCtr fact ->  module forward
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleGoToDoor,
                                    self.moduleCells,self.moduleStepForwardDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.doorCtrFact,
                                self.moduleCells,self.moduleStepForwardDoor)

        #module and bumpFact ->  moduleBackDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleGoToDoor,
                                    self.moduleCells,self.moduleBackDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.bumpFact,
                                self.moduleCells,self.moduleBackDoor)

        #module start the testInDoorTimer,testTimer start goToDoorTimer.
        #goToDoor prevents testTimer
        self.goTestInDoorTimer.stateStartsTimer(self.moduleCells,
                                                self.moduleGoToDoor)
        self.goTestInDoorTimer.timerStartsState(self.goToDoorTimer.timerCells,0)
        self.goToDoorTimer.timerPreventsState(self.goTestInDoorTimer.timerCells,
                                              0)

        #module and timer  -> turn right (default)
        self.goToDoorTimer.timerStartsState(self.moduleCells,
                                            self.moduleTurnRightDoor)


        #goTestInDoorTimer stimulates inDoorFact
        if (nealParameters.simulator=="spinnaker"):
            timerToInFrontDoorFactWt = 0.0006 #0.0008 turns inFrontDoor on
        elif (nealParameters.simulator=="nest"):
            timerToInFrontDoorFactWt = 0.0004 
        self.goTestInDoorTimer.timerActivatesState(self.factCells, 
                            self.inFrontDoorFact, timerToInFrontDoorFactWt)

        #doorTimer  primes doorLeft, doorCt,  and doorRightFact 
        timerToDoorPosFactWt = 0.0008
        self.goToDoorTimer.timerActivatesState(self.factCells, 
                            self.doorLeftFact, timerToDoorPosFactWt)
        self.goToDoorTimer.timerActivatesState(self.factCells, 
                            self.doorCtrFact, timerToDoorPosFactWt)
        self.goToDoorTimer.timerActivatesState(self.factCells, 
                            self.doorRightFact, timerToDoorPosFactWt)


        #reset also needs to stop the timers.
        self.goToDoorTimer.stateStopsTimer(self.resetCells,0)
        self.goTestInDoorTimer.stateStopsTimer(self.resetCells,0)

        #module goDoor and (inFrontDoor) -> moduleInDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleGoToDoor,
            self.moduleCells,self.moduleInDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.inFrontDoorFact,
            self.moduleCells,self.moduleInDoor)
        
        self.goToDoorTimer.stateStopsTimer(self.moduleCells,self.moduleInDoor)
        self.goTestInDoorTimer.stateStopsTimer(self.moduleCells,
                                               self.moduleInDoor)
 
    #goalIDObj -slow-> moduleIDObj
    #try to ID with IDObjtimer
    #If can't timer turns on moduleCantIDObj
    #modCantID + objLeft -> modTurnLeftObj
    #modCantID + objCtr -> modForwardObj
    #modCantID + objRight -> modTurnRightObj
    #default via cantIDObjTimer -> modTurnRight
    #moduleIDObj + shape -> moduleObjIdentified 
    def connectIDObj(self):
        #start the module from the goal
        self.fsa.stateTurnsOnStateSlow(self.goalCells,self.goalIDObj,
                                   self.moduleCells,self.moduleIDObj)

        #three modules ObjIdentified, IDObj, and CantIDObj.  
        #Cant stops ID
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleCantIDObj,
                                    self.moduleCells,self.moduleIDObj)

        #moduleID and IDObjTimer  -> modCantID
        self.iDObjTimer.stateStartsTimer(self.moduleCells,
                                            self.moduleIDObj)
        self.iDObjTimer.timerStartsState(self.moduleCells,
                                            self.moduleCantIDObj)

        #iDObjTimer primes objLeft, objCt and objRightFact
        timerToObjPosFactWt = 0.001 #.002 goes on its own
        self.iDObjTimer.timerActivatesState(self.factCells, self.objLeftFact,
                                            timerToObjPosFactWt)
        self.iDObjTimer.timerActivatesState(self.factCells, self.objCtrFact,
                                            timerToObjPosFactWt)
        self.iDObjTimer.timerActivatesState(self.factCells, self.objRightFact,
                                            timerToObjPosFactWt)
        if (nealParameters.simulator=="spinnaker"):
            timerToShapeColourPosWt = 0.0003
        elif (nealParameters.simulator=="nest"):
            timerToShapeColourPosWt = 0.00045
        for obj in range (self.redPyrLeftFact, self.blueStalRightFact):
            self.iDObjTimer.timerActivatesState(self.factCells, obj,
                                            timerToShapeColourPosWt)

        #modCantID and doorLt fact -> modObjLeft
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleCantIDObj,
            self.moduleCells,self.moduleTurnLeftObj)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.objLeftFact,
            self.moduleCells,self.moduleTurnLeftObj)

        #modCantID and objCtr fact -> modForwardObj
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleCantIDObj,
            self.moduleCells,self.moduleForwardObj)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.objCtrFact,
            self.moduleCells,self.moduleForwardObj)

        #modCantID and objRightfact -> module right
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleCantIDObj,
            self.moduleCells,self.moduleTurnRightObj)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.objRightFact,
            self.moduleCells,self.moduleTurnRightObj)


        #modCantID and bump -> moduleBackObj
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleCantIDObj,
            self.moduleCells,self.moduleBackObj)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.bumpFact,
            self.moduleCells,self.moduleBackObj)

        #Default with timer
        #moduleID and IDObjTimer  -> modCantID
        self.cantIDObjTimer.stateStartsTimer(self.moduleCells,
                                            self.moduleCantIDObj)
        self.cantIDObjTimer.timerStartsState(self.moduleCells,
                                            self.moduleTurnRightObj)
        
        #if you can't id the obj, still try to get the objPosFact going.
        self.cantIDObjTimer.timerActivatesState(self.factCells, self.objLeftFact,
                                            timerToObjPosFactWt)
        self.cantIDObjTimer.timerActivatesState(self.factCells, self.objCtrFact,
                                            timerToObjPosFactWt)
        self.cantIDObjTimer.timerActivatesState(self.factCells, self.objRightFact,
                                            timerToObjPosFactWt)


        #modID and shapeFact -> module IDed
        #during explore in cabot3, it's also turned on by a cmShapeIn
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleIDObj,
                    self.moduleCells,self.moduleObjIdentified)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.bluePyrFact,
                    self.moduleCells,self.moduleObjIdentified)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.blueStalFact,
                    self.moduleCells,self.moduleObjIdentified)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.redPyrFact,
                    self.moduleCells,self.moduleObjIdentified)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.redStalFact,
                    self.moduleCells,self.moduleObjIdentified)
        
        #cantID and shapeFact -> module IDed (in case the shapefact comes on
        #after it's switched to cantID).
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleCantIDObj,
                    self.moduleCells,self.moduleObjIdentified)
        
        #When the module stops the goal, stop the idobj and cantIDtimers
        self.iDObjTimer.stateStopsTimer(self.moduleCells,
                                        self.moduleObjIdentified) 
        self.cantIDObjTimer.stateStopsTimer(self.moduleCells,
                                        self.moduleObjIdentified) 

        #reset stops timer
        self.cantIDObjTimer.stateStopsTimer(self.resetCells,0)

    #goalThroughDoor --slow--> moduleGoThroughDoor
    #goThroughDoorTimer primes inDoorLeft,ctr and right facts.
    #moduleGoThroughDoor + inDoorLeftFact -> moduleTurnLeftInDoor
    #moduleGoThroughDoor + inDoorCtFact -> moduleForwardInDoor
    #moduleGoThroughDoor + inDoorRightFact -> moduleTurnRightInDoor
    #moduleGoThroughDoor + bumpFact -> moduleBackInDoor
    #moduleGoThroughDoor + objCtrFact Stops Goal (moduleThroughDoor)
    #moduleGoThroughDoor + objLeftFact Stops Goal (moduleThroughDoor)
    #moduleGoThroughDoor + objRightFact Stops Goal (moduleThroughDoor)
    #goThroughDoorTimer ->  moduleNoInDoorFound
    #moduleNoDoorFound -> noDoorFoundTimer
    #noInDoorFoundTimer primes objCtrFact, 
    #noInDoorFoundTimer defaults to forward.
    def connectGoThroughDoor(self):
        #start the module from the goal
        self.fsa.stateTurnsOnStateSlow(self.goalCells,self.goalGoThroughDoor,
                            self.moduleCells,self.moduleGoThroughDoor)

        #moduleGoThroughDoor starts Timer
        self.goThroughDoorTimer.stateStartsTimer(self.moduleCells,
                                            self.moduleGoThroughDoor)

        #if no in door is found the moduleNoInDoorFound is set which starts,
        #the noindoorfound timer
        self.goThroughDoorTimer.timerStartsState(self.moduleCells, 
                    self.moduleNoInDoorFound)  #should be half starts state
        self.noInDoorFoundTimer.stateStartsTimer(self.moduleCells,
                                               self.moduleNoInDoorFound)

        self.noInDoorFoundTimer.timerStartsState(self.moduleCells, 
                    self.moduleForwardInDoor)  #should be half starts state

        #reset stops timer
        self.goThroughDoorTimer.stateStopsTimer(self.resetCells,0)
        self.noInDoorFoundTimer.stateStopsTimer(self.resetCells,0)

        #goThroughDoorTimer primes inDoorjLeft, inDoorCtr and inDoorRightFact
        if (nealParameters.simulator=="spinnaker"):
            timerToInDoorFactWt = 0.001
        elif (nealParameters.simulator=="nest"):
            timerToInDoorFactWt = 0.0005 
        self.goThroughDoorTimer.timerActivatesState(self.factCells, 
                                self.inDoorLeftFact,timerToInDoorFactWt)
        self.goThroughDoorTimer.timerActivatesState(self.factCells, 
                                self.inDoorCtrFact,timerToInDoorFactWt)
        self.goThroughDoorTimer.timerActivatesState(self.factCells, 
                                self.inDoorRightFact,timerToInDoorFactWt)

        #noInDoorFoundTimer primes objCtr
        if (nealParameters.simulator=="spinnaker"):
            timerToObjCtrFactWt = 0.0004 #sep 18 0.0007 
        if (nealParameters.simulator=="nest"):
            timerToObjCtrFactWt = 0.0005
        self.noInDoorFoundTimer.timerActivatesState(self.factCells, 
                                self.objLeftFact,timerToObjCtrFactWt)
        self.noInDoorFoundTimer.timerActivatesState(self.factCells, 
                                self.objCtrFact,timerToObjCtrFactWt)
        self.noInDoorFoundTimer.timerActivatesState(self.factCells, 
                                self.objRightFact,timerToObjCtrFactWt)


        #modGoThroughDoor and InDoorLt fact -> modLeftInDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleGoThroughDoor,self.moduleCells,self.moduleTurnLeftInDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.inDoorLeftFact,
            self.moduleCells,self.moduleTurnLeftInDoor)

        #modGoThroughDoor and InDoorCtrFact -> modForwardInDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleGoThroughDoor,self.moduleCells,self.moduleForwardInDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.inDoorCtrFact,
            self.moduleCells,self.moduleForwardInDoor)

        #modGoThroughDoor and InDoorRightFact -> modTurnRightInDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleGoThroughDoor,self.moduleCells,self.moduleTurnRightInDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.inDoorRightFact,
            self.moduleCells,self.moduleTurnRightInDoor)

        #modGoThroughDoor and bumpFact -> modBackInDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleGoThroughDoor,self.moduleCells,self.moduleBackInDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.bumpFact,
            self.moduleCells,self.moduleBackInDoor)

        #modGoThroughDoor and ctrObjFact -> modThroughDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleGoThroughDoor,self.moduleCells,self.moduleThroughDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.objLeftFact,
            self.moduleCells,self.moduleThroughDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.objCtrFact,
            self.moduleCells,self.moduleThroughDoor)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.objRightFact,
            self.moduleCells,self.moduleThroughDoor)

    #Reuse the idobj, gotodoor and goThroughdoor goals; the subgoals
    #It's run through modules with the things that turn off subgoals
    #pushing on to the next module.
    #Finish with exploreDoneFact, moduleExploreDone, and actionExploreDone
    def connectExplore(self):
        #start the explore module which stays on until the goal is met 
        self.fsa.stateTurnsOnStateSlow(self.goalCells,self.goalExplore,
                                       self.moduleCells,self.moduleExplore)
        #modExploreStart comes on first and is used to set exploreIDon.
        #It should not come on again.
        self.fsa.stateTurnsOnStateSlow(self.goalCells,self.goalExplore,
                            self.moduleCells,self.moduleExploreStart)

        if (nealParameters.simulator == 'spinnaker'):
            inhibExploreStartWt = 0.1
        elif (nealParameters.simulator == 'nest'):
            inhibExploreStartWt = -0.1
        #moduleExplore + moduleExploreStart -> moduleExpIDObj
        #the submodules(meio,megtd,megthd) prevent exploreStart coming on again.
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleExplore,
                                       self.moduleCells,self.moduleExploreIDObj)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleExploreStart,self.moduleCells,self.moduleExploreIDObj)
        self.fsa.stateInhibitsState(self.moduleCells,self.moduleExploreIDObj,
                                    self.moduleCells,self.moduleExploreStart,
                                    inhibExploreStartWt)

        #moduleExplore + modObjIDed -> moduleExpGoToDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleExplore,
                                    self.moduleCells,self.moduleExploreGoToDoor)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
          self.moduleObjIdentified,self.moduleCells,self.moduleExploreGoToDoor)
        self.fsa.stateInhibitsState(self.moduleCells,self.moduleExploreGoToDoor,
                                    self.moduleCells,self.moduleExploreStart,
                                    inhibExploreStartWt)

        #moduleExplore + modInDoor -> moduleExpGoThroughDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleExplore,
                    self.moduleCells,self.moduleExploreGoThroughDoor)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
          self.moduleInDoor,self.moduleCells,self.moduleExploreGoThroughDoor)
        self.fsa.stateInhibitsState(self.moduleCells,
                self.moduleExploreGoThroughDoor,self.moduleCells,
                self.moduleExploreStart,inhibExploreStartWt)

        #moduleExplore + modThroughDoor -> moduleExpIDObj
        #modExp and preventing expStart done early in this function
        self.fsa.stateHalfTurnsOnState(self.moduleCells,self.moduleThroughDoor,
                        self.moduleCells,self.moduleExploreIDObj)

        #The explore submodules turn on their associated goals.
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
            self.moduleExploreIDObj,self.goalCells,self.goalIDObj)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
            self.moduleExploreGoToDoor,self.goalCells,self.goalGoToDoor)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
            self.moduleExploreGoThroughDoor,self.goalCells,
                                       self.goalGoThroughDoor)

        #The subgoal terminating actions turn off the moduleExpSubgoals 

        #modExplore and exploreDoneFact -> modExploreDone 
        self.fsa.stateHalfTurnsOnState(self.moduleCells, self.moduleExplore,
                                       self.moduleCells,self.moduleExploreDone)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.exploreDoneFact,
            self.moduleCells,self.moduleExploreDone)

    #First the planner needs to ask the cogmap what to move to.
    #It does this by having a moveBeforeColourShapeFact, and gets back
    #the moveToColourShapeFact.
    #Once you have the moduleMoveToShape, do what you do with explore
    #except turning off
    #Turn Off 
    def connectMoveBefore(self):
        #goalBeforeShapes turnOn moduleMoveBeforeShapes
        self.fsa.stateTurnsOnStateSlow(self.goalCells,
                            self.goalMoveBeforeBluePyramid,self.moduleCells,
                            self.moduleMoveBeforeBluePyramid)
        self.fsa.stateTurnsOnStateSlow(self.goalCells,
                            self.goalMoveBeforeRedStal,self.moduleCells,
                            self.moduleMoveBeforeRedStal)
        self.fsa.stateTurnsOnStateSlow(self.goalCells,
                            self.goalMoveBeforeBlueStal,self.moduleCells,
                            self.moduleMoveBeforeBlueStal)
        self.fsa.stateTurnsOnStateSlow(self.goalCells,
                            self.goalMoveBeforeRedPyramid,self.moduleCells,
                            self.moduleMoveBeforeRedPyramid)

        #moduleMoveBeforeShapes turn on moveBeforeShapeFacts
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
                    self.moduleMoveBeforeBluePyramid,self.factCells,
                                       self.moveBeforeBluePyramidFact)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
                    self.moduleMoveBeforeRedStal,self.factCells,
                                       self.moveBeforeRedStalFact)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
                    self.moduleMoveBeforeBlueStal,self.factCells,
                                       self.moveBeforeBlueStalFact)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
                    self.moduleMoveBeforeRedPyramid,self.factCells,
                                       self.moveBeforeRedPyramidFact)

        #If you're looking at this in performance order, this should now
        #ask the cogmap what is before that shape fact.
        
        #There are four moveToShapeModules all stimulated by the 
        #moveBeforeShapeModules and the moveToShapeFacts.
        #Cant move to yourself
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeBluePyramid,
                        self.moduleCells,self.moduleMoveToRedStal)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeBluePyramid,
                        self.moduleCells,self.moduleMoveToBlueStal)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeBluePyramid,
                        self.moduleCells,self.moduleMoveToRedPyramid)
        #before RedStal
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeRedStal,self.moduleCells,
                                       self.moduleMoveToBluePyramid)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeRedStal,
                        self.moduleCells,self.moduleMoveToBlueStal)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeRedStal,
                        self.moduleCells,self.moduleMoveToRedPyramid)
        #before BlueStal
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeBlueStal,self.moduleCells,
                                       self.moduleMoveToBluePyramid)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeBlueStal,
                        self.moduleCells,self.moduleMoveToRedStal)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeBlueStal,
                        self.moduleCells,self.moduleMoveToRedPyramid)
        #before RedPyramid
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeRedPyramid,
                        self.moduleCells,self.moduleMoveToBluePyramid)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeRedPyramid,
                        self.moduleCells,self.moduleMoveToRedStal)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                        self.moduleMoveBeforeRedPyramid,
                        self.moduleCells,self.moduleMoveToBlueStal)

        #the moveToShapeFacts turn on the moveToShapeModules
        self.fsa.stateHalfTurnsOnState(self.factCells,
                                self.moveToBluePyramidFact,self.moduleCells,
                                       self.moduleMoveToBluePyramid)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.moveToRedStalFact,
                            self.moduleCells,self.moduleMoveToRedStal)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.moveToBlueStalFact,
                            self.moduleCells,self.moduleMoveToBlueStal)
        self.fsa.stateHalfTurnsOnState(self.factCells,self.moveToRedPyramidFact,
                            self.moduleCells,self.moduleMoveToRedPyramid)

        #moveToShapeModules turn off moveBeforeShapeModules
        #and prevent them until the goal is finished
        self.fsa.stateTurnsOffState(self.moduleCells,
                        self.moduleMoveToBluePyramid,self.moduleCells,
                                    self.moduleMoveBeforeRedStal)
        self.fsa.stateTurnsOffState(self.moduleCells,
                        self.moduleMoveToBluePyramid,self.moduleCells,
                                    self.moduleMoveBeforeBlueStal)
        self.fsa.stateTurnsOffState(self.moduleCells,
                        self.moduleMoveToBluePyramid,self.moduleCells,
                                    self.moduleMoveBeforeRedPyramid)
        #red stals
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleMoveToRedStal,
                    self.moduleCells,self.moduleMoveBeforeBluePyramid)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleMoveToRedStal,
                    self.moduleCells,self.moduleMoveBeforeBlueStal)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleMoveToRedStal,
                    self.moduleCells,self.moduleMoveBeforeRedPyramid)
        #blue stal 
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleMoveToBlueStal,
                    self.moduleCells,self.moduleMoveBeforeBluePyramid)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleMoveToBlueStal,
                    self.moduleCells,self.moduleMoveBeforeRedStal)
        self.fsa.stateTurnsOffState(self.moduleCells,self.moduleMoveToBlueStal,
                    self.moduleCells,self.moduleMoveBeforeRedPyramid)
        #red pyramid
        self.fsa.stateTurnsOffState(self.moduleCells,
                        self.moduleMoveToRedPyramid,self.moduleCells,
                                    self.moduleMoveBeforeBluePyramid)
        self.fsa.stateTurnsOffState(self.moduleCells,
                        self.moduleMoveToRedPyramid,self.moduleCells,
                                    self.moduleMoveBeforeRedStal)
        self.fsa.stateTurnsOffState(self.moduleCells,
                        self.moduleMoveToRedPyramid,self.moduleCells,
                                    self.moduleMoveBeforeBlueStal)

        if (nealParameters.simulator == 'spinnaker'):
            inhibMoveStartWt = 0.1
        elif (nealParameters.simulator == 'nest'):
            inhibMoveStartWt = -0.1
        #Do the search
        #largely duplicate the explore from connectexplore
        #use four modules; moduleMoveStart,moduleMoveIDObj, moduleMoveGoToDoor 
        #moduleMoveGoThroughDoor = 
        #modMoveStart comes on first and is used to turn mMIDOBJ on.
        #It should not come on again.
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
            self.moduleMoveToBluePyramid,self.moduleCells,self.moduleMoveStart)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
            self.moduleMoveToRedStal,self.moduleCells,self.moduleMoveStart)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
            self.moduleMoveToBlueStal,self.moduleCells,self.moduleMoveStart)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
            self.moduleMoveToRedPyramid,self.moduleCells,self.moduleMoveStart)

        #moduleMoveToShape + moduleMoveStart -> moduleMoveIDObj
        #the submodules(mmio,mmgtd,mmgthd) prevent exploreStart coming on again.
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleMoveToBluePyramid,self.moduleCells,self.moduleMoveIDObj)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleMoveToRedStal,self.moduleCells,self.moduleMoveIDObj)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleMoveToBlueStal,self.moduleCells,self.moduleMoveIDObj)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleMoveToRedPyramid,self.moduleCells,self.moduleMoveIDObj)

        self.fsa.stateHalfTurnsOnState(self.moduleCells,
            self.moduleMoveStart,self.moduleCells,self.moduleMoveIDObj)
        self.fsa.stateInhibitsState(self.moduleCells,self.moduleMoveIDObj,
                                    self.moduleCells,self.moduleMoveStart,
                                    inhibMoveStartWt)

        #moduleMoveToShape + modObjIDed -> moduleMoveGoToDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
                    self.moduleMoveToBluePyramid,
                    self.moduleCells,self.moduleMoveGoToDoor)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
                    self.moduleMoveToRedStal,
                    self.moduleCells,self.moduleMoveGoToDoor)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
                    self.moduleMoveToBlueStal,
                    self.moduleCells,self.moduleMoveGoToDoor)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
                    self.moduleMoveToRedPyramid,
                    self.moduleCells,self.moduleMoveGoToDoor)

        self.fsa.stateHalfTurnsOnState(self.moduleCells,
          self.moduleObjIdentified,self.moduleCells,self.moduleMoveGoToDoor)
        self.fsa.stateInhibitsState(self.moduleCells,self.moduleMoveGoToDoor,
                                    self.moduleCells,self.moduleMoveStart,
                                    inhibMoveStartWt)

        #moduleMoveToShape + modInDoor -> moduleMoveGoThroughDoor
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
                    self.moduleMoveToBluePyramid,
                    self.moduleCells,self.moduleMoveGoThroughDoor)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
                    self.moduleMoveToRedStal,
                    self.moduleCells,self.moduleMoveGoThroughDoor)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
                    self.moduleMoveToBlueStal,
                    self.moduleCells,self.moduleMoveGoThroughDoor)
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
                    self.moduleMoveToRedPyramid,
                    self.moduleCells,self.moduleMoveGoThroughDoor)

        self.fsa.stateHalfTurnsOnState(self.moduleCells,
          self.moduleInDoor,self.moduleCells,self.moduleMoveGoThroughDoor)
        self.fsa.stateInhibitsState(self.moduleCells,
            self.moduleMoveGoThroughDoor,self.moduleCells,self.moduleMoveStart,
                                    inhibMoveStartWt)

        #moduleMoveToShape + modThroughDoor -> moduleMoveIDObj
        #the moduleMoveToShape -> moduleMoveIDObj already primed above
        self.fsa.stateHalfTurnsOnState(self.moduleCells,
                    self.moduleThroughDoor,
                    self.moduleCells,self.moduleMoveIDObj)

        #The move submodules turn on their associated goals.
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
            self.moduleMoveIDObj,self.goalCells,self.goalIDObj)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
                self.moduleMoveGoToDoor,self.goalCells,self.goalGoToDoor)
        self.fsa.stateTurnsOnStateSlow(self.moduleCells,
                self.moduleMoveGoThroughDoor,self.goalCells,
                                       self.goalGoThroughDoor)

        #turn the goal off via the moduleMovedToShape and the movedToShapeAction
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                self.moduleMoveToBluePyramid,
                    self.moduleCells,self.moduleMovedToBluePyramid)
        self.fsa.stateHalfTurnsOnState(self.factCells, self.bluePyrFact,
                    self.moduleCells,self.moduleMovedToBluePyramid)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                self.moduleMoveToRedStal,
                    self.moduleCells,self.moduleMovedToRedStal)
        self.fsa.stateHalfTurnsOnState(self.factCells, self.redStalFact,
                    self.moduleCells,self.moduleMovedToRedStal)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                self.moduleMoveToBlueStal,
                    self.moduleCells,self.moduleMovedToBlueStal)
        self.fsa.stateHalfTurnsOnState(self.factCells, self.blueStalFact,
                    self.moduleCells,self.moduleMovedToBlueStal)
        self.fsa.stateHalfTurnsOnState(self.moduleCells, 
                self.moduleMoveToRedPyramid,
                    self.moduleCells,self.moduleMovedToRedPyramid)
        self.fsa.stateHalfTurnsOnState(self.factCells, self.redPyrFact,
                    self.moduleCells,self.moduleMovedToRedPyramid)

        #turn on movedToShapeAction and via the action turn off the goal 
        #and the modules in connectMoveDoneAction

    def connectPlanner(self):
        self.makeGoalCAs()
        self.makeModuleCAs()
        self.makeFactCAs()
        self.connectFacts()
        self.connectModules()
        self.connectGoalsAndModules()
        self.connectModulesAndActions()
        self.actionCausesReset()
        self.resetStopsFacts()
        self.connectGoToDoor()
        self.connectGoThroughDoor()
        self.connectIDObj()
        self.connectExplore()
        self.connectMoveBefore()

    def printPklSpikes(self,fileName):
        fileHandle = open(fileName)
        neoObj = pickle.load(fileHandle)
        segments = neoObj.segments
        segment = segments[0]
        spikeTrains = segment.spiketrains
        neurons = len(spikeTrains)
        for neuronNum in range (0,neurons):
            if (len(spikeTrains[neuronNum])>0):
                spikes = spikeTrains[neuronNum]
                for spike in range (0,len(spikes)):
                    print neuronNum, spikes[spike]
        fileHandle.close()

    def printResults(self):
        if (not nealParameters.printSmall):
            self.moveLeftTimer.printResults('results/planTimerMoveLeft')
            self.moveRightTimer.printResults('results/planTimerMoveRight')
            self.goTestInDoorTimer.printResults('results/planTimerGoTestInDoor')
            self.goToDoorTimer.printResults('results/planTimerGoToDoor')
            self.iDObjTimer.printResults('results/planTimerIDObj')
            self.cantIDObjTimer.printResults('results/planTimerCantIDObj')
            self.goThroughDoorTimer.printResults('results/planTimerGoThroughDoor')
            self.noInDoorFoundTimer.printResults('results/planTimerNoInDoorFound')
        if (nealParameters.simulator=="spinnaker"): 
            self.goalCells.printSpikes('results/planGoal.sp')
            self.moduleCells.printSpikes('results/planModule.sp')
            self.factCells.printSpikes('results/planFact.sp')
            self.actionCells.printSpikes('results/planAction.sp')
            self.resetCells.printSpikes('results/planReset.sp')
        elif (nealParameters.simulator=="nest"): 
            if (not nealParameters.printSmall):
                self.goalCells.printSpikes('results/planGoal.pkl')
                self.moduleCells.printSpikes('results/planModule.pkl')
                self.resetCells.printSpikes('results/planReset.pkl')
            self.factCells.printSpikes('results/planFact.pkl')
            self.actionCells.printSpikes('results/planAction.pkl')
            #print 'action'
            #self.printPklSpikes('results/planAction.pkl')

    #test functions
    def testSetGoal(self,goalNumber,spikeGenerator):
        print "set goal ", goalNumber
        #self.fsa.turnOnStateSpikeSource(spikeGenerator,goalNumber,self.goalCells)
        self.fsa.turnOnStateFromSpikeSource(spikeGenerator,
                                            self.goalCells, goalNumber)

    def testStimulateFact(self,factNumber,spikeGenerator):
        self.fsa.turnOnStateFromSpikeSource(spikeGenerator,
                                            self.factCells, factNumber)

#-- Access Functions
    #The goal should be set by the user.  Typically this is done
    #from the parser.
    def getGoalCells(self):
        return self.goalCells

    def getActionCells(self):
        return self.actionCells

    def getModuleCells(self):
        return self.moduleCells

    def getFactCells(self):
        return self.factCells

    #We need to get the action cells in nest to set up the output from cabot
    def getActionSpikeDet(self):
        return self.actionSpikeDet
