"""
This tests the before goal.  It turns on facts from a spike source to 
make the whole thing work.  First it sets the goal; the facts
should have the cmRequestBeforeFact (e.g. moveBeforeBluePyramidFact after
the second spikeSource.) Then third spike source is the 
says what the cm would respond as the objectbefore (e.g 
moveToTheRedPyramidFact). 
It should be trying to id the object now, and a default rightAction goes.
The fourth presents a posColourShape fact, which
fulfills the subgoal goalIDObj.  It then presents an inDoor fact, which
fullfils the subgoal goalGoToDoor;  it then presents the objCtr fact, 
which fulfills the subgoal goalGoThroughDoor.  It finishes with redPyrFact
"""
import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from planClass import PlanClass

RUN_DURATION = 2500
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [400.0]
    inputSpikeTimes2 = [850.0]
    inputSpikeTimes3 = [1100.0]
    inputSpikeTimes4 = [1300.0]
    inputSpikeTimes5 = [1500.0]
    
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}

    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')


    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5]

    
#---main 
neal = NealCoverFunctions(nealParameters.simulator)
init()
planner = PlanClass(nealParameters.simulator)

spikeGenerators = createInputs()
firstSpikeGenerator =  spikeGenerators[0]
secondSpikeGenerator =  spikeGenerators[1]
thirdSpikeGenerator =  spikeGenerators[2]
fourthSpikeGenerator =  spikeGenerators[3]
fifthSpikeGenerator =  spikeGenerators[4]
sixthSpikeGenerator =  spikeGenerators[5]

planner.testSetGoal(planner.goalMoveBeforeBluePyramid,firstSpikeGenerator)
planner.testStimulateFact(planner.moveToRedPyramidFact,secondSpikeGenerator)
planner.testStimulateFact(planner.bluePyrFact,thirdSpikeGenerator)
planner.testStimulateFact(planner.inFrontDoorFact,fourthSpikeGenerator)
planner.testStimulateFact(planner.objCtrFact,fifthSpikeGenerator)
planner.testStimulateFact(planner.redPyrCtrFact,sixthSpikeGenerator) 

sim.run(RUN_DURATION)

planner.printResults()

