"""
This tests the three goals idObj, goToDoor and goThroughDoor and the bump.  
It turns on facts from a spike source to make
the whole thing work.  First is idObj then a bump (1 at 100), 
then  it turns the goal off by turning on bluePyrCtr.
Next it's toDoor with a bump, and a inDoorCt to finish the goal.
Finally it's through the door, with a bump, and objCtr to finish the goal.
The actions should be three backs before the goals are turned off.
"""
import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from planClass import PlanClass

RUN_DURATION = 1400
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [100.0]
    inputSpikeTimes2 = [500.0]
    inputSpikeTimes3 = [610.0]
    inputSpikeTimes4 = [700.0]
    inputSpikeTimes5 = [800.0]
    inputSpikeTimes6 = [910.0]
    inputSpikeTimes7 = [1000.0]
    inputSpikeTimes8 = [1100.0]
    
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeArray6 = {'spike_times': [inputSpikeTimes6]}
    spikeArray7 = {'spike_times': [inputSpikeTimes7]}
    spikeArray8 = {'spike_times': [inputSpikeTimes8]}

    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')
    spikeGen6=sim.Population(1, sim.SpikeSourceArray, spikeArray6,
                                   label='inputSpikes_6')
    spikeGen7=sim.Population(1, sim.SpikeSourceArray, spikeArray7,
                                   label='inputSpikes_7')
    spikeGen8=sim.Population(1, sim.SpikeSourceArray, spikeArray8,
                                   label='inputSpikes_8')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,
            spikeGen6,spikeGen7,spikeGen8]

    
#spinnaker crashes if a spike source isn't connected to
#something.  Make the something here.
def makeSpinnakerSink():
    sinkCell = 0
    if (nealParameters.simulator == 'spinnaker'):
        sinkCell=sim.Population(1,sim.IF_cond_exp,{})
    return sinkCell

def turnOnSink(spikeSource,sinkCell):
    if (nealParameters.simulator == 'spinnaker'):
        connector = [(0,0,1.0, nealParameters.DELAY)]
        neal.nealProjection(spikeSource,sinkCell,connector,'inhibitory') 

def turnOnSinks(firstSink,spikeGenerators,sinkCell):
    for sinkNumber  in range (firstSink,8):
        turnOnSink(spikeGenerators[sinkNumber],sinkCell)



#---main 
neal = NealCoverFunctions(nealParameters.simulator)
init()
planner = PlanClass(nealParameters.simulator)

spikeGenerators = createInputs()
firstSpikeGenerator =  spikeGenerators[0]
secondSpikeGenerator =  spikeGenerators[1]
thirdSpikeGenerator =  spikeGenerators[2]
fourthSpikeGenerator =  spikeGenerators[3]
fifthSpikeGenerator =  spikeGenerators[4]
sixthSpikeGenerator =  spikeGenerators[5]
seventhSpikeGenerator =  spikeGenerators[6]
eighthSpikeGenerator =  spikeGenerators[7]
ninthSpikeGenerator =  spikeGenerators[8]

planner.testSetGoal(planner.goalIDObj,firstSpikeGenerator)
planner.testStimulateFact(planner.bumpFact,secondSpikeGenerator)
planner.testStimulateFact(planner.bluePyrCtrFact,thirdSpikeGenerator)

planner.testSetGoal(planner.goalGoToDoor,fourthSpikeGenerator)
planner.testStimulateFact(planner.bumpFact,fifthSpikeGenerator)
planner.testStimulateFact(planner.inFrontDoorFact,sixthSpikeGenerator)

planner.testSetGoal(planner.goalGoThroughDoor,seventhSpikeGenerator)
planner.testStimulateFact(planner.bumpFact,eighthSpikeGenerator)
planner.testStimulateFact(planner.objCtrFact,ninthSpikeGenerator)

sim.run(RUN_DURATION)

planner.printResults()

