"""
This tests the explore goal.  It turns on facts from a spike source to 
make the whole thing work.  It should give a default right first.  Then,
it presents a posColourShape fact, which fulfills the subgoal goalIDObj.  
It then presents an inDoor fact, which
fullfils the subgoal goalGoToDoor;  it then presents the objCtr fact, 
which fulfills the subgoal goalGoThroughDoor.  It finishes with exploreDoneFact,
which also needs a colourPosShape fact slightly before.
"""
import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from planClass import PlanClass

RUN_DURATION = 1500
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [800.0]
    inputSpikeTimes2 = [1000.0]
    inputSpikeTimes3 = [1200.0]
    inputSpikeTimes4 = [1400.0]
    inputSpikeTimes5 = [1410.0]
    inputSpikeTimes6 = [300.0]
    inputSpikeTimes7 = [350.0]
    
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5]

    
#spinnaker crashes if a spike source isn't connected to
#something.  Make the something here.
def makeSpinnakerSink():
    sinkCell = 0
    if (nealParameters.simulator == 'spinnaker'):
        sinkCell=sim.Population(1,sim.IF_cond_exp,{})
    return sinkCell

def turnOnSink(spikeSource,sinkCell):
    if (nealParameters.simulator == 'spinnaker'):
        connector = [(0,0,1.0, nealParameters.DELAY)]
        neal.nealProjection(spikeSource,sinkCell,connector,'inhibitory') 

def turnOnSinks(firstSink,spikeGenerators,sinkCell):
    for sinkNumber  in range (firstSink,5):
        turnOnSink(spikeGenerators[sinkNumber],sinkCell)

#---main 
neal = NealCoverFunctions(nealParameters.simulator)
init()
planner = PlanClass(nealParameters.simulator)

spikeGenerators = createInputs()
firstSpikeGenerator =  spikeGenerators[0]
secondSpikeGenerator =  spikeGenerators[1]
thirdSpikeGenerator =  spikeGenerators[2]
fourthSpikeGenerator =  spikeGenerators[3]
fifthSpikeGenerator =  spikeGenerators[4]
sixthSpikeGenerator =  spikeGenerators[5]

sinkCell = makeSpinnakerSink()
turnOnSinks(5,spikeGenerators,sinkCell)
planner.testSetGoal(planner.goalExplore,firstSpikeGenerator)
planner.testStimulateFact(planner.redPyrRightFact,secondSpikeGenerator)
planner.testStimulateFact(planner.inFrontDoorFact,thirdSpikeGenerator)
planner.testStimulateFact(planner.objCtrFact,fourthSpikeGenerator)
#need to turn both the explore done fact and the shape fact on as
#this is what would happen and it needs to turn off the goalIDed on its own.
planner.testStimulateFact(planner.bluePyrCtrFact,fifthSpikeGenerator) 
planner.testStimulateFact(planner.exploreDoneFact,sixthSpikeGenerator)

sim.run(RUN_DURATION)

planner.printResults()

