"""
This tests the idObjgoal.  It turns on facts from a spike source to make
the whole thing work.  First it turns on a left (1 at 100), then
it turns on a forward (2 at 200 but it's IDing so waits until 315).
It turns on right 0 at 500.  It default to right at 840.
In the main, it turns the goal off by turning on bluePyrCtr
but you can switch it to any place shape.
"""
import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from planClass import PlanClass

RUN_DURATION = 1800
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [100.0]
    inputSpikeTimes2 = [400.0]
    inputSpikeTimes3 = [700.0]
    inputSpikeTimes4 = [1100.0]
    inputSpikeTimes5 = [250.0]
    inputSpikeTimes6 = [300.0]
    inputSpikeTimes7 = [350.0]
    
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeArray6 = {'spike_times': [inputSpikeTimes6]}
    spikeArray7 = {'spike_times': [inputSpikeTimes7]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')
    spikeGen6=sim.Population(1, sim.SpikeSourceArray, spikeArray6,
                                   label='inputSpikes_6')
    spikeGen7=sim.Population(1, sim.SpikeSourceArray, spikeArray7,
                                   label='inputSpikes_7')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,
            spikeGen6, spikeGen7]

    
#spinnaker crashes if a spike source isn't connected to
#something.  Make the something here.
def makeSpinnakerSink():
    sinkCell = 0
    if (nealParameters.simulator == 'spinnaker'):
        sinkCell=sim.Population(1,sim.IF_cond_exp,{})
    return sinkCell

def turnOnSink(spikeSource,sinkCell):
    if (nealParameters.simulator == 'spinnaker'):
        connector = [(0,0,1.0, nealParameters.DELAY)]
        neal.nealProjection(spikeSource,sinkCell,connector,'inhibitory') 

def turnOnSinks(firstSink,spikeGenerators,sinkCell):
    for sinkNumber  in range (firstSink,8):
        turnOnSink(spikeGenerators[sinkNumber],sinkCell)



#---main 
neal = NealCoverFunctions(nealParameters.simulator)
init()
planner = PlanClass(nealParameters.simulator)

spikeGenerators = createInputs()
firstSpikeGenerator =  spikeGenerators[0]
secondSpikeGenerator =  spikeGenerators[1]
thirdSpikeGenerator =  spikeGenerators[2]
fourthSpikeGenerator =  spikeGenerators[3]
fifthSpikeGenerator =  spikeGenerators[4]

sinkCell = makeSpinnakerSink()
turnOnSinks(5,spikeGenerators,sinkCell)
planner.testSetGoal(planner.goalIDObj,firstSpikeGenerator)
planner.testStimulateFact(planner.objLeftFact,secondSpikeGenerator)
planner.testStimulateFact(planner.objCtrFact,thirdSpikeGenerator)
planner.testStimulateFact(planner.objRightFact,fourthSpikeGenerator)
#planner.testStimulateFact(planner.bluePyrFact,fifthSpikeGenerator)
planner.testStimulateFact(planner.bluePyrCtrFact,fifthSpikeGenerator)

sim.run(RUN_DURATION)

planner.printResults()

