"""
It finds a bug in spinnaker if you run with the moduleExplores turning
off moduleExploreStart, it pops on.
This tests the explore goal.  It turns on facts from a spike source to 
make the whole thing work.  First it presents a posColourShape fact, which
fulfills the subgoal goalIDObj.  It then presents an inDoor fact, which
fullfils the subgoal goalGoToDoor;  it then presents the objCtr fact, 
which fulfills the subgoal goalGoThroughDoor.  It finishes with exploreDoneFact,
which also needs a colourPosShape fact slightly before.
"""
import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from planClass import PlanClass

RUN_DURATION = 6000
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [400.0]
    inputSpikeTimes2 = [3650.0]
    inputSpikeTimes3 = [4100.0]
    inputSpikeTimes4 = [4200.0]
    inputSpikeTimes5 = [4400.0]
    inputSpikeTimes6 = [4700.0]
    inputSpikeTimes7 = [4950.0]
    inputSpikeTimes8 = [5100.0]
    inputSpikeTimes9 = [5250.0]
    inputSpikeTimes10 = [5400.0]
    inputSpikeTimes11 = [5450.0]
    
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeArray6 = {'spike_times': [inputSpikeTimes6]}
    spikeArray7 = {'spike_times': [inputSpikeTimes7]}
    spikeArray8 = {'spike_times': [inputSpikeTimes8]}
    spikeArray9 = {'spike_times': [inputSpikeTimes9]}
    spikeArray10 = {'spike_times': [inputSpikeTimes10]}
    spikeArray11 = {'spike_times': [inputSpikeTimes11]}

    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')
    spikeGen6=sim.Population(1, sim.SpikeSourceArray, spikeArray6,
                                   label='inputSpikes_6')
    spikeGen7=sim.Population(1, sim.SpikeSourceArray, spikeArray7,
                                   label='inputSpikes_7')
    spikeGen8=sim.Population(1, sim.SpikeSourceArray, spikeArray8,
                                   label='inputSpikes_8')
    spikeGen9=sim.Population(1, sim.SpikeSourceArray, spikeArray9,
                                   label='inputSpikes_9')
    spikeGen10=sim.Population(1, sim.SpikeSourceArray, spikeArray10,
                                   label='inputSpikes_10')
    spikeGen11=sim.Population(1, sim.SpikeSourceArray, spikeArray11,
                                   label='inputSpikes_11')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,
            spikeGen6,spikeGen7,spikeGen8,spikeGen9,spikeGen10,spikeGen11]

    
#---main 
neal = NealCoverFunctions(nealParameters.simulator)
init()
planner = PlanClass(nealParameters.simulator)

spikeGenerators = createInputs()
firstSpikeGenerator =  spikeGenerators[0]
secondSpikeGenerator =  spikeGenerators[1]
thirdSpikeGenerator =  spikeGenerators[2]
fourthSpikeGenerator =  spikeGenerators[3]
fifthSpikeGenerator =  spikeGenerators[4]
sixthSpikeGenerator =  spikeGenerators[5]
seventhSpikeGenerator =  spikeGenerators[6]
eighthSpikeGenerator =  spikeGenerators[7]
ninthSpikeGenerator =  spikeGenerators[8]
tenthSpikeGenerator =  spikeGenerators[9]
eleventhSpikeGenerator =  spikeGenerators[10]
twelfthSpikeGenerator =  spikeGenerators[11]

planner.testSetGoal(planner.goalExplore,firstSpikeGenerator)
planner.testStimulateFact(planner.bluePyrCtrFact,secondSpikeGenerator)
planner.testStimulateFact(planner.inDoorCtrFact,thirdSpikeGenerator)
planner.testStimulateFact(planner.objCtrFact,fourthSpikeGenerator)
planner.testStimulateFact(planner.redStalCtrFact,fifthSpikeGenerator) 
planner.testStimulateFact(planner.inDoorCtrFact,sixthSpikeGenerator) 
planner.testStimulateFact(planner.objCtrFact,seventhSpikeGenerator) 
planner.testStimulateFact(planner.blueStalCtrFact,eighthSpikeGenerator) 
planner.testStimulateFact(planner.inDoorCtrFact,ninthSpikeGenerator) 
planner.testStimulateFact(planner.objCtrFact,tenthSpikeGenerator) 
planner.testStimulateFact(planner.redPyrCtrFact,eleventhSpikeGenerator) 
planner.testStimulateFact(planner.exploreDoneFact,twelfthSpikeGenerator)

sim.run(RUN_DURATION)

planner.printResults()

