#State 0 comes on.  Input goes to state 1.
#State 0 and 1 turn on state 2.
#State 2 slides to 3; state 3 slides to 4 which is not a state.

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 200
#initialize the simulator. 
def init():
        #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)


def createTwoInputs():
    inputSpikeTimes0 = [10.0]
    #inputSpikeTimes1 = [52.0]
    inputSpikeTimes1 = [50.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')

    return [spikeGen0,spikeGen1]

def createNeurons():
    if (nealParameters.simulator == 'nest'):
            numNeurons = fsa.CA_SIZE * 5
    elif (nealParameters.simulator == 'spinnaker'):
            numNeurons = 100
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)
    return cells

def setupRecording(cells):
    if  (nealParameters.simulator == 'nest'):
        cells.record({'spikes','v'})
    elif  (nealParameters.simulator == 'spinnaker'):
        cells.record()

def printResults(spinnCells):
    spinnCells.printSpikes('temp.sp')



##--main---
neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)

init()

spikeGenerators = createTwoInputs()
firstSpikeGenerator =  spikeGenerators[0]
secondSpikeGenerator =  spikeGenerators[1]

stateCells = createNeurons()

setupRecording(stateCells)


#Build the FSA
fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,stateCells,0)
fsa.turnOnStateFromSpikeSource(secondSpikeGenerator,stateCells,1)
fsa.makeCA(stateCells,0)
fsa.makeCA(stateCells,1)
fsa.makeCA(stateCells,2)
fsa.makeCA(stateCells,3)
fsa.stateHalfTurnsOnState(stateCells,0,stateCells,2)
fsa.stateTurnsOffState(stateCells,2,stateCells,0)
fsa.stateHalfTurnsOnState(stateCells,1,stateCells,2)
fsa.stateTurnsOffState(stateCells,2,stateCells,1)

fsa.stateTurnsOnState(stateCells,2,stateCells,3)
fsa.stateTurnsOffState(stateCells,3,stateCells,2)
fsa.stateTurnsOnState(stateCells,3,stateCells,4)
fsa.stateTurnsOffState(stateCells,4,stateCells,3)

sim.run(RUN_DURATION)

fsa.testPrintResults(stateCells)



