#This is a three state FSA.  The states are 0,2 and 4.  The inputs
#are 1, 3.  0 and 1 turn 2 on.  2 and 3 turn on 4. 4 turns off 2 and 3.
#3 turns off 1 and 2. 

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 200

def createThreeInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [50.0]
    inputSpikeTimes2 = [100.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    return [spikeGen0,spikeGen1,spikeGen2]

def createNeurons():
    if (nealParameters.simulator == 'nest'):
            numNeurons = fsa.CA_SIZE * 5
    elif (nealParameters.simulator == 'spinnaker'):
            numNeurons = 100
        
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)

    return cells


##--main---
neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)

fsa.testInit()

#setup input
spikeGenerators = createThreeInputs()
firstSpikeGenerator =  spikeGenerators[0]
secondSpikeGenerator =  spikeGenerators[1]
thirdSpikeGenerator =  spikeGenerators[2]

#make neurons and make them recordable
stateCells = createNeurons()
fsa.testSetupRecording(stateCells)

#build the fsa and run it
#Build the FSA 
fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,stateCells,0)
fsa.turnOnStateFromSpikeSource(secondSpikeGenerator,stateCells,1)
#0 and 1 turn on 2
fsa.turnOnStateFromSpikeSource(thirdSpikeGenerator,stateCells,3)
#2 and 3 turn on 4
fsa.makeCA(stateCells,0)
fsa.makeCA(stateCells,1)
fsa.makeCA(stateCells,2)
fsa.makeCA(stateCells,3)
fsa.makeCA(stateCells,4)
#0 and 1 turn on 2
fsa.stateHalfTurnsOnState(stateCells,0,stateCells,2)
fsa.stateTurnsOffState(stateCells,2,stateCells,0)
fsa.stateHalfTurnsOnState(stateCells,1,stateCells,2)
fsa.stateTurnsOffState(stateCells,2,stateCells,1)
#2 and 3 turn on 4
fsa.stateHalfTurnsOnState(stateCells,2,stateCells,4)
fsa.stateTurnsOffState(stateCells,4,stateCells,2)
fsa.stateHalfTurnsOnState(stateCells,3,stateCells,4)
fsa.stateTurnsOffState(stateCells,4,stateCells,3)

#run the simulator
fsa.testRunFSA(RUN_DURATION)

#print results
fsa.testPrintResults(stateCells)



