#This builds and test a two state FSA.  It calls the test functions
#in the FSAClass.  
#state 0 turns on state 2.  State1 turns on 2 slow, but 2 doesn't turn off 0.

import nealParams as nealParameters
#get the simulator
if  (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle
import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 2000

def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)


def createInput():
    inputSpikeTimes0 = [10.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')

    return spikeGen0



##--main---
neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)
init()
#setup input
firstSpikeGenerator =  createInput()


#make neurons and make them recordable
goalCells = sim.Population(150,sim.IF_cond_exp, fsa.CELL_PARAMS)
moduleCells = sim.Population(490,sim.IF_cond_exp, fsa.CELL_PARAMS)
fsa.testSetupRecording(goalCells)
fsa.testSetupRecording(moduleCells)

#build the fsa and run it
fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,goalCells,8)
fsa.makeCA(goalCells,0)
fsa.makeCA(goalCells,8)
fsa.makeCA(moduleCells,25)
fsa.stateTurnsOnState(goalCells,0,goalCells,8)
fsa.stateTurnsOffState(goalCells,8,goalCells,0)
fsa.stateTurnsOnStateSlow(goalCells,8,moduleCells,25)
fsa.testRunFSA(RUN_DURATION)

#print results
goalCells.printSpikes('tempGoal.sp')
moduleCells.printSpikes('tempModule.sp')



