#This makes a 3 state Slide.  State 0 comes on, then turns on state 1,
#which turns off 0. 1 turns on 2 which turns off 1. 2 is not a state, so it 
#stops.
#State 0 slides to state 1, state 1 slides to 2, which is not a state.

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 100
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createOneInput():
    inputSpikeTimes0 = [10.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                             label='inputSpikes_0')
    return spikeGen0


def createNeurons():
    if (nealParameters.simulator == 'nest'):
        numNeurons = fsa.CA_SIZE * 3
    elif (nealParameters.simulator == 'spinnaker'):
        numNeurons = 100
        
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)

    return cells

def setupRecording(cells):
    if  (nealParameters.simulator == 'nest'):
        cells.record({'spikes','v'})
    elif (nealParameters.simulator == 'spinnaker'):
        cells.record()

def printResults(spinnCells):
    #print
    if simulator == "nest":
        spikes = nest.GetStatus(nestRecorder)
        test = spikes[0]['events']['times']
        print 'spikes', test
    elif simulator == 'spinnaker':
        spinnCells.printSpikes('temp.sp')



##--main---
neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)

init()
firstSpikeGenerator =  createOneInput()

stateCells = createNeurons()

setupRecording(stateCells)

#Build the FSA
fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,stateCells,0)
fsa.makeCA(stateCells,0)
fsa.makeCA(stateCells,1)
#fsa.makeCA(stateCells,2)
fsa.stateTurnsOnState(stateCells,0,stateCells,1)
fsa.stateTurnsOffState(stateCells,1,stateCells,0)
fsa.stateTurnsOnState(stateCells,1,stateCells,2)
fsa.stateTurnsOffState(stateCells,2,stateCells,1)

sim.run(RUN_DURATION)

fsa.testPrintResults(stateCells)



