#This is a 1 state machine turned on by a spike source.
#If you want to over stimulate it, modify create one input.  
#You can also test the range  of spikesoure inputs.  If you want to test that
#pick (or modify) the fsa.stimulateStateFromSpikeSource.

#In spinnaker, if you over stimulate it, it won't persist once that stops.
#In nest, if you give it more than 2 spikes, it won't persist.
#In spinnaker, it self corrects from 7ms to 5ms.  
#In nest, it self corrects from 6ms to 5ms

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 200
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)


def printPklSpikes(fileName):
    fileHandle = open(fileName)
    neoObj = pickle.load(fileHandle)
    segments = neoObj.segments
    segment = segments[0]
    spikeTrains = segment.spiketrains
    neurons = len(spikeTrains)
    for neuronNum in range (0,neurons):
        if (len(spikeTrains[neuronNum])>0):
            spikes = spikeTrains[neuronNum]
            for spike in range (0,len(spikes)):
                print neuronNum, spikes[spike]
    fileHandle.close()
    

def createOneInput():
    #inputSpikeTimes0 = [10.0,13.0,16.0,19.0,21.0] #spinn every 3ms
    #inputSpikeTimes0 = [10.0,14.0,18.0,22.0,26.0] #spinn every 4ms
    inputSpikeTimes0 = [10.0,14.0,18.0,22.0,26.0,30.0,34.0,38.0,42.0,46.0,50.0] #
    #inputSpikeTimes0 = [10.0,15.0,20.0,25.0,30.0,35.0] #spinn every 5ms; nest fails
    #inputSpikeTimes0 = [10.0,15.0] #spinn every 5ms; nest fails
    #inputSpikeTimes0 = [10.0,16.0] #every 5ms in nest and spinn
    #inputSpikeTimes0 = [10.0,100.0] #every 5ms in spinn; nest fails
    #inputSpikeTimes0 = [10.0] #every 5ms 

    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    return [spikeGen0]

def createNeurons():
    if (nealParameters.simulator == 'nest'):
        numNeurons = fsa.CA_SIZE * 3
    elif (nealParameters.simulator == 'spinnaker'):
        numNeurons = 100
        
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)
    return cells

def setupRecording(cells):
    if (nealParameters.simulator == 'nest'):
        cells.record({'spikes','v'})
    elif  (nealParameters.simulator == 'spinnaker'):
        cells.record()

def printResults(simCells):
    #print
    if  (nealParameters.simulator == 'nest'):
        simCells.printSpikes('temp.pkl')
        #printPklSpikes('temp.pkl')
    elif (nealParameters.simulator == 'spinnaker'):
        simCells.printSpikes('temp.sp')

##--main---
neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)
init()
spikeGenerators = createOneInput()
firstSpikeGenerator =  spikeGenerators[0]

stateCells = createNeurons()

setupRecording(stateCells)

#Build the FSA
#fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,stateCells,0)
#fsa.stimulateStateFromSpikeSource(firstSpikeGenerator,stateCells,0,0.053) #spinn and nest nothing
#fsa.stimulateStateFromSpikeSource(firstSpikeGenerator,stateCells,0,0.08) #spinn 6 to 5ms; nest every 5ms
#fsa.stimulateStateFromSpikeSource(firstSpikeGenerator,stateCells,0,0.07) #6 to 5ms; nest every 5ms
#fsa.stimulateStateFromSpikeSource(firstSpikeGenerator,stateCells,0,0.06) #7 to 5 ms; nest every 6 ms to every 5ms
#fsa.stimulateStateFromSpikeSource(firstSpikeGenerator,stateCells,0,0.054) #7 to 5 ms; nest nothing
fsa.stimulateStateFromSpikeSource(firstSpikeGenerator,stateCells,0,0.2) 


fsa.makeCA(stateCells,0)

sim.run(RUN_DURATION)

printResults(stateCells)



