"""
This has two states, a timer, and a trigger neuron.  The first spike gen
starts the first state.  The second time gen (along with the first state)
triggers the timer.  When the timer ends, it starts the second state, which
shuts off the first state.
"""
import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from timerClass import TimerClass
from stateMachineClass import FSAHelperFunctions

RUN_DURATION = 400
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createInputs():
    inputSpikeTimes0 = [10.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')

    inputSpikeTimes1 = [50.0]
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    return [spikeGen0,spikeGen1]

    inputSpikeTimes2 = [100.0]
    inputSpikeTimes3 = [150.0]
    inputSpikeTimes4 = [200.0]
    inputSpikeTimes5 = [250.0]
    inputSpikeTimes6 = [300.0]
    inputSpikeTimes7 = [350.0]
    
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeArray6 = {'spike_times': [inputSpikeTimes6]}
    spikeArray7 = {'spike_times': [inputSpikeTimes7]}
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')
    spikeGen6=sim.Population(1, sim.SpikeSourceArray, spikeArray6,
                                   label='inputSpikes_6')
    spikeGen7=sim.Population(1, sim.SpikeSourceArray, spikeArray7,
                                   label='inputSpikes_7')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,
            spikeGen6, spikeGen7]

#---main 
args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 2):
    temp = int(args[1])
else:
    temp = -1

neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)

init()

stateCells=sim.Population(2*(fsa.CA_SIZE)+1,sim.IF_cond_exp,fsa.CELL_PARAMS)
if  (nealParameters.simulator == 'nest'):
    stateCells.record({'spikes','v'})
elif  (nealParameters.simulator == 'spinnaker'):
    stateCells.record()
fsa.makeCA(stateCells,0)
fsa.makeCA(stateCells,1)

timer = TimerClass(nealParameters.simulator,10)
timer.makeStopTimerSynapses()
#compoundTimerCells = timer.getTimerCells()

spikeGens = createInputs()
spikeGenerator0 = spikeGens[0]
fsa.turnOnStateFromSpikeSource(spikeGenerator0,stateCells,0)

#turn on the extra neuron
extraNeuron = 20
spikeGenerator1 = spikeGens[1]
connector = [(0,extraNeuron,fsa.INPUT_WEIGHT, nealParameters.DELAY)]
neal.nealProjection(spikeGenerator1, stateCells, connector,'excitatory')
timer.oneNeuronHalfStartsTimer(stateCells,extraNeuron)
timer.stateHalfStartsTimer(stateCells,0)
timer.timerStartsState(stateCells,1)
fsa.stateTurnsOffState(stateCells,1,stateCells,0)

sim.run(RUN_DURATION)

timer.printResults("results/testCompoundTimer")
if (nealParameters.simulator == 'nest'):
    stateCells.printSpikes("results/testCompoundState.pkl")
elif  (nealParameters.simulator == 'spinnaker'):
    stateCells.printSpikes("results/testCompoundState.sp")


