#This tests the self stopping timer, and turning it on.
#Initially, a state is turned on, but doesn't start the timer.
#The oneNeuron (with the state) turns the timer on.
#The timer should produce spikes throughout the cascade, that stop at the 
#last state. The state should fire persistently at 5ms.
import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from timerClass import TimerClass

RUN_DURATION = 400
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createStateCells():
    if (nealParameters.simulator == 'nest'):
        numNeurons = fsa.CA_SIZE
    elif (nealParameters.simulator == 'spinnaker'):
        numNeurons = 100
        
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)
    return cells

def createInputs():
    inputSpikeTimes0 = [10.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    inputSpikeTimes1 = [60.0]
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
    return [spikeGen0, spikeGen1]

def setupRecording(cells):
    if (nealParameters.simulator == 'nest'):
        cells.record({'spikes','v'})
    elif  (nealParameters.simulator == 'spinnaker'):
        cells.record()
        cells.record_v()

def printResults(oneCell,stateCells):
    #print
    if  (nealParameters.simulator == 'nest'):
        oneCell.printSpikes('results/tempOne.pkl')
        stateCells.printSpikes('results/tempState.pkl')
    elif (nealParameters.simulator == 'spinnaker'):
        oneCell.printSpikes('results/tempOne.sp')
        stateCells.printSpikes('results/tempState.sp')

#---main 

neal = NealCoverFunctions(nealParameters.simulator)
fsa = FSAHelperFunctions(nealParameters.simulator)
init()

#make neurons
oneCell=sim.Population(1,sim.IF_cond_exp,fsa.CELL_PARAMS)
setupRecording(oneCell)
stateCells = createStateCells()
setupRecording(stateCells)
timer = TimerClass(nealParameters.simulator,10)

spikeGenerators = createInputs()
spikeGenToState = spikeGenerators[0]
spikeGenToOneNeuron = spikeGenerators[1]

#connect things
timer.makeStopTimerSynapses()
fsa.makeCA(stateCells,0)

connector = [(0,0,fsa.INPUT_WEIGHT,nealParameters.DELAY)]
neal.nealProjection(spikeGenToOneNeuron, oneCell, connector,'excitatory')
fsa.turnOnStateFromSpikeSource(spikeGenToState,stateCells,0)

timer.oneNeuronHalfStartsTimer(oneCell,0)
timer.stateHalfStartsTimer(stateCells,0)

sim.run(RUN_DURATION)

timer.printResults("results/testTimer")
printResults(oneCell,stateCells)
