#The timer shouldn't come on.  The neuron only turns it half on.
import sys

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from timerClass import TimerClass

RUN_DURATION = 400
#initialize the simulator. 
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createInputs():
    inputSpikeTimes0 = [10.0,60.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    return spikeGen0

def setupRecording(cells):
    if (nealParameters.simulator == 'nest'):
        cells.record({'spikes','v'})
    elif  (nealParameters.simulator == 'spinnaker'):
        cells.record()
        cells.record_v()

def printResults(simCells):
    #print
    if  (nealParameters.simulator == 'nest'):
        simCells.printSpikes('results/tempOne.pkl')
    elif (nealParameters.simulator == 'spinnaker'):
        simCells.printSpikes('results/tempOne.sp')

#---main 

neal = NealCoverFunctions(nealParameters.simulator)
init()


fsa = FSAHelperFunctions(nealParameters.simulator)
oneCell=sim.Population(1,sim.IF_cond_exp,fsa.CELL_PARAMS)
setupRecording(oneCell)

timer = TimerClass(nealParameters.simulator,10)
timer.makeStopTimerSynapses()

spikeGenerator = createInputs()
connector = [(0,0,fsa.INPUT_WEIGHT,nealParameters.DELAY)]
neal.nealProjection(spikeGenerator, oneCell, connector,'excitatory')
timer.oneNeuronHalfStartsTimer(oneCell,0)

sim.run(RUN_DURATION)

timer.printResults("results/testTimer")
printResults(oneCell)
