"""
This makes several different types of timers and provides mechanisms
for the timers to effect other states and be turned on by other 
states, spike sources and neurons.
"""


#---- Code
import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

class TimerClass:
    forwardWeight = 0.004
    oneNeuronFullWeight = 0.08
    oneNeuronHalfWeight = 0.04

    def __init__(self):
        self.simulator = simName

    def __init__(self, simName,numberStates):
        self.simulator = simName
        self.numberStates = numberStates
        self.neal = NealCoverFunctions(self.simulator)
        self.fsa = FSAHelperFunctions(self.simulator)
        self.fsa.initParams()
        self.timerCells = self.createNeurons(numberStates)
        if (not nealParameters.printSmall):
            self.setupRecording()

    #create the neurons for the words and the states.
    def createNeurons(self,numberStates):
        numberTimerCells = numberStates * self.fsa.CA_SIZE
        localTimerCells=sim.Population(numberTimerCells,sim.IF_cond_exp,
                    self.fsa.CELL_PARAMS)
        
        return localTimerCells

    #-----start timer functions
    def connectSpikeGenToStart(self,spikeGen):
        self.fsa.turnOnStateFromSpikeSource(spikeGen,self.timerCells,0)

    def stateStartsTimer(self,stateNeurons,state):
        self.fsa.stateTurnsOnState(stateNeurons,state,self.timerCells,0)

    #undone not in test
    def oneNeuronStartsTimer(self,fromNeurons,fromNeuron):
        self.fsa.oneNeuronStimulatesState(fromNeurons,fromNeuron,
            self.timerCells,0,self.oneNeuronFullWeight)

    def oneNeuronHalfStartsTimer(self,fromNeurons,fromNeuron):
        self.fsa.oneNeuronStimulatesState(fromNeurons,fromNeuron,
            self.timerCells,0,self.oneNeuronHalfWeight)

    def stateHalfStartsTimer(self,stateNeurons,state):
        self.fsa.stateHalfTurnsOnState(stateNeurons,state,self.timerCells,0)

    #----stop functions
    #undone needs a test
    def stateStopsTimer(self,stateNeurons,state):
        for timerCA in range (0,self.numberStates):
            self.fsa.stateTurnsOffState(stateNeurons,state,
                                        self.timerCells,timerCA)

    #---timer output functions
    #The last state of a timer starts a state
    def timerStartsState(self,stateNeurons,state):
        self.fsa.stateTurnsOnState(self.timerCells,self.numberStates-1,
                                   stateNeurons,state)

    def timerPreventsState(self,stateNeurons,state):
        for CA in range (0,self.numberStates-1):
            self.fsa.stateTurnsOffState(self.timerCells,CA,
                                   stateNeurons,state)

    #undone needs a test
    def timerActivatesState(self,stateNeurons,state,wt):
        for CA in range (0,self.numberStates-1):
            self.fsa.stateStimulatesState(self.timerCells,CA,stateNeurons,
                                          state,wt)


    #---create timer functions
    #The timer goes from state to state and the last stops on its own
    def makeStopTimerSynapses(self):
        for CA in range (0,self.numberStates-1):
            self.fsa.makeCA(self.timerCells,CA)
            self.fsa.stateStimulatesState(self.timerCells,CA,self.timerCells,
                                             CA+1,self.forwardWeight)
            self.fsa.stateTurnsOffState(self.timerCells,CA+1,self.timerCells,CA)
            
    #The timer goes from state to state and the last stops on its own
    #Only one state is on at at time
    def makeOneStateStopTimerSynapses(self):
        for CA in range (0,self.numberStates-1):
            self.fsa.makeCA(self.timerCells,CA)
            self.fsa.stateStimulatesState(self.timerCells,CA,self.timerCells,
                                             CA+1,self.forwardWeight)
            self.fsa.stateTurnsOffState(self.timerCells,CA+1,self.timerCells,CA)
        for toCA in range (0,self.numberStates-1):
            for fromCA in range (toCA+2,self.numberStates):
                self.fsa.stateTurnsOffState(self.timerCells,fromCA,
                                            self.timerCells,toCA)

    #The timer goes from state to state and the last stops on its own
    #If the state that turns it on remains on, this timer will not restart
    #until its done
    def makeStopNoRestartTimerSynapses(self):
        for CA in range (0,self.numberStates-1):
            self.fsa.makeCA(self.timerCells,CA)
            self.fsa.stateStimulatesState(self.timerCells,CA,self.timerCells,
                                             CA+1,self.forwardWeight)
            self.fsa.stateTurnsOffState(self.timerCells,CA+1,self.timerCells,CA)

        #have the later CAs prevent all the earlier ones
        for fromCA in range (2,self.numberStates-1):
            for toCA in range (0,fromCA-1):
                self.fsa.stateTurnsOffState(self.timerCells,fromCA,
                                            self.timerCells,toCA)

        
    def setupRecording(self):
        if  (nealParameters.simulator == 'nest'):
            self.timerCells.record({'spikes','v'})
        elif  (nealParameters.simulator == 'spinnaker'):
            self.timerCells.record()

    def printPklSpikes(self,fileName):
        fileHandle = open(fileName)
        neoObj = pickle.load(fileHandle)
        segments = neoObj.segments
        segment = segments[0]
        spikeTrains = segment.spiketrains
        neurons = len(spikeTrains)
        for neuronNum in range (0,neurons):
            if (len(spikeTrains[neuronNum])>0):
                spikes = spikeTrains[neuronNum]
                for spike in range (0,len(spikes)):
                    print neuronNum, spikes[spike]
        fileHandle.close()
    
    def printResults(self,fileName):
        if (nealParameters.printSmall):
            return
        if (nealParameters.simulator == 'nest'):
            fileName = fileName + ".pkl"
            self.timerCells.printSpikes(fileName)
            #self.printPklSpikes(fileName)
        elif  (nealParameters.simulator == 'spinnaker'):
            fileName = fileName + ".sp"
            self.timerCells.printSpikes(fileName)

#-- Access Functions
    def getTimerCells(self):
        return self.timerCells



