#This takes a file name, a picture from the data directory.  It runs the
#vision system with some simple plan fact action. 
#undone
#in results/visionObjRec.sp and results/planFact.sp (and other files)
#example call is
#python visionTest.py rtRedStal1.jpg

import sys

"""
The test for vision.  
"""

#imports for PyNN
import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np
from getPixelsClass import imageToPixelClass

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

#Other NEAL files and functions

from visionClass import visionArea
from planClass import planArea

#------set up object to fact connections----------
def factTurnsFactOn ():
    synWeight = 1.7
    connector = []
    for cFrom in range (0,5):
        for cTo in range (0,5):
            connector = connector + [(cFrom,cTo,synWeight,DELAY)]
    return connector
    
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)


def createInputs():
    #inputSpikeTimes = [10.0,40.0,70.0,100.0,200.0,230.0,260.0,290.0]
    inputSpikeTimes = [10.0,40.0,70.0,100.0,200.0,230.0,260.0,290.0,320.0,350.0,380.0,500.0,530.0,560.0,590.0]
    #inputSpikeTimes = [10.0]
    
    spikeArray = {'spike_times': [inputSpikeTimes]}
    spikeGen=sim.Population(1,sim.SpikeSourceArray,spikeArray,
                                   label='inputSpikes_0')

    return [spikeGen]

def connectBWInputs(spikeGen,pixels):
    toInputWt = 0.1

    #this is for no environment
    bwInputNeurons = vision.getBWVisionInputCells()
    connector = []
    for toNeuron in pixels:
        connector = connector + [(0,toNeuron,toInputWt,
                                  nealParameters.DELAY)]
    neal.nealProjection(spikeGen,bwInputNeurons,connector,'excitatory')
    
def connectColourInputs(spikeGen,pixels):
    toInputWt = 0.1

    #this is for no environment
    colourInputNeurons = vision.getColourVisionInputCells()
    connector = []
    for toNeuron in pixels:
        connector = connector + [(0,toNeuron,toInputWt,
                                  nealParameters.DELAY)]
    neal.nealProjection(spikeGen,colourInputNeurons,connector,'excitatory')
    
def testFact():
    fsa = FSAHelperFunctions(nealParameters.simulator)
    fsa.initParams()

    #inputSpikeTimes = [10.0,20.0,30.0,40.0]
    inputSpikeTimes = [200.0]  #reset
    inputSpikeTimes2 = [400.0,405.0] #idObj

    toInputWt = 0.1
    spikeArray = {'spike_times': [inputSpikeTimes]}
    spikeGen=sim.Population(1,sim.SpikeSourceArray,spikeArray,
                                   label='inputSpikes_0')
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')


    factNeurons = plan.getPlanFactCells()

    fsa.turnOnStateFromSpikeSource(spikeGen,factNeurons,plan.resetFact)
    fsa.turnOnStateFromSpikeSource(spikeGen2,factNeurons,plan.idObjFact)

def runTest(time):
    sim.run(time)

#The IDObj fact supports the objRec all 12 pyramids and stalactites 
def connectIDObjToObjRec(plan,vision):
    fsa = FSAHelperFunctions(nealParameters.simulator)
    fsa.initParams()
    synWeight = 0.001 #0.0015 #0.001  apr12
    objRecCells = vision.getObjRecCells()
    factCells = plan.getPlanFactCells()

    #idObj supports all of 12 pyramid and stalactite objRecs
    for objState in range (vision.redPyrLeftObj, vision.blueStalRightObj):
         fsa.oneStateStimulatesState(factCells,plan.idObjFact,objRecCells,
                    objState,synWeight)

#one to one connections
def connectOneObjRecToOneFact(objectNum,factNum,synWeight,vision,plan): 
    fsa = FSAHelperFunctions(nealParameters.simulator)
    fsa.initParams()
   
    objRecCells = vision.getObjRecCells()
    factCells = plan.getPlanFactCells()
    fsa.oneStateStimulatesState(objRecCells,objectNum,factCells,factNum,
                                synWeight)

def connectObjRecToFacts(vision,plan):  
    synWeight = 0.003 #weight for obj
    pyrWeight = 0.002 #0.02 0.005 #weight for red,blue;pyr,stal;center,left,right 
    
    #object rec stimulates parallel facts
    connectOneObjRecToOneFact(0,5,synWeight,vision,plan)
    connectOneObjRecToOneFact(vision.objCtr,plan.objCtrFact,synWeight,vision,
                              plan)
    connectOneObjRecToOneFact(2,7,synWeight,vision,plan)
    connectOneObjRecToOneFact(3,8,synWeight,vision,plan)
    connectOneObjRecToOneFact(4,9,synWeight,vision,plan)
    connectOneObjRecToOneFact(5,10,synWeight,vision,plan)
    connectOneObjRecToOneFact(vision.redPyrLeftObj,plan.redPyrLeftFact,
                              pyrWeight,vision,plan)#redpyrleft
    connectOneObjRecToOneFact(7,12,pyrWeight,vision,plan)
    connectOneObjRecToOneFact(8,13,pyrWeight,vision,plan)
    connectOneObjRecToOneFact(9,14,pyrWeight,vision,plan)#bluepyrleft
    connectOneObjRecToOneFact(10,15,pyrWeight,vision,plan)
    connectOneObjRecToOneFact(11,16,pyrWeight,vision,plan)
    connectOneObjRecToOneFact(12,17,pyrWeight,vision,plan)#redstalleft
    connectOneObjRecToOneFact(13,18,pyrWeight,vision,plan)
    connectOneObjRecToOneFact(14,19,pyrWeight,vision,plan)
    connectOneObjRecToOneFact(15,20,pyrWeight,vision,plan)#bluestalleft
    connectOneObjRecToOneFact(16,21,pyrWeight,vision,plan)
    connectOneObjRecToOneFact(17,22,pyrWeight,vision,plan)
    connectOneObjRecToOneFact(18,23,synWeight,vision,plan)#indoor
    connectOneObjRecToOneFact(19,24,synWeight,vision,plan)
    connectOneObjRecToOneFact(20,25,synWeight,vision,plan)

#------------Main Body---------------
args = sys.argv
numberArgs = args.__len__()
dirName = "data/"
if (numberArgs == 2):
    fileName = args[1]
else:
    fileName = "leftBluePyramid1.jpg"
fileName = dirName + fileName

neal = NealCoverFunctions(nealParameters.simulator)
init()

vision = visionArea(nealParameters.simulator)
vision.allocateVisionNeurons()
vision.makeVisionSynapses()

plan = planArea(nealParameters.simulator)
plan.allocatePlanNeurons()
plan.makePlanSynapses()

connectObjRecToFacts(vision,plan)
connectIDObjToObjRec(plan,vision)

#-------------------setup recording
#inpPop.record(['spikes','v'])
vision.setVisionRecording()
plan.setPlanRecording()

#open image file
imageTranslator = imageToPixelClass(nealParameters.INPSIZE)
pixels = imageTranslator.getPixelsFromFile(fileName) 
#pixels = imageTranslator.getPixelsFromFile("data/leftBluePyramid1.jpg") #rtObject
#pixels = imageTranslator.getPixelsFromFile("data/rtBluePyramid3.jpg") #rtObject

#pixels = imageTranslator.getPixelsFromFile("data/temp1.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/temp39.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/pyr1.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/doorLeft1.jpg") #doorLt
#pixels = imageTranslator.getPixelsFromFile("data/doorCtr.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/doorCtr1.jpg") #doorLt doorCtr
#pixels = imageTranslator.getPixelsFromFile("data/doorRt.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/rtBluePyramid1.jpg") #rtObject
#pixels = imageTranslator.getPixelsFromFile("data/rtBluePyramid2.jpg") #ctrObject rtObject
#pixels = imageTranslator.getPixelsFromFile("data/ctrBluePyramid.jpg") #ctrObject
#pixels = imageTranslator.getPixelsFromFile("data/ctrBluePyramid1.jpg") #ctrObject
#pixels = imageTranslator.getPixelsFromFile("data/stal2.jpg") #objCtr doorRt
#pixels = imageTranslator.getPixelsFromFile("data/ctrRedStal1.jpg") #ctrObject

#pixels = imageTranslator.getPixelsFromFile("data/rtBluePyramid3.jpg") #rtObject
#pixels = imageTranslator.getPixelsFromFile("data/inDoor1.jpg") 

bwPixels = pixels[0]
colourPixels = pixels[1]

spikeGen = createInputs()
connectBWInputs(spikeGen[0],bwPixels)
connectColourInputs(spikeGen[0],colourPixels)
testFact()

runTest(nealParameters.SIM_LENGTH)

#--------------print results

vision.printVisionNets()
print 'Object Recognition Spikes'
plan.printPlanNets()
#vision.printPklSpikes("results/visionObjRec.pkl")


