"""
The vision class without the connections. This uses the v1Connections and
retinaConnections to get the weights.
"""

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
    import spynnaker_external_devices_plugin.pyNN as externaldevices
    from  spinnVisionInputClass import spinnVisionInputArea
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle
    from  nestVisionInputClass import nestVisionInputArea

import numpy as np

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions

from retinaConnections import get3x3OnExcite, get3x3OnInhib
from retinaConnections import get3x3OffExcite,get3x3OffInhib
from retinaConnections import get6x6OnExcite, get6x6OnInhib
from retinaConnections import get6x6OffExcite, get6x6OffInhib
from retinaConnections import get9x9OnExcite, get9x9OnInhib
from retinaConnections import get9x9OffExcite,get9x9OffInhib

from v1Connections import getHEdgeTopOrBottomFrom3x3OnExcite
from v1Connections import getHEdgeTopOrBottomFrom3x3OffExcite
from v1Connections import getHEdgeBottomFrom6x6OffInhib
from v1Connections import getHEdgeBottomFrom9x9OffInhib
from v1Connections import getHEdgeTopFrom6x6OffInhib
from v1Connections import getHEdgeTopFrom9x9OffInhib
from v1Connections import getVEdgeLeftOrRightFrom3x3OffExcite
from v1Connections import getVEdgeLeftFrom9x9OffInhib
from v1Connections import getVEdgeRightFrom9x9OffInhib
from v1Connections import getSEdgeTopOrBottomFrom3x3OffExcite
from v1Connections import getSEdgeTopOrBottomFrom3x3OnExcite
from v1Connections import getSEdgeTopOrBottomFrom6x6OffInhib
from v1Connections import getSEdgeTopOrBottomFrom9x9OffInhib
from v1Connections import getSEdgeTopFrom6x6OffInhib
from v1Connections import getSEdgeTopFrom9x9OffInhib
from v1Connections import getSEdgeBottomFrom6x6OffInhib
from v1Connections import getSEdgeBottomFrom9x9OffInhib
from v1Connections import getBEdgeTopOrBottomFrom3x3OnExcite
from v1Connections import getBEdgeTopOrBottomFrom3x3OffExcite
from v1Connections import getBEdgeTopOrBottomFrom6x6OnInhib
from v1Connections import getBEdgeTopFrom6x6OffInhib
from v1Connections import getBEdgeTopFrom9x9OffInhib
from v1Connections import getBEdgeBottomFrom6x6OffInhib
from v1Connections import getBEdgeBottomFrom9x9OffInhib

class visionArea:
    objLeft = 0
    objCtr = 1
    objRight = 2
    doorLeft = 3
    doorCtr = 4
    doorRight = 5
    redPyrLeftObj = 6
    redPyrCtrObj = 7
    redPyrRightObj = 8
    bluePyrLeftObj = 9
    bluePyrCtrObj = 10
    bluePyrRightObj = 11
    redStalLeftObj = 12
    redStalCtrObj = 13
    redStalRightObj = 14
    blueStalLeftObj = 15
    blueStalCtrObj = 16
    blueStalRightObj = 17
    inDoorLeftObj = 18
    inDoorCtrObj = 19
    inDoorRightObj = 20
    inFrontDoorObj = 21

    def __init__(self, simName):
        self.neal = NealCoverFunctions(simName)
        self.fsa = FSAHelperFunctions(simName)
        self.fsa.initParams()
        self.numFacts = 29
        self.numObjects = 22
        self.numGoObjTimerSteps = 20
        self.numIDObjTimerSteps = 24
        self.numDoorTimerSteps = 24

        if (nealParameters.simulator == 'nest'):
            self.nestVisionInput = nestVisionInputArea()
        elif (nealParameters.simulator == 'spinnaker'):
            self.spinnVisionInput = spinnVisionInputArea()
        self.baseVisionCellParams = {}
            #self.baseVisionCellParams = {'v_thresh':-55.0, 'v_reset' : -70.0, 
            #'tau_refrac':b 2.0 , 'tau_syn_E': 5.0,  
            #'v_rest' : -65.0,'i_offset':0.0}

        self.simulator_name = simName
        self.threeOn = False
        self.threeOff = True
        self.sixOn = True
        self.sixOff = True
        self.nineOn = False
        self.nineOff = True

        self.hEdgeTop = True
        self.hEdgeBottom = True
        self.vEdgeLeft = True
        self.vEdgeRight = True
        self.sEdgeTop = True
        self.sEdgeBottom = True
        self.bEdgeTop = True
        self.bEdgeBottom = True
        self.andAngle = False
        self.orAngle = False

        

    #---create neurons
    def allocateSpinnakerVisualInputNeurons(self):
        if (nealParameters.useSpikeServer):
            self.bwInputCells = sim.Population(
                nealParameters.neuronsPerVisionSubnet,
                externaldevices.SpikeInjector,
                {'port': 12347}, label="bwVisionSpikeToBoard")
            externaldevices.activate_live_output_for(self.bwInputCells,
                database_notify_port_num=12347)        
            self.bwInputRecordCells = sim.Population(
                nealParameters.neuronsPerVisionSubnet,
                sim.IF_cond_exp,self.baseVisionCellParams)

            self.colourInputCells = sim.Population(1200,
                    externaldevices.SpikeInjector,
                    {'port': 12348}, label="colourVisionSpikeToBoard")
            externaldevices.activate_live_output_for(self.colourInputCells,
                    database_notify_port_num=12348)        
            self.colourInputRecordCells = sim.Population(1200,
                    sim.IF_cond_exp,self.baseVisionCellParams)
        else:
            self.bwInputCells = sim.Population(
                nealParameters.neuronsPerVisionSubnet,
                sim.IF_cond_exp,self.baseVisionCellParams)

            self.colourInputCells = sim.Population(1200,
                sim.IF_cond_exp,self.baseVisionCellParams)

    def allocateNestVisualInputNeurons(self):
        self.bwInputCells = sim.Population(
            nealParameters.neuronsPerVisionSubnet,
                sim.IF_cond_exp,self.baseVisionCellParams)

        self.colourInputCells = sim.Population(1200,
                sim.IF_cond_exp,self.baseVisionCellParams)

    def allocateRetinaNeurons(self):
        if self.threeOn:
            numThreeOnCells = nealParameters.neuronsPerVisionSubnet
        else:
            numThreeOnCells = 1

        if self.threeOff:
            numThreeOffCells = nealParameters.neuronsPerVisionSubnet
        else:
            numThreeOffCells = 1

        if self.sixOn:
            numSixOnCells = nealParameters.neuronsPerVisionSubnet
        else:
            numSixOnCells = 1

        if self.sixOff:
            numSixOffCells = nealParameters.neuronsPerVisionSubnet
        else:
            numSixOffCells = 1

        if self.nineOn:
            numNineOnCells = nealParameters.neuronsPerVisionSubnet
        else:
            numNineOnCells = 1

        if self.nineOff:
            numNineOffCells = nealParameters.neuronsPerVisionSubnet
        else:
            numNineOffCells = 1

        self.threeOnCells = sim.Population(numThreeOnCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.threeOffCells= sim.Population(numThreeOffCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.sixOnCells= sim.Population(numSixOnCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.sixOffCells= sim.Population(numSixOffCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.nineOnCells= sim.Population(numNineOnCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.nineOffCells= sim.Population(numNineOffCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        
    def allocateV1Neurons(self):
        if self.hEdgeTop:
            numHEdgeTopCells = nealParameters.neuronsPerVisionSubnet
        else:
            numHEdgeTopCells = 1
        if self.hEdgeBottom:
            numHEdgeBottomCells = nealParameters.neuronsPerVisionSubnet
        else:
            numHEdgeBottomCells = 1

        if self.vEdgeLeft:
            numVEdgeLeftCells = nealParameters.neuronsPerVisionSubnet
        else:
            numVEdgeLeftCells =  1
        if self.vEdgeRight:
            numVEdgeRightCells = nealParameters.neuronsPerVisionSubnet
        else:
            numVEdgeRightCells = 1
        if self.sEdgeTop:
            numSEdgeTopCells = nealParameters.neuronsPerVisionSubnet
        else:
            numSEdgeTopCells = 1
        if self.sEdgeBottom:
            numSEdgeBottomCells = nealParameters.neuronsPerVisionSubnet
        else:
            numSEdgeBottomCells = 1
        if self.bEdgeTop:
            numBEdgeTopCells = nealParameters.neuronsPerVisionSubnet
        else:
            numBEdgeTopCells = 1
        if self.bEdgeBottom:
            numBEdgeBottomCells = nealParameters.neuronsPerVisionSubnet
        else:
            numBEdgeBottomCells = 1

        self.hEdgeTopCells = sim.Population(numHEdgeTopCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.hEdgeBottomCells = sim.Population(numHEdgeBottomCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.vEdgeLeftCells = sim.Population(numVEdgeLeftCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.vEdgeRightCells = sim.Population(numVEdgeRightCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.sEdgeTopCells = sim.Population(numSEdgeTopCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.sEdgeBottomCells = sim.Population(numSEdgeBottomCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.bEdgeTopCells = sim.Population(numBEdgeTopCells,
                sim.IF_cond_exp,self.baseVisionCellParams)
        self.bEdgeBottomCells = sim.Population(numBEdgeBottomCells,
                sim.IF_cond_exp,self.baseVisionCellParams)


    def allocateObjRecNeurons(self):
        numObjRecCells = self.numObjects*self.fsa.CA_SIZE
        self.objRecCells = sim.Population(numObjRecCells,
                            sim.IF_cond_exp, self.fsa.CELL_PARAMS)
        #objCellParams = {'tau_refrac':2.0,'tau_syn_E':5.0,
        #                     'tau_syn_I':1.0,'v_thresh':-55.0}
        #self.objRecCells = sim.Population(numObjRecCells,
        #        sim.IF_cond_exp,objCellParams)
        
    def allocateVisionNeurons(self): 
        if (nealParameters.simulator=="spinnaker"):
            self.allocateSpinnakerVisualInputNeurons()
        elif (nealParameters.simulator=="nest"):
            self.allocateNestVisualInputNeurons()
        else:
            print 'bad pynn simulator for AllocateVisionNeurons'
        self.allocateRetinaNeurons()
        self.allocateV1Neurons()
        self.allocateObjRecNeurons()

    #----------------setup input from the environment for nest
    def setNestToUseStaticFile(self,fileName):
        self.nestVisionInput.setStaticFileName(fileName)

    def setNestVisionInput(self,currentTime):
        if (self.nestVisionInput.useStaticFileName()):
            #print "use nest static file input"
            mostRecentFile = self.nestVisionInput.getStaticFileName()
        else:
            #get the most recent picture
            mostRecentFile = self.nestVisionInput.getMostRecentFile()

        pictureNeuronsOn = self.nestVisionInput.getPictureInput(mostRecentFile)
        bwNeuronsOn = pictureNeuronsOn[0]
        colourNeuronsOn = pictureNeuronsOn[1]
        #consider, getPictureInput may fail returning [[-1],[-1]]

        #turn bw spikes on
        voltageList = []
        for neuronNumber in range (0,nealParameters.neuronsPerVisionSubnet):
            voltageList = voltageList + [-65.0]
        for neuronNumber in bwNeuronsOn:
            voltageList[neuronNumber] = -30.0
        self.bwInputCells.initialize(v=voltageList)

        #turn colour spikes on
        voltageList = []
        for neuronNumber in range (0,1200):
            voltageList = voltageList + [-65.0]
        for neuronNumber in colourNeuronsOn:
            voltageList[neuronNumber] = -30.0
        self.colourInputCells.initialize(v=voltageList)



    #----------------setup input to retina connections
    def makeInputToRetSynapses(self):
        #3x3On
        if self.threeOn:
            excConnectorList = get3x3OnExcite()
            self.neal.nealProjection(self.bwInputCells,self.threeOnCells, 
                                excConnectorList,'excitatory')

            inhConnectorList = get3x3OnInhib()
            self.neal.nealProjection(self.bwInputCells,self.threeOnCells, 
                                inhConnectorList,'inhibitory')

        #3x3Off
        if self.threeOff:
            excConnectorList = get3x3OffExcite()
            self.neal.nealProjection(self.bwInputCells, 
                            self.threeOffCells,excConnectorList,'excitatory')

            inhConnectorList = get3x3OffInhib()
            self.neal.nealProjection(self.bwInputCells, 
                            self.threeOffCells,inhConnectorList,'inhibitory')

        #6x6On
        if self.sixOn:
            excConnectorList = get6x6OnExcite()
            self.neal.nealProjection(self.bwInputCells, self.sixOnCells, 
                                excConnectorList,'excitatory')
            inhConnectorList = get6x6OnInhib()
            self.neal.nealProjection(self.bwInputCells, self.sixOnCells, 
                                inhConnectorList,'inhibitory')

        #6x6Off
        if self.sixOff:
            excConnectorList = get6x6OffExcite()
            self.neal.nealProjection(self.bwInputCells, self.sixOffCells, 
                                excConnectorList,'excitatory')
            inhConnectorList = get6x6OffInhib()
            self.neal.nealProjection(self.bwInputCells, self.sixOffCells, 
                                inhConnectorList,'inhibitory')

        #9x9On
        if self.nineOn:
            excConnectorList = get9x9OnExcite()
            self.neal.nealProjection(self.bwInputCells, self.nineOnCells,
                                excConnectorList,'excitatory')
            inhConnectorList = get9x9OnInhib()
            self.neal.nealProjection(self.bwInputCells, 
                            self.nineOnCells, inhConnectorList,'inhibitory')

        #9x9off
        if self.nineOff:
            excConnectorList = get9x9OffExcite()
            self.neal.nealProjection(self.bwInputCells, 
                            self.nineOffCells, excConnectorList,'excitatory')
            inhConnectorList = get9x9OffInhib()
            self.neal.nealProjection(self.bwInputCells, 
                            self.nineOffCells, inhConnectorList,'inhibitory')

    #-------------------setup retina to V1 connections---
    def makeRetToV1Synapses(self):
        if self.hEdgeTop:
            excConnectorList = getHEdgeTopOrBottomFrom3x3OffExcite()
            self.neal.nealProjection(self.threeOffCells,self.hEdgeTopCells, 
                                excConnectorList,'excitatory')
            inhConnectorList = getHEdgeTopFrom6x6OffInhib()
            self.neal.nealProjection(self.sixOffCells, self.hEdgeTopCells, 
                                inhConnectorList,'inhibitory')
            inhConnectorList = getHEdgeTopFrom9x9OffInhib()
            self.neal.nealProjection(self.nineOffCells, self.hEdgeTopCells, 
                                inhConnectorList,'inhibitory')

        if self.hEdgeBottom:
            excConnectorList = getHEdgeTopOrBottomFrom3x3OffExcite()
            self.neal.nealProjection(self.threeOffCells,
                        self.hEdgeBottomCells, excConnectorList,'excitatory')
            inhConnectorList = getHEdgeBottomFrom6x6OffInhib()
            self.neal.nealProjection(self.sixOffCells, self.hEdgeBottomCells, 
                                inhConnectorList,'inhibitory')
            inhConnectorList = getHEdgeBottomFrom9x9OffInhib()
            self.neal.nealProjection(self.nineOffCells, self.hEdgeBottomCells, 
                                inhConnectorList,'inhibitory')

        if self.vEdgeLeft:
            excConnectorList = getVEdgeLeftOrRightFrom3x3OffExcite()
            self.neal.nealProjection(self.threeOffCells,self.vEdgeLeftCells, 
                                excConnectorList,'excitatory')
            inhConnectorList = getVEdgeLeftFrom9x9OffInhib()
            self.neal.nealProjection(self.nineOffCells, self.vEdgeLeftCells, 
                                inhConnectorList,'inhibitory')

        if self.vEdgeRight:
            excConnectorList = getVEdgeLeftOrRightFrom3x3OffExcite()
            self.neal.nealProjection(self.threeOffCells,self.vEdgeRightCells, 
                                excConnectorList,'excitatory')
            inhConnectorList = getVEdgeRightFrom9x9OffInhib()
            self.neal.nealProjection(self.nineOffCells, self.vEdgeRightCells, 
                                inhConnectorList,'inhibitory')

        if self.sEdgeTop:
            excConnectorList = getSEdgeTopOrBottomFrom3x3OffExcite()
            self.neal.nealProjection(self.threeOffCells,self.sEdgeTopCells, 
                                excConnectorList,'excitatory')

            inhConnectorListTB = getSEdgeTopOrBottomFrom6x6OffInhib()
            inhConnectorListT = getSEdgeTopFrom6x6OffInhib()
            inhConnectorList = inhConnectorListT+inhConnectorListTB
            self.neal.nealProjection(self.sixOffCells, self.sEdgeTopCells, 
                                inhConnectorList,'inhibitory')

            inhConnectorListTB = getSEdgeTopOrBottomFrom9x9OffInhib()
            inhConnectorListT = getSEdgeTopFrom9x9OffInhib()
            inhConnectorList = inhConnectorListT+inhConnectorListTB
            self.neal.nealProjection(self.nineOffCells, self.sEdgeTopCells, 
                                inhConnectorList,'inhibitory')

        if self.sEdgeBottom:
            #if (nealParameters.simulator == 'spinnaker'):
            #    excConnectorList = getSEdgeTopOrBottomFrom3x3OnExcite()
            #    self.neal.nealProjection(self.threeOnCells,
            #        self.sEdgeBottomCells, excConnectorList,'excitatory')
            #else:
            excConnectorList = getSEdgeTopOrBottomFrom3x3OffExcite()
            self.neal.nealProjection(self.threeOffCells,
                    self.sEdgeBottomCells, excConnectorList,'excitatory')

            inhConnectorListTB = getSEdgeTopOrBottomFrom6x6OffInhib()
            inhConnectorListB = getSEdgeBottomFrom6x6OffInhib()
            inhConnectorList = inhConnectorListB+inhConnectorListTB
            self.neal.nealProjection(self.sixOffCells, self.sEdgeBottomCells, 
                                inhConnectorList,'inhibitory')

            inhConnectorListTB = getSEdgeTopOrBottomFrom9x9OffInhib()
            inhConnectorListB = getSEdgeBottomFrom9x9OffInhib()
            inhConnectorList = inhConnectorListB+inhConnectorListTB
            self.neal.nealProjection(self.nineOffCells, self.sEdgeBottomCells, 
                        inhConnectorList,'inhibitory')

        if self.bEdgeTop:
            excConnectorList = getBEdgeTopOrBottomFrom3x3OffExcite()
            self.neal.nealProjection(self.threeOffCells,self.bEdgeTopCells, 
                                excConnectorList,'excitatory')

            inhConnectorList6 = getBEdgeTopFrom6x6OffInhib()
            self.neal.nealProjection(self.sixOffCells, self.bEdgeTopCells, 
                                inhConnectorList6,'inhibitory')

            inhConnectorListBB6On = getBEdgeTopOrBottomFrom6x6OnInhib()
            self.neal.nealProjection(self.sixOnCells, self.bEdgeBottomCells, 
                                inhConnectorListBB6On,'inhibitory')

            inhConnectorList9 = getBEdgeTopFrom9x9OffInhib()
            #self.neal.nealProjection(self.nineOffCells, self.bEdgeTopCells, 
            #                    inhConnectorList9,'inhibitory')

        if self.bEdgeBottom:
            excConnectorList = getBEdgeTopOrBottomFrom3x3OffExcite()
            self.neal.nealProjection(self.threeOffCells,
                        self.bEdgeBottomCells, excConnectorList,'excitatory')

            inhConnectorListBB6 = getBEdgeBottomFrom6x6OffInhib()
            self.neal.nealProjection(self.sixOffCells, self.bEdgeBottomCells, 
                                inhConnectorListBB6,'inhibitory')

            inhConnectorListBB6On = getBEdgeTopOrBottomFrom6x6OnInhib()
            self.neal.nealProjection(self.sixOnCells, self.bEdgeBottomCells, 
                                inhConnectorListBB6On,'inhibitory')

            inhConnectorList9 = getBEdgeBottomFrom9x9OffInhib()
            #self.neal.nealProjection(self.nineOffCells, self.bEdgeBottomCells, 
            #                    inhConnectorList9,'inhibitory')


        #undone may 2016  where are the angles?


    #-------------------setup Object Recognition and V1 to ObjRec connections---
    def connectColourToObject (self,colourOffset,leftStart,rightEnd,factNum,
                               synWeight):
        connector = []
        for cRow in range (0,20):
            for cCol in range (int(20*leftStart),
                               int(20*rightEnd)):
                for toOffset in range (0,self.fsa.CA_SIZE):
                    fromNeuron = cRow*20+ cCol+colourOffset
                    toNeuron = toOffset + (factNum*self.fsa.CA_SIZE)
                    connector=connector+[(fromNeuron,toNeuron,synWeight,
                                          nealParameters.DELAY)]
        if (synWeight < 0):
            print "connectColourToObject caleld with negative weight"
        else:
            self.neal.nealProjection(self.colourInputCells,self.objRecCells,
                            connector,'excitatory')

    def connectColourToTopObject (self,colourOffset,leftStart,rightEnd,factNum,
                               synWeight):
        connector = []
        for cRow in range (0,10):
            for cCol in range (int(20*leftStart),
                               int(20*rightEnd)):
                for toOffset in range (0,self.fsa.CA_SIZE):
                    fromNeuron = cRow*20+ cCol+colourOffset
                    toNeuron = toOffset + (factNum*self.fsa.CA_SIZE)
                    connector=connector+[(fromNeuron,toNeuron,synWeight,
                                          nealParameters.DELAY)]
        if (synWeight < 0):
            print "connectColourToObject caleld with negative weight"
        else:
            self.neal.nealProjection(self.colourInputCells,self.objRecCells,
                            connector,'excitatory')

    def colourInhibitsObject (self,colourOffset,leftStart,rightEnd,factNum,
                               synWeight):
        connector = []
        for cRow in range (0,20):
            for cCol in range (int(20*leftStart),
                               int(20*rightEnd)):
                for toOffset in range (0,self.fsa.CA_SIZE):
                    fromNeuron = cRow*20+ cCol+colourOffset
                    toNeuron = toOffset + (factNum*self.fsa.CA_SIZE)
                    connector=connector+[(fromNeuron,toNeuron,synWeight,
                                          nealParameters.DELAY)]
        self.neal.nealProjection(self.colourInputCells,self.objRecCells,
                            connector,'inhibitory')

    def connectColourToInFrontDoor(self):
        if (nealParameters.simulator=="spinnaker"):
            clrToInDoorOutWt =  0.001 
        else: 
            clrToInDoorOutWt = 0.001

        clrToInDoorInWt = 0.0001
        self.connectColourToTopObject(400,0.0,0.2,self.inFrontDoorObj,
                                      clrToInDoorOutWt)
        self.connectColourToTopObject(400,0.2,0.8,self.inFrontDoorObj,
                                      clrToInDoorInWt)
        self.connectColourToTopObject(400,0.8,1.0,self.inFrontDoorObj,
                                      clrToInDoorOutWt)

    #The left gets a bit on the left and most of the right, with inhibition
    #from the ctr left.  The right is the opposite
    #The ctr has inhibition from the ctr.
    def connectColourToInDoor(self):
        if (nealParameters.simulator=="spinnaker"):
            clrToInDoorWt =  0.001 
            inhibWt = 0.003  
        else: 
            clrToInDoorWt = 0.0005 # .001 #.0025 when top
            inhibWt = -0.001 # when top-0.03

        self.connectColourToTopObject(400,0.0,0.2,self.inDoorLeftObj,clrToInDoorWt)#green inDoorLt
        self.connectColourToTopObject(400,0.5,1.0,self.inDoorLeftObj,clrToInDoorWt)#green inDoorLt
        self.connectColourToTopObject(400,0.0,0.35,self.inDoorCtrObj,clrToInDoorWt)#green inDoorCtr
        self.connectColourToTopObject(400,0.65,1.0,self.inDoorCtrObj,clrToInDoorWt)#green inDoorCtr
        self.connectColourToTopObject(400,0.0,0.5,self.inDoorRightObj,clrToInDoorWt)#green inDoorRt
        self.connectColourToTopObject(400,0.8,1.0,self.inDoorRightObj,clrToInDoorWt)#green inDoorRt

        #Use the missing colour at the back of the corridor to specify
        #where in the door you are.
        self.colourInhibitsObject(400,0.1,0.3,self.inDoorLeftObj,inhibWt)#green inDoorLt
        self.colourInhibitsObject(400,0.4,0.6,self.inDoorCtrObj,inhibWt)#green inDoorCtr
        self.colourInhibitsObject(400,0.7,0.9,self.inDoorRightObj,inhibWt)#green inDoorRt 

    #red and blue turn on the three object facts.
    def connectColourToObjRec(self):
        self.connectColourToInFrontDoor()
        self.connectColourToInDoor()
        if (nealParameters.simulator=="spinnaker"):
            #The tradeoff here is between recognising the object too
            #soon (start corridor), but not too late (end coor)
            clrToObjWt = 0.009 #jul18 .008
            greenToDoorWt = 0.002 #jul03 .003
        else: #nest
            clrToObjWt = 0.009 
            greenToDoorWt = 0.0015 #sep6 .003

        clrToPyrWt =  0.003 #.001jul4

        #blue and red to  leftObj ctrObj and rightObjFact 
        self.connectColourToObject(800,0,0.3,self.objLeft,clrToObjWt) 
        self.connectColourToObject(800,0.3,0.7,self.objCtr,clrToObjWt) 
        self.connectColourToObject(800,0.7,1.0,self.objRight,clrToObjWt) 
        self.connectColourToObject(0,0,0.3,self.objLeft,clrToObjWt) 
        self.connectColourToObject(0,0.3,0.7,self.objCtr,clrToObjWt) 
        self.connectColourToObject(0,0.7,1.0,self.objRight,clrToObjWt) 

        #green to doors
        self.connectColourToObject(400,0,0.3,self.doorLeft,greenToDoorWt) 
        self.connectColourToObject(400,0.3,0.7,self.doorCtr,greenToDoorWt) 
        self.connectColourToObject(400,0.7,1.0,self.doorRight,greenToDoorWt) 

        #colours (0 red, 800 blue) to colourShapeLocs
        self.connectColourToObject(0,0,0.3,self.redPyrLeftObj,clrToPyrWt) 
        self.connectColourToObject(0,0.3,0.7,self.redPyrCtrObj,clrToPyrWt) 
        self.connectColourToObject(0,0.7,1.0,8,clrToPyrWt) #red redPyrRight
        self.connectColourToObject(800,0,0.3,9,clrToPyrWt) #blue bluePyrLeft
        self.connectColourToObject(800,0.3,0.7,10,clrToPyrWt) #blue bluePyrCtr
        self.connectColourToObject(800,0.7,1.0,11,clrToPyrWt) #blue bluePyrRight
        self.connectColourToObject(0,0,0.3,12,clrToPyrWt) #red  redStalLeft
        self.connectColourToObject(0,0.3,0.7,13,clrToPyrWt) #red redStalCtr
        self.connectColourToObject(0,0.7,1.0,14,clrToPyrWt) #red redStalRight
        self.connectColourToObject(800,0,0.3,15,clrToPyrWt) #blue blueStalLeft
        self.connectColourToObject(800,0.3,0.7,16,clrToPyrWt) #blue blueStalCtr
        self.connectColourToObject(800,0.7,1.0,17,clrToPyrWt) #blue blueStalRght

    def connectV1FeatureToObject (self, synWeight,leftStart,rightEnd,factNum):
        connector = []
        for cRow in range (0,nealParameters.INPSIZE):
            for cCol in range (int(nealParameters.INPSIZE*leftStart),
                               int(nealParameters.INPSIZE*rightEnd)):
                for toOffset in range (0,self.fsa.CA_SIZE):
                    fromNeuron = cRow*nealParameters.INPSIZE+ cCol
                    toNeuron = toOffset + (factNum*self.fsa.CA_SIZE)
                    connector=connector+[(fromNeuron,toNeuron,synWeight,
                                          nealParameters.DELAY)]
        return connector

    #set the particular v1 features to their position approriate objects
    #eg a pyramid on the left has a bEdgeTop in the left half of the visField
    def connectV1ToPyrStalObjRec(self):
        featWt = 0.003 #0.005 jul4
        #redPyrLeft
        connectors = self.connectV1FeatureToObject(featWt,0,0.3,6)
        self.neal.nealProjection(self.hEdgeBottomCells,self.objRecCells, 
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        
        #redPyrCtr
        connectors = self.connectV1FeatureToObject(featWt,0.3,0.7,
                                                   self.redPyrCtrObj)
        self.neal.nealProjection(self.hEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')

        #redPyrRight
        connectors = self.connectV1FeatureToObject(featWt,0.7,1.0,8)
        self.neal.nealProjection(self.hEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')

        #bluePyrLeft
        connectors = self.connectV1FeatureToObject(featWt,0,0.3,9)
        self.neal.nealProjection(self.hEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        
        #bluePyrCtr
        connectors = self.connectV1FeatureToObject(featWt,0.3,0.7,
                                                   self.bluePyrCtrObj)
        self.neal.nealProjection(self.hEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')


        #bluePyrRight
        connectors = self.connectV1FeatureToObject(featWt,0.7,1.0,11)
        self.neal.nealProjection(self.hEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')

        #redStalLeft
        connectors = self.connectV1FeatureToObject(featWt,0,0.3,12)
        self.neal.nealProjection(self.hEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')

        #redStalCtr
        connectors = self.connectV1FeatureToObject(featWt,0.3,0.7,
                                                   self.redStalCtrObj)
        self.neal.nealProjection(self.hEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')

        #redStalRight 
        connectors = self.connectV1FeatureToObject(featWt,0.7,1.0,14)
        self.neal.nealProjection(self.hEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')

        #blueStalLeft
        connectors = self.connectV1FeatureToObject(featWt,0,0.3,15)
        self.neal.nealProjection(self.hEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')

        #blueStalCtr
        connectors = self.connectV1FeatureToObject(featWt,0.3,0.7,
                                                   self.blueStalCtrObj)
        self.neal.nealProjection(self.hEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')

        #blueStalRight
        connectors = self.connectV1FeatureToObject(featWt,0.7,1.0,
                                                   self.blueStalRightObj)
        self.neal.nealProjection(self.hEdgeTopCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.bEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')
        self.neal.nealProjection(self.sEdgeBottomCells,self.objRecCells,
                            connectors,'excitatory')


    def objPreventsObj(self,fromObj,toObj):
        if (nealParameters.simulator=="spinnaker"):
            preventWeight = 0.2
        else:
            preventWeight = -0.2
        connector = []
        for fromOffset in range (0,self.fsa.CA_SIZE):
            fromNeuron = fromOffset + (fromObj*self.fsa.CA_SIZE)
            for toOffset in range (0,self.fsa.CA_SIZE):
                toNeuron = toOffset + (toObj*self.fsa.CA_SIZE)
                connector = connector + [(fromNeuron,toNeuron,preventWeight,
                                      nealParameters.DELAY)]
        self.neal.nealProjection(self.objRecCells,self.objRecCells,connector,
                                 'inhibitory')

    def connectObjRec(self):
        #objects prevent Other Objects
        self.objPreventsObj(0,1) #objLeft prevents objCentre
        self.objPreventsObj(0,2) #objLeft prevents objRight
        self.objPreventsObj(1,0) #objCentre prevents objLeft
        self.objPreventsObj(1,1) #objCentre prevents objRight
        self.objPreventsObj(2,0) #objRight prevents objLeft
        self.objPreventsObj(2,1) #objRight prevents objCentre

        #doors prevent other doors
        self.objPreventsObj(self.doorLeft,self.doorCtr) 
        self.objPreventsObj(3,5) #doorLeft prevents doorRight
        self.objPreventsObj(4,3) #doorCentre prevents doorLeft
        self.objPreventsObj(4,5) #doorCentre prevents doorRight
        self.objPreventsObj(5,3) #doorRight prevents doorLeft
        self.objPreventsObj(5,4) #doorRight prevents doorCentre


    def connectSpinnakerInputToSpinnakerRecord(self):
        if (nealParameters.simulator != 'spinnaker'):
            return
        if (not nealParameters.useSpikeServer):
            return
        connector = []
        synWt = 0.1
        for neuron in range (0,nealParameters.neuronsPerVisionSubnet):
            connector=connector+[(neuron,neuron,synWt,nealParameters.DELAY)]
        self.neal.nealProjection(self.bwInputCells,self.bwInputRecordCells,
                            connector,'excitatory')

        connector = []
        for neuron in range (0,1200):
            connector=connector+[(neuron,neuron,synWt,nealParameters.DELAY)]
        self.neal.nealProjection(self.colourInputCells,
            self.colourInputRecordCells, connector,'excitatory')

    def makeVisionSynapses(self):
        self.makeInputToRetSynapses()
        self.makeRetToV1Synapses() 
        self.connectColourToObjRec()
        self.connectObjRec() 
        self.connectV1ToPyrStalObjRec()
        self.connectSpinnakerInputToSpinnakerRecord()

    #---input functions
    def setInput(self,imagePixels,inputSource):
        weight = 20.0
        connectors = []
        cConnectors = []

        for onPixel in imagePixels:
            connectors = connectors + [(0,onPixel,weight,DELAY)]

        #print connectors
        excConnector = FromListConnector(connectors)
        self.nealProjection(inputSource,self.inputCells,excConnector,'excitatory')
        return connectors

    #---- recording functions
    def setVisionRecording(self):
            if (nealParameters.simulator == 'spinnaker'):
                if (nealParameters.useSpikeServer):
                    self.bwInputRecordCells.record(),
                    self.colourInputRecordCells.record(),
                else:
                    self.bwInputCells.record()
                    self.colourInputCells.record()
                self.threeOnCells.record()
                self.threeOffCells.record()
                self.sixOnCells.record()
                self.sixOffCells.record()
                self.nineOnCells.record()
                self.nineOffCells.record()
                self.hEdgeTopCells.record()
                self.hEdgeBottomCells.record()
                self.vEdgeLeftCells.record()
                self.vEdgeRightCells.record()
                self.sEdgeTopCells.record()
                self.sEdgeBottomCells.record()
                self.bEdgeTopCells.record()
                self.bEdgeBottomCells.record()
                self.objRecCells.record()
            elif (nealParameters.simulator == 'nest'):
                self.bwInputCells.record({'spikes'})
                self.colourInputCells.record({'spikes'})
                self.threeOnCells.record({'spikes'})
                self.threeOffCells.record({'spikes'})
                self.sixOnCells.record({'spikes'})
                self.sixOffCells.record({'spikes'})
                self.nineOnCells.record({'spikes'})
                self.nineOffCells.record({'spikes'})
                self.hEdgeTopCells.record({'spikes'})
                self.hEdgeBottomCells.record({'spikes'})
                self.vEdgeLeftCells.record({'spikes'})
                self.vEdgeRightCells.record({'spikes'})
                self.sEdgeTopCells.record({'spikes'})
                self.sEdgeBottomCells.record({'spikes'})
                self.bEdgeTopCells.record({'spikes'})
                self.bEdgeBottomCells.record({'spikes'})
                self.objRecCells.record({'spikes'})


    #------print functions
    #With pynest we have to use a spike detector to print
    #It records for a whole population.
    #Pass in the detector, print header, the first neuron in the population
    def printPynestSpikes(self,spikeDet,outputHeader,popStartNeuron):
        spikes = nest.GetStatus(spikeDet)
        testTimes = spikes[0]['events']['times']
        testSenders = spikes[0]['events']['senders']
        print outputHeader
        for spikeNumber in range (0,len(testTimes)):
            print testTimes[spikeNumber], testSenders[spikeNumber]-popStartNeuron

    def printPklSpikes(self,fileName):
        fileHandle = open(fileName)
        neoObj = pickle.load(fileHandle)
        segments = neoObj.segments
        segment = segments[0]
        spikeTrains = segment.spiketrains
        neurons = len(spikeTrains)
        for neuronNum in range (0,neurons):
            if (len(spikeTrains[neuronNum])>0):
                spikes = spikeTrains[neuronNum]
                for spike in range (0,len(spikes)):
                    print neuronNum, spikes[spike]
        fileHandle.close()

    def printVisionNets(self):
        if (nealParameters.simulator == 'spinnaker'):
            suffix = '.sp'
            if (nealParameters.useSpikeServer):
                self.bwInputRecordCells.printSpikes('results/visionBWInput.sp')
                self.colourInputRecordCells.printSpikes('results/visionColourInput.sp')
        elif (nealParameters.simulator == 'nest'):
            suffix = '.pkl'
            self.bwInputCells.printSpikes('results/visionBWInput'+suffix)
            self.colourInputCells.printSpikes('results/visionColourInput'+suffix)

        if (not nealParameters.useSpikeServer):
            self.bwInputCells.printSpikes('results/visionBWInput'+suffix)
            self.colourInputCells.printSpikes('results/visionColourInput'
                                                  +suffix)
        self.objRecCells.printSpikes('results/visionObjRec'+suffix)
        self.threeOnCells.printSpikes('results/visionThreeOn'+suffix)
        self.threeOffCells.printSpikes('results/visionThreeOff'+suffix)
        self.sixOnCells.printSpikes('results/visionSixOn'+suffix)
        self.sixOffCells.printSpikes('results/visionSixOff'+suffix)
        self.nineOnCells.printSpikes('results/visionNineOn'+suffix)
        self.nineOffCells.printSpikes('results/visionNineOff'+suffix)
        self.hEdgeTopCells.printSpikes('results/visionHEdgeTop'+suffix)
        self.hEdgeBottomCells.printSpikes('results/visionHEdgeBottom'+suffix)
        self.vEdgeLeftCells.printSpikes('results/visionVEdgeLeft'+suffix)
        self.vEdgeRightCells.printSpikes('results/visionVEdgeRight'+suffix)
        self.sEdgeTopCells.printSpikes('results/visionSEdgeTop'+suffix)
        self.sEdgeBottomCells.printSpikes('results/visionSEdgeBottom'+suffix)
        self.bEdgeTopCells.printSpikes('results/visionBEdgeTop'+suffix)
        self.bEdgeBottomCells.printSpikes('results/visionBEdgeBottom'+suffix)

    #-cover functions to return internal values
    def getBWVisionInputCells(self):
        return self.bwInputCells

    def getColourVisionInputCells(self):
        return self.colourInputCells

    def getObjRecCells(self):
        return self.objRecCells
