#This assumes that you are using spinnaker,
#there is an environment running 
# and use spikeserver is set to True
#in results/visionObjRec.sp 
#example call is
#python visionTest.py rtRedStal1.jpg  
#the extra argument is a reminant
"""
The test for vision.  
"""

import sys

#imports for PyNN
import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np
from getPixelsClass import imageToPixelClass

from nealCoverClass import NealCoverFunctions

#Other NEAL files and functions

from visionClass import visionArea

#------set up object to fact connections----------
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)


def createInputs():
    #inputSpikeTimes = [10.0,40.0,70.0,100.0,200.0,230.0,260.0,290.0]
    inputSpikeTimes = [10.0,40.0,70.0,100.0,200.0,230.0,260.0,290.0,320.0,350.0,380.0,500.0,530.0,560.0,590.0]
    #inputSpikeTimes = [10.0]
    
    spikeArray = {'spike_times': [inputSpikeTimes]}
    spikeGen=sim.Population(1,sim.SpikeSourceArray,spikeArray,
                                   label='inputSpikes_0')

    return [spikeGen]

def connectBWInputs(spikeGen,pixels):
    toInputWt = 0.1

    #this is for no environment
    bwInputNeurons = vision.getBWVisionInputCells()
    connector = []
    for toNeuron in pixels:
        connector = connector + [(0,toNeuron,toInputWt,
                                  nealParameters.DELAY)]
    neal.nealProjection(spikeGen,bwInputNeurons,connector,'excitatory')
    
def connectColourInputs(spikeGen,pixels):
    toInputWt = 0.1

    #this is for no environment
    colourInputNeurons = vision.getColourVisionInputCells()
    connector = []
    for toNeuron in pixels:
        connector = connector + [(0,toNeuron,toInputWt,
                                  nealParameters.DELAY)]
    neal.nealProjection(spikeGen,colourInputNeurons,connector,'excitatory')
    

def runTest(time):
    if (nealParameters.simulator=="spinnaker"):
        sim.run(nealParameters.SIM_LENGTH)
    elif (nealParameters.simulator=="nest"):
        callbackTimeStepLength = 30
        steps = int(nealParameters.SIM_LENGTH/callbackTimeStepLength)
        for step in range (0, steps):
            currentTime = step*callbackTimeStepLength
            print "nest simulation timestep ", currentTime
            vision.setNestVisionInput(currentTime)
            sim.run(callbackTimeStepLength)

#------------Main Body---------------
args = sys.argv
numberArgs = args.__len__()
dirName = "data/"
if (numberArgs == 2):
    fileName = args[1]
else:
    fileName = "leftBluePyr1.jpg"
fileName = dirName + fileName

neal = NealCoverFunctions(nealParameters.simulator)
init()

vision = visionArea(nealParameters.simulator)
vision.allocateVisionNeurons()
vision.makeVisionSynapses()

#-------------------setup recording
#inpPop.record(['spikes','v'])
vision.setVisionRecording()

if (nealParameters.simulator=="spinnaker"):
    print "need to plug in spinnVisionInputClass"
elif (nealParameters.simulator=="nest"):
    vision.setNestToUseStaticFile(fileName)
runTest(nealParameters.SIM_LENGTH)

#--------------print results

vision.printVisionNets()


