#This takes a file name, a picture from the data directory. 
#In spinnaker it should have useSpikeServer set to False
#in results/visionObjRec.sp 
#example call is
#python visionTest.py rtRedStal1.jpg

import sys

"""
The test for vision.  
"""

#imports for PyNN
import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle

import numpy as np
from getPixelsClass import imageToPixelClass

from nealCoverClass import NealCoverFunctions

#Other NEAL files and functions

from visionClass import visionArea

#------set up object to fact connections----------
def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)


def createInputs():
    #inputSpikeTimes = [10.0,40.0,70.0,100.0,200.0,230.0,260.0,290.0]
    #inputSpikeTimes = [10.0,40.0,70.0,100.0,200.0,230.0,260.0,290.0,320.0,350.0,380.0,500.0,530.0,560.0,590.0]
    inputSpikeTimes = [26.0,47.0,72.0,93,118.0,154.0,175.0,201.0,226.0,278.0,299.0,324.0,350.0,376.0,412.0,462.0,484.0,511.0,537.0,555.0,583.0]
    #inputSpikeTimes = [10.0]
    
    spikeArray = {'spike_times': [inputSpikeTimes]}
    spikeGen=sim.Population(1,sim.SpikeSourceArray,spikeArray,
                                   label='inputSpikes_0')

    return [spikeGen]

def connectBWInputs(spikeGen,pixels):
    toInputWt = 0.1 

    #this is for no environment
    bwInputNeurons = vision.getBWVisionInputCells()
    connector = []
    for toNeuron in pixels:
        connector = connector + [(0,toNeuron,toInputWt,
                                  nealParameters.DELAY)]
    neal.nealProjection(spikeGen,bwInputNeurons,connector,'excitatory')
    
def connectColourInputs(spikeGen,pixels):
    toInputWt = 0.1

    #this is for no environment
    colourInputNeurons = vision.getColourVisionInputCells()
    connector = []
    for toNeuron in pixels:
        connector = connector + [(0,toNeuron,toInputWt,
                                  nealParameters.DELAY)]
    neal.nealProjection(spikeGen,colourInputNeurons,connector,'excitatory')
    

def runTest(time):
    sim.run(time)

#------------Main Body---------------
args = sys.argv
numberArgs = args.__len__()
dirName = "data/"
if (numberArgs == 2):
    fileName = args[1]
else:
    fileName = "leftBluePyr1.jpg"
fileName = dirName + fileName

neal = NealCoverFunctions(nealParameters.simulator)
init()

vision = visionArea(nealParameters.simulator)
vision.allocateVisionNeurons()
vision.makeVisionSynapses()

#-------------------setup recording
#inpPop.record(['spikes','v'])
vision.setVisionRecording()

#open image file
imageTranslator = imageToPixelClass(nealParameters.INPSIZE)
pixels = imageTranslator.getPixelsFromFile(fileName) 
#pixels = imageTranslator.getPixelsFromFile("data/leftBluePyramid1.jpg") #rtObject
#pixels = imageTranslator.getPixelsFromFile("data/rtBluePyramid3.jpg") #rtObject

#pixels = imageTranslator.getPixelsFromFile("data/temp1.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/temp39.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/pyr1.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/doorLeft1.jpg") #doorLt
#pixels = imageTranslator.getPixelsFromFile("data/doorCtr.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/doorCtr1.jpg") #doorLt doorCtr
#pixels = imageTranslator.getPixelsFromFile("data/doorRt.jpg")
#pixels = imageTranslator.getPixelsFromFile("data/rtBluePyramid1.jpg") #rtObject
#pixels = imageTranslator.getPixelsFromFile("data/rtBluePyramid2.jpg") #ctrObject rtObject
#pixels = imageTranslator.getPixelsFromFile("data/ctrBluePyramid.jpg") #ctrObject
#pixels = imageTranslator.getPixelsFromFile("data/ctrBluePyramid1.jpg") #ctrObject
#pixels = imageTranslator.getPixelsFromFile("data/stal2.jpg") #objCtr doorRt
#pixels = imageTranslator.getPixelsFromFile("data/ctrRedStal1.jpg") #ctrObject

#pixels = imageTranslator.getPixelsFromFile("data/rtBluePyramid3.jpg") #rtObject
#pixels = imageTranslator.getPixelsFromFile("data/inDoor1.jpg") 

bwPixels = pixels[0]
colourPixels = pixels[1]

spikeGen = createInputs()
connectBWInputs(spikeGen[0],bwPixels)
connectColourInputs(spikeGen[0],colourPixels)
#testFact()

runTest(nealParameters.SIM_LENGTH)

#--------------print results

vision.printVisionNets()
print 'Object Recognition Spikes'
#vision.printPklSpikes("results/visionObjRec.pkl")


